/*
* 
* ProFiDo - The Processes Fitting Toolkit Dortmund
* Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License along
* with this program; if not, write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*
*/

package core;

import java.io.Serializable;
import java.util.Vector;

import core.graph.Edge;

/**
 * This class represents the properties of file parameter groups.
 * A file parameter group is special type used for the input and output 
 * parameters. These attributes are not displayed in the properties window,
 * since the filenames are represented by the incoming and outgoing edges.  
 */

public class FileNameParameterGroup extends ParameterGroup implements Serializable {
	private static final long serialVersionUID = 1L;
	private static String CONVERTED_SUFFIX = ".conv";
	private static String UNCONVERTED_SUFFIX = ".org";

	/** 
	 * Indicates if this parameter accepts multiple filename or just one
	 * as specified in the configuration xml-file.
	 */
	private boolean isMulti;
	
	/**
	 * The edges representing the filename and therefore the value 
	 * of this parameter.
	 */
	private Vector<Edge> edges;
	
	/**
	 * Binary of the converter used to convert the file associated to this
	 * parameter group. (Only used when this parameter group represents
	 * an output parameter)
	 */
	private String outputConverterBinary;

	/**
	 * Binary of the converter used to convert the file associated to this
	 * parameter group. (Only used when this parameter group represents
	 * an input parameter)
	 */
	private String inputConverterBinary;
		
	/**
	 * Creates the parameter group according to the passed values. 
	 */
	public FileNameParameterGroup(String name, String key, String description, String successor, boolean visible, boolean isMulti) {
		super(name, key, description, successor, visible, true);
		this.isMulti = isMulti;
		this.edges = null;
		inputConverterBinary = "";
		outputConverterBinary = "";
		
		this.edges = null;		
	}

	@Override
	public String getDescription() {
		return this.description + "\n";
	}	
	
	/**
	 * Generates and returns the part of the command line call specified by
	 * this parameter group, taking into account the passed path which should
	 * indicate where the (intermediate) results of the workflow are stored.
	 */
	public String getCommandlineCallString(String resultPath)  {
		String result = "";

		if ((this.edges != null) && (this.edges.size() > 0)) {
			String curSuffix = "";
			if (!inputConverterBinary.equals("")) 
				curSuffix = CONVERTED_SUFFIX;
			else if (!outputConverterBinary.equals(""))
				curSuffix = UNCONVERTED_SUFFIX;

			if (isMulti) {
				for (Edge curEdge : this.edges) {
					result = result + " " + key + "\"" + resultPath + curEdge.getFileName().getName() + curSuffix + "\"";	
				}
			} else {
				result = key + "\"" + resultPath + edges.get(0).getFileName().getName() + curSuffix + "\" ";	
			}
		}
		
		return result.trim();
	}
		
	
	/**
	 * This method creates the converter calls for the files associated to this
	 * parameter group.
	 * 
	 * @param softwarePath The path where the binaries are located.
	 * @param resultPath The path where the results should be stored.
	 */
	public String createConverterCalls(String softwarePath, String resultPath) {
		String result = "";

		if (edges != null) {
			// Input converter present
			if (!inputConverterBinary.equals("")) {
				for (Edge curEdge : edges) {
					result = result + "\n" + "\"" + softwarePath + inputConverterBinary+"\"" + " \"" + resultPath + curEdge.getFileName().getName() + "\" \"" + resultPath + curEdge.getFileName().getName() + CONVERTED_SUFFIX + "\"";
				}
			} else if (!outputConverterBinary.equals("")) {
				for (Edge curEdge : edges) {
					result = result + "\n" + "\"" + softwarePath + outputConverterBinary+ "\"" + " \"" + resultPath + curEdge.getFileName().getName() + UNCONVERTED_SUFFIX + "\" \"" + resultPath + curEdge.getFileName().getName() + "\"";
				}
			}
		}		
		return result.trim();
	}


	public void setOutputConverterBinary(String binary) {
		this.outputConverterBinary = binary;
	}

	public void setInputConverterBinary(String binary) {
		this.inputConverterBinary = binary;		
	}

	@Override
	public String getCommandlineCallString() {
		return this.getCommandlineCallString("");
	}

	/**
	 * Adds an edge to the edges representing this filename.
	 */
	public void addEdge(Edge newEdge) {
		if (edges == null)
			edges = new Vector<Edge>();
		edges.add(newEdge);


	}
	
	public void setEdges(Vector<Edge> edges) {
		this.edges = edges;	
	}

	/** 
	 * Determines if this filename parameter group has edges assosiated to it.
	 */
	public boolean hasEdges() {
		return ((edges != null) && (edges.size() > 0));
	}

	/**
	 * Remove all edges.
	 */
	public void clearEdges() {
		edges = null;
		
	}

	@Override
	public String getDisplayFriendlyValueString() {
		return "";
	}

}
