/*
* 
* ProFiDo - The Processes Fitting Toolkit Dortmund
* Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License along
* with this program; if not, write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*
*/

package core;

import java.io.Serializable;

/**
 * This class defines the properties of parameter groups. Each command line 
 * parameter of a job is represented by a parameter group, consisting of one 
 * keyword and zero to several values dedicated to this key. 
 * 
 * The complete command line call of a job is represented by the binary name 
 * and a list of parameters which consist of several parameter groups.
 * A parameter group might have the attributes visible, disabled and type, 
 * specifying the visibility within the canvas and weather the parameter is
 * employed when creating the command line call or not. 
 * Possible types might be filename, bool, text or select. 
 * 
 * The possible parameters for each job type are specified in the xml-file by
 * the tags &ltname&gt, &ltdescription&gt, &ltkey&gt and &ltdefault&gt followed 
 * by one or several parameters each specified within the tag &ltparameter&gt.
 * &ltname&gt contains an identifier for the attribute. The content of 
 * &ltdescription&gt is used to display further information about the parameter
 * group. The tag &ltkey&gt is used for the command line call generation when 
 * exporting the workflow.
 * If a tool has several parameter groups the &ltsuccessor&gt tag will give 
 * the name of the succeeding parameter group within the command line call 
 * for the corresponding program.
 * 
 *
 */

public abstract class ParameterGroup implements Serializable {
	private static final long serialVersionUID = 1L;

	/**
	 * Name of the corresponding command line parameter as specified within the 
	 * configuration xml-file.
	 */
	protected String name;
	
	/** 
	 * Description of the corresponding command line parameter as specified 
	 * within the configuration xml-file.
	 */
	protected String description;

	/** 
	 * Name of this parameters succeeding command line parameter as specified 
	 * within the configuration xml-file.
	 */
	protected String successorName;
	
	/** 
	 * Standard visibility of this parameter group as specified in this 
	 * parameters description in the configuration xml-file.
	 */
	private boolean defaultVisibility;

	/** 
	 * Standard value for enabled of this parameter group as specified in this 
	 * parameters description in the configuration xml-file.
	 */
	private boolean defaultValueEnabled;
	
	/** 
	 * Key used to start the corresponding parameter when generating the 
	 * command line call as specified within the configuration xml-file.
	 */
	protected String key;
	
	/**
	 * Determine if the parameter group should be displayed on the canvas of the
	 * workflow.
	 */
	protected boolean visible;
	
	/**
	 * Determine if the parameter is employed when creating the workflow.
	 */
	protected boolean enabled;
		
	/** 
	 * Reference to the succeeding parameter group object in accordance to the
	 * specified name from the configuration xml-file.
	 */
	private ParameterGroup successor;
		
	/**
	 * Creates the parameter group according to the passed values. 
	 */
	public ParameterGroup(String name, String key, String description, String successor, boolean visible, boolean enabled) {
		super();
		this.name = name;
		this.key = key;
		this.description = description;
		this.successorName = successor;
		this.defaultVisibility = visible;
		this.visible = visible;
		this.defaultValueEnabled = enabled;
		this.enabled = enabled;
	}	
	
	/**
	 * Generates and returns the value part of this parameter groups command 
	 * line call. If necessary due to the type of the parameter group the values
	 * are returned in a easy readable way.
	 */ 
	public abstract String getDisplayFriendlyValueString();
	
	/**
	 * Generates and returns the part of the command line call specified by
	 * this parameter group.
	 */
	public abstract String getCommandlineCallString();
		
	/**
	 * Generates (if necessary) and returns the description of this parameter group.
	 */
	public abstract String getDescription();
	
	public boolean getDefaultVisibility() {
		return defaultVisibility;
	}

	public boolean getDefaultValueEnabled() {
		return defaultValueEnabled;
	}
	
	public String getName() {
		return name;
	}
	
	public void setName(String name) {
		this.name = name;
	}

	public boolean isVisible() {
		return visible;
	}

	public void setVisible(boolean visible) {
		this.visible = visible;
	}

	public void setEnable(boolean enable) {
		this.enabled = enable;
	}
	
	public boolean isEnabled() {
		return enabled;
	}

	public String getKey() {
		return key;
	}

	public String getSuccessorName() {
		return successorName;
	}

	public void setSuccessor(ParameterGroup successor) {
		this.successor = successor;	
	}

	public ParameterGroup getSuccessor() {
		return successor;
	}
}