/*
* 
* ProFiDo - The Processes Fitting Toolkit Dortmund
* Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License along
* with this program; if not, write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*
*/

package core;

import java.io.Serializable;

import core.history.HistoryObject;



/**
 * This class stores and represents one string value as used by the 
 * TextParameterGroup. Since tool calls depend on a specific sequence of values 
 * within each TextParameterGroup, the successor attribute specifies the name of 
 * the next text parameter that should be used when constructing the call for the 
 * corresponding program.
 * A parameter might be of type &ltstatic&gt or &ltdynamic&gt, which are 
 * represented by the corresponding subclasses.
 */

public abstract class TextParameter implements Serializable {
	private static final long serialVersionUID = 1L;
	
	/**
	 * Name of the corresponding part of the command line parameter as specified 
	 * within the configuration xml-file.
	 */
	protected String name;
	
	/**
	 * Description of the corresponding part of the command line parameter as specified 
	 * within the configuration xml-file.
	 */
	protected String description;
	
	/** 
	 * Name of the succeeding part of the command line parameter as specified 
	 * within the configuration xml-file.
	 */
	protected String successor;

	/**
	 * Creates the text parameter according to the passed values. 
	 */
	public TextParameter(String name, String description, String successor) {
		super();
		this.name = name;
		this.description = description;
		this.successor = successor;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public String getSuccessor() {
		return successor;
	}

	public void setSuccessor(String successor) {
		this.successor = successor;
	}

	/**
	 * Returns a string showing the format used for entering and displaying 
	 * this parameter.
	 */
	public abstract String getFormat();
	
	/**
	 * Returns this TextParameters current temporary value
	 */
	public abstract String getTemporaryValue();
	
	/**
	 * Returns this TextParameters current value
	 */
	public abstract String getValue();

	/**
	 * This method parses the first value in the passed string for a value.
	 * If a suitable value is found it is stored as this parameters temporary 
	 * value and the remaining input string is returned.   
	 */
	public abstract String parseInputToTemporaryValue(String text);
	
	/**
	 * Creates a HistoryObject which adopts the temporary value of this TextParameter
	 * as its final value. 
	 */
	public abstract HistoryObject createHistoryObjectToInheritTemporaryValues();
	
	
}