/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package core;

import java.io.Serializable;
import java.util.Vector;

import core.history.HistoryObject;

/**
 * This class represents the properties of text parameter groups.
 * A text parameter group consists of an arbitrary amount of text values 
 * dedicated to this groups key. Each of the values is stored and represented 
 * by an instance of the TextParameter class. They are managed in the parameters 
 * vector. 
 */

public class TextParameterGroup extends ParameterGroup implements Serializable {
	private static final long serialVersionUID = 1L;

	/**
	 * Stores the TextParameter objects representing and storing the single values of
	 * this parameter group.
	 */
	private Vector<TextParameter> parameters;

	/** 
	 * Default values of the entire parameter as text as specified in the 
	 * configuration xml-file.
	 */	
	private String defaultValue;

	/**
	 * Creates the parameter group according to the passed values. 
	 */
	public TextParameterGroup(String name, String key, String description, String successor, boolean visible, boolean enabled, Vector<TextParameter> parameters) {
		super(name, key, description, successor, visible, enabled);
		this.parameters = parameters;
		this.defaultValue = "";	
	}

	public String getDefaultValue() {
		return defaultValue;
	}

	/**
	 * Returns a string showing the format used for entering or displaying 
	 * this parameter.
	 */
	public String getFormat() {
		String result = "";

		// Iterate through all parameters 
		for (TextParameter curParameter : parameters) {
			result = result + " " + curParameter.getFormat();
		}
		result = result.trim();
		return result;
	}

	/**
	 * This method parses a string containing the entire values of this parameter 
	 * in order to store each single value as the temporary value of the 
	 * corresponding TextParameter object.   
	 */
	public void parseInputToTemporaryValues(String text) {
		text = text.trim();
		// Hand the string over to each TextParameter object
		for (TextParameter curParameter : parameters) {
			// During parsing each parameter object removes the
			// parsed part from the string
			text = curParameter.parseInputToTemporaryValue(text);
		}
	}

	@Override
	public String getDisplayFriendlyValueString() {
		// Interate through all parameter objects and append their value
		String result = "";
		for (TextParameter curParameter : parameters) {
			result = result + " " + curParameter.getValue();
		}
		result = result.trim();
		return result;
	}

	/**
	 * Generates and returns the value part of this parameter groups command 
	 * line call, using the currently set temporary values.
	 */ 
	public String getTemporaryValueString() {
		// Interate through all TextParameter objects and append their value
		String result = "";
		for (TextParameter curParameter : parameters) {
			result = result + " " + curParameter.getTemporaryValue();
		}
		result = result.trim();
		return result;
	}

	@Override
	public String getCommandlineCallString() {
		if (enabled)
			return key + getDisplayFriendlyValueString();
		else
			return "";
	}

	/**
	 * Creates a vector containing HistoryObjects for all TextParameter objects
	 * stored in the parameter vector, each adopting the corresponding parameters 
	 * temporary value.
	 */
	public Vector<HistoryObject> createHistoryObjectsToAdoptNewValues() {
		Vector<HistoryObject> changes = new Vector<HistoryObject>();
		for (TextParameter curParam : parameters) {
			HistoryObject curChange = curParam.createHistoryObjectToInheritTemporaryValues();
			if (curChange != null)
				changes.add(curChange);
		}
		return changes;
	}

	/**
	 * This method parses the default value string in order to store each 
	 * single value as the value of the corresponding TextParameter object.   
	 */
	public void setDefaultValue(String defaultValue) {
		this.defaultValue = defaultValue;

		// Parse default value to parameters
		parseInputToTemporaryValues(defaultValue);

		// Inerhit temporary values
		Vector<HistoryObject> actions = createHistoryObjectsToAdoptNewValues();
		for (HistoryObject curAction : actions)
			curAction.performAction();
	}

	@Override
	public String getDescription() {
		String result = this.description + "\n";
		// Take into account the descriptions of the single values
		for (TextParameter curParameter : parameters) {
			if (!curParameter.getDescription().equals(""))
				result = result + "<" + curParameter.getName() + "> " + curParameter.getDescription() + "\n";
		}
		return result;
	}

	/**
	 * This method sets the object references within the dynamic text parameters 
	 * according to the specified reference name. Further information about 
	 * the dynamic text parameters can be found in the classes documentation.
	 */
	public void createReferences() {
		for (TextParameter curParameter : parameters) {
			// Only process DynamicTextParameters
			if (curParameter instanceof DynamicTextParameter) {
				// Get reference name
				String referenceName = ((DynamicTextParameter)curParameter).getDynamicReferenceString();
				// Find corresponding parameter
				TextParameter referenceParameter = findParameterByName(referenceName);
				// Set reference
				((DynamicTextParameter) curParameter).setDynamicReference(referenceParameter);
			}

		}
	}

	/**
	 * This method find a parameter by its name and return it.
	 */
	private TextParameter findParameterByName(String name) {
		for (TextParameter curParameter : parameters) {
			if (curParameter.getName().equals(name))
				return curParameter;
		}
		return null;
	}


}
