/*
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
package core.graph;

import gui.helper.Grid;
import gui.helper.TextManager;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;
import java.io.Serializable;
import java.util.Vector;

/**
 * This abstract class defines the basic properties of a node.
 */
public abstract class BasicNode {
	/**
	 * Used to mark the node as used in graph algorithm. 
	 */
	protected boolean marked = false;
	
	/**
	 * X-Position of the center of this node on the canvas in pixel.
	 */
	protected int x = 0;
	/**
	 * Y-Position of the center of this node on the canvas in pixel.
	 */
	protected int y = 0;
	/**
	 * Y-Position of this node within the grid in grid cells.
	 */
	protected int gridY = -1;
	/**
	 * X-Position of this node within the grid in grid cells.
	 */
	protected int gridX = -1;
	/**
	 * Reference to the grid, used for calculating positions.
	 */
	protected Grid grid;
	/**
	 * Reference to the graph which this node is part of.
	 */
	protected Graph graph;

	/**
	 * Creates the node according to the passed values. 
	 */	
	public BasicNode(int pX, int pY, Grid pGrid, Graph graph){
		this.x = pX;
		this.y = pY;
		this.grid = pGrid;
		this.graph = graph;
	}

	public void setGraph(Graph graph) {
		this.graph = graph;
	}

	/**
	 * This method determines whether the passed coordinates are within the node.
	 * This method is used to determine the node at the cursers position.
	 * 
	 * @param x the x-coordinates of the cursers point
	 * @param y the y-coordinates of the cursers point
	 */
	public abstract boolean isInside(int x2, int y2);

	public void setGrid(Grid grid) {
		this.grid = grid;
	}

	public int getX() {
		return x;		
	}

	public int getY() {
		return y;		
	}

	/**
	 * Moves the node to the passed position. If pDoRealign is set, the node
	 * is aligned to the nearest grid cell center, otherwise it is moved exactly
	 * to the specified coordinates.
	 */
	public abstract void moveToPixelCoordinates(int pX, int pY, boolean pDoRealign);
	
	/**
	 * Set the nodes grid position and performs a realign.
	 */
	public abstract void moveToGridCoordinates(Point newPosition);
	
	/**
	 * This method returns the bounding box of the node.
	 */
	protected abstract Rectangle getBoundingBox();
	
	/**
	 * This method determines the new x- and y-pixel position of this node
	 * based on its current grid x- and y-position. This method is used, when
	 * resizing the workspace and therefore changing the grid cell positions.
	 */
	public void realignToGrid() {
		if ((gridX != -1) && (gridY != -1)) {
			Point newLocation = grid.getCellCenter(gridX, gridY);
			this.x = newLocation.x;
			this.y = newLocation.y;
		}	
		
	}

	/**
	 * This method draws the node.
	 * @param pGraphics Graphics object to draw the node on.
	 */
	public abstract void draw(Graphics pGraphics);
	
	public Grid getGrid() {
		return grid;
	}

	public Point getGridPosition() {
		return new Point(this.gridX, this.gridY);
	}

	public void setMarked(Boolean status) {
		marked = status;		
	}

	public boolean isMarked() {
		return marked;		
	}

	public Point getPosition() {
		return new Point(this.x, this.y);
	}
	
	/**
	 * Returns an information string representing informations about this node.
	 */
	public abstract String getTooltipText();


	/**
	 * Return this nodes name as String.
	 */
	public abstract String getName();



}