/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package core.graph;

import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Vector;

import javax.swing.JList;

import com.sun.corba.se.internal.Interceptors.PIORB;

import core.TypedFile;
import core.history.History;
import core.history.HistoryObject;
import core.storage.ConfigurationXMLReader;

import gui.AboutFrame;
import gui.HelpFrame;
import gui.ShowHistoryFrame;
import gui.WorkflowPanel;
import gui.helper.Grid;

/**
 * This class represents the graph which shapes the workflow 
 * with all its components.
 * 
 * A workflow consists of nodes and edges, where each node represents different 
 * calculation steps and edges determine the data flow. Nodes correspond to 
 * to different tools whilst edges determine the order of the execution.
 * The workflow can be exported into a Shell script in order to be executed.
 * This shell script consists of one setup script which prepares the workspace and 
 * the workflow script which executes the actual workflow.
 * 
 */

public class Graph implements Serializable{
	private static final long serialVersionUID = 1L;

	public static boolean DISPLAY_EDGE_INFORMATIONS;
	public static boolean DISPLAY_NODE_INFORMATIONS;
	public static boolean ALWAYS_DISPLAY_ORDER_KEYS;
	public static boolean DISPLAY_ALL_EDGE_INFORMATIONS;
	public static boolean DISPLAY_ALL_NODE_INFORMATIONS;
	
	private static int NEXT_FREE_ID;

	/**
	 * Nodes of this graph.
	 */
	private Vector<GraphNode> nodes;

	/**
	 * Edges of this graph.
	 */
	private Vector<Edge> edges;

	/** 
	 * Name of this graph.
	 */
	private String name;

	// TODO Merge variables in a suitable way
	private String outputFolder;
	private String expFolder;

	/**
	 * Reference to the xmlReader used for reading the configuration
	 * xml-file.
	 */
	private ConfigurationXMLReader reader;

	/**
	 * History to keep track of changes to the graph.
	 */
	private History history;

	/**
	 * Indicates if this graph has been saved since the last changes.
	 */
	private boolean saved;

	/**
	 * Creates the graph according to the passed values. 
	 */
	public Graph(String name, String outputFolder, String expFolder, ConfigurationXMLReader reader, WorkflowPanel workflowPanel, ShowHistoryFrame historyFrame) {		
		nodes = new Vector<GraphNode>();
		edges = new Vector<Edge>();
		//this.workflowPanel = workflowPanel;
		this.reader = reader;
		this.name = name;
		this.outputFolder = outputFolder;
		this.expFolder = expFolder;
		this.history = new History(workflowPanel, historyFrame);
		this.saved = true;
	}

	public String getOutputFolder() {
		return outputFolder;
	}

	public void setOutputFolder(String outputFolder) {
		this.outputFolder = outputFolder;
	}

	public String getExpFolder() {
		return expFolder;
	}

	public void setExpFolder(String outputFolder) {
		this.expFolder = outputFolder;
	}

	/**
	 * Adds the passed node to the graph.
	 */
	public void addNode(GraphNode k) {		
		nodes.add(k);		
	}

	/** 
	 * Adds the passed edge to the graph.
	 */
	public void addEdge(Edge k) {
		// Check if edge is already existent in the graph
		if (this.getSameEdge(k) == null) {
			edges.add(k);		
			k.getFrom().addOutgoingEdge(k);		
			k.getTo().addIncomingEdge(k);		
		}
	}

	/**
	 * This method returns the list of all edges which are connected to the passed
	 * node.
	 */
	public Vector<Edge> getAllEdgesConnectedToNode(BasicNode pAbstractNode) {	
		Vector<Edge> result = new Vector<Edge>();

		if (!(pAbstractNode instanceof PointNode)) {
			result.addAll(((GraphNode) pAbstractNode).getIncomingEdges());
			result.addAll(((GraphNode) pAbstractNode).getOutgoingEdges());
		}
		return result;
	}

	/**
	 * Removes the passed node from the graph.
	 */
	public void removeNode(BasicNode pAbstractNode) {	
		if (pAbstractNode instanceof PointNode) {
			((PointNode) pAbstractNode).getEdge().deletePointNode((PointNode) pAbstractNode);
		} else {		
			nodes.remove(pAbstractNode);			
			// Hide edit windows if it was visible
			if (pAbstractNode instanceof JobNode) {
				JobNode curNode = (JobNode)pAbstractNode;
				if (curNode.hasEditDialog()) {
					curNode.hideEditDialog();					
				}
			}	
		}
	}

	/**
	 * Removes all edges connecting the two passed nodes. 
	 */
	public void removeEdge(GraphNode pFrom, GraphNode pTo) {
		Edge curEdge = getSameEdge(new Edge(pFrom, pTo, ""));
		while (curEdge != null) {
			removeEdge(curEdge);

			curEdge = getSameEdge(new Edge(pFrom, pTo, ""));
		}
	}	

	/**
	 * Removes the passed edge from the graph and deletes all 
	 * references to it in all nodes connected to it.
	 */
	public void removeEdge(Edge pEdge) {
		edges.remove(pEdge);

		// Remove references in nodes
		pEdge.getFrom().removeEdge(pEdge);
		pEdge.getTo().removeEdge(pEdge);			
	}	

	/**
	 * Searches and returns edge with the same start and end point as the passed one.
	 */
	public Edge getSameEdge(Edge k) {
		for (Edge curEdge : edges) {
			if (curEdge.getFrom().equals(k.getFrom()) && (curEdge.getTo().equals(k.getTo()))) 
				return curEdge;							
		}
		return null;		
	}

	/**
	 * This method removes all edges from the graph.
	 */
	public void removeAllEdges() {	
		for (GraphNode curEdge : nodes) {
			curEdge.removeAllEdges();			
		}
		this.edges = new Vector<Edge>();
		saved = false;
	}

	public Vector<Edge> getEdges() {
		return edges;		
	}

	public Vector<GraphNode> getAbstractNodes() {
		return nodes;		
	}

	/**
	 * This method draws the entire workflow 
	 */
	public void draw(Graphics pGraphics, Grid pGrid) {
		// Edges
		for (Edge curEdge : edges) {	
			curEdge.draw(pGraphics);
		}

		// Nodes
		for (GraphNode curAbstractNode : nodes) {
			curAbstractNode.draw(pGraphics);
		}
	}

	/**
	 * Realigns all nodes and edges to the grid after resizing of the gui.
	 */
	public void realignToGrid(Grid pGrid) {
		for (GraphNode curAbstractNode : nodes) {
			curAbstractNode.realignToGrid();
		}	
		for (Edge curEdge : edges) {
			curEdge.realignToGrid();
		}
	}

	/**
	 * This method determines the node which is located at the passed coordinates 
	 * and returns it.
	 */
	public BasicNode getBasicNodeAtPosition(int x, int y, Grid pGrid) {
		for (GraphNode curAbstractNode : nodes) {
			if (curAbstractNode.isInside(x,y)) 
				return curAbstractNode;		
		}
		for (Edge curEdge : edges) {
			PointNode curEdgePointNode = curEdge.getPointNodeAt(x,y);
			if (curEdgePointNode != null)
				return curEdgePointNode;
		}
		return null;

	}

	/**
	 * This method determines the edge which is located at the passed coordinates 
	 * and returns it.
	 */
	public Edge getEdgeAtPosition(int x, int y, Grid pGrid) {
		for (Edge curEdge : edges) {
			boolean inside = curEdge.isInsidePointNodeOfEdge(x,y,pGrid);
			if (inside == true)
				return curEdge;	
		}
		return null;
	}

	public ConfigurationXMLReader getReader() {
		return reader;
	}

	public Grid getGrid() {
		for (GraphNode curAbstractNode : nodes) {
			return curAbstractNode.getGrid();
		}
		return null;
	}

	
	/**
	 * Removes any mark (marked, edited, visible) from all node in the graph.
	 */
/*
	public void unmarkEverything() {
		for (GraphNode curAbstractNode : nodes) {
			curAbstractNode.setCurrentlyEdited(false);
			curAbstractNode.setEditDialogVisible(false);
			curAbstractNode.setMarked(false);
		}

	}
*/
	
	public String getName() {
		return name;
	}

	/**
	 * Returns the next free id and increments it.
	 */
	public int getNextFreeID() {
		Graph.NEXT_FREE_ID++;
		return (Graph.NEXT_FREE_ID - 1);
	}

	public void setName(String newName) {
		saved = false;
		this.name = newName;
	}

	/**
	 * Generate a file name which is not used yet. 
	 */
	public String generateNewFileName(String center) {
		return getName() + "_" + center + getNextFreeID();
	}

	/**
	 * This method creates the two shell scripts allowing the workflow
	 * represented by this graph to be executet. 
	 */
	public void createScripts(String targetFileName) throws IOException {
		// Determine which binaries and scripts are used
		Vector<Integer> usedProgrammIDs = new Vector<Integer>();
		Vector<Integer> usedInputConverterIDs = new Vector<Integer>();
		Vector<Integer> usedOutputConverterIDs = new Vector<Integer>();
		
		for (GraphNode curAbstractNode : this.nodes) {
			int curID = curAbstractNode.getReferenceID();
			if (curAbstractNode instanceof JobNode) {
				if (usedProgrammIDs.indexOf(curID) == -1)
					usedProgrammIDs.add(curID);
			} else if (curAbstractNode instanceof InputFileNode) {
				if (usedInputConverterIDs.indexOf(curID) == -1)
					usedInputConverterIDs.add(curID);			
			} else if (curAbstractNode instanceof OutputFileNode) {
				if (usedOutputConverterIDs.indexOf(curID) == -1)
					usedOutputConverterIDs.add(curID);			
			}
		}
		
		// Determine all requirements
		Vector<String> requirements = reader.getRequirements(usedProgrammIDs, usedInputConverterIDs, usedOutputConverterIDs);
		
		createWorkflowScript(targetFileName, requirements);
		// Ensure proper extension
		if (targetFileName.indexOf(".sh") == -1) 
			targetFileName = targetFileName + ".sh";
		
		String setupFileName = targetFileName.substring(0, targetFileName.lastIndexOf(".")) + "_setup.sh"; 
		createSetupScript(setupFileName, usedProgrammIDs, requirements);
	}

	public String createRequirmentsCheck(Vector<String> requirements, boolean exitOnError) {
		String result = "# Before any further steps, check if all required external binaries are present in the path.\n";
		result = result + "requirements=\"";
		for (String curRequirement : requirements)
			result = result + curRequirement.trim() + " ";
		result = result.trim();
		result = result + "\"\n";
		result = result + "missingCommands=\"\"\n";
		result = result + "for curCmd in $requirements; do\n";
		result = result + "\tif ! [ `command -v $curCmd` ]; then\n";
		result = result + "\t\tmissingCommands=\"$missingCommands\\n$curCmd\"\n";
		result = result + "\tfi\n";
		result = result + "done\n";
		result = result + "if ! [ -z \"$missingCommands\" ]; then\n";
		result = result + "\techo \"Warning, the following required external commands were not found on your system.\\nPlease ensure their presence before executing the workflow.$missingCommands\"\n";
		if (exitOnError)
			result = result + "\texit 1\n";
		result = result + "fi\n";
				
		return result;	
	}
	
	/**
	 * This method creates a script to create the working environment for the
	 * actual workflow execution. 
	 * This script setups all the necessary tools for the workflow. It creates all
	 * needed folders and copies all used the binaries.
	 * @param requirements 
	 * @param usedProgrammIDs 
	 */
	public void createSetupScript(String targetFileName, Vector<Integer> usedProgrammIDs, Vector<String> requirements) throws IOException {
		String result = generateBatchHeader();
		
		result = result + createRequirmentsCheck(requirements, false) + "\n";
		result = result + "# Before the workflow can be used, this script sets up the environment.\n";
		result = result + "\n";
		result = result + "# First, all folders are set.\n";
	
		result = result + "export SOFTWARESOURCE=\"" + reader.getConfigValue("softwaresource").get(0) + "\"\n";		
		result = result + "export TARGETDIR=\"" +  (new TypedFile(targetFileName)).getParent() + "\"\n";
		
		result = result + "\n";
		result = result + "\n";
		result = result + "# If no parameter was passed, remove old Folders\n";
		result = result + "if [ -z \"$1\" ]; then \n";
		result = result + "		\\rm -f -R ${TARGETDIR}/Results/\n";
		result = result + "		\\rm -f -R ${TARGETDIR}/Software/\n";
		result = result + "fi\n";
		result = result + "\n";
		result = result + "\n";
		result = result + "# Creater Folders\n";
		result = result + "\\mkdir -p \"${TARGETDIR}/Results/\"\n";
		result = result + "\\mkdir -p \"${TARGETDIR}/Software/\"\n";
//		result = result + "\\mkdir -p \"${TARGETDIR}/Test/\"\n";
		result = result + "\n";
		result = result + "# Copy Software\n";

		// Copy all used binaries
		for (int curID : usedProgrammIDs) {
			String binarayPath = reader.getProgrammBinaryPathByID(curID);
			if (!binarayPath.equals(""))
				binarayPath = binarayPath.substring(0, binarayPath.length() - 1);
			String binarayName = reader.getProgrammBinaryNameByID(curID);

			if (binarayPath.equals(""))
				result = result + "\\cp -f -R \"${SOFTWARESOURCE}/" + binarayName + "\"" + " \"${TARGETDIR}/Software/" + binarayName + "\"" + "\n";
			else
				result = result + "\\cp -f -R \"${SOFTWARESOURCE}/" + binarayPath + "\"" + " \"${TARGETDIR}/Software/" + binarayPath + "\"" + "\n";
		}

		// Copy all other needed folders
		Vector<String> auxiliaryfolders = reader.getConfigValue("auxiliaryfolder");
		for (String auxiliaryfolder : auxiliaryfolders) {

			result = result + "\\cp -f -R " +"\""+ auxiliaryfolder + "\"" + " \"${TARGETDIR}/Software\"\n";	
		}

		File targetFile = new File(targetFileName); 
		FileWriter writer = new FileWriter(targetFile); 
		BufferedWriter buffer = new BufferedWriter(writer); 
		buffer.write(result);
		buffer.close();

		targetFile.setExecutable(true);
	}

	/**
	 * This method creates the script which carry out the workflow.
	 * This script executes the tools of the workflow.
	 * @param requirements 
	 */
	public void createWorkflowScript(String targetFileName, Vector<String> requirements) throws IOException {
		String resultHead = generateBatchHeader();
		
		resultHead = resultHead + createRequirmentsCheck(requirements, true) + "\n";
		resultHead = resultHead + "# Before the actual execution, variables are set. To adapt the\n";
		resultHead = resultHead + "# batch to another location just change the BASEDIR variable.\n";
		
		resultHead = resultHead + "export BASEDIR=\"" + (new TypedFile(targetFileName)).getParent() + "\"\n";
		
		resultHead = resultHead + "\n";
		resultHead = resultHead + "\n";

		resultHead = resultHead + "# Derived folders: \n";
		resultHead = resultHead + "export RESULTDIR=\"${BASEDIR}/Results\" \n";	
		resultHead = resultHead + "export SOFTWAREDIR=\"${BASEDIR}/Software\" \n";
		resultHead = resultHead + "\n";
		resultHead = resultHead + "\n";
		resultHead = resultHead + "# ----- Convert non-XML files ----- \n";

		String resultTail = "";
		String inputVisited = "";
		String outputVisited = "";

		// Create Vectors 
		Vector<GraphNode> visitedAbstractNodes = new Vector<GraphNode>();
		Vector<GraphNode> unvisitedAbstractNodes = new Vector<GraphNode>();

		// First, find FileAbstractNodes	and create their calls and
		// the beginning (and the end) of the batch
		this.sortNodes();
		for (GraphNode curAbstractNode : nodes) {
			if (curAbstractNode instanceof InputFileNode) {

				visitedAbstractNodes.add(curAbstractNode);

				resultHead = resultHead + "\n" + curAbstractNode.getCommandLineCall("${SOFTWAREDIR}/", "${RESULTDIR}/");	
				inputVisited = inputVisited + curAbstractNode.getName();
			} else if (curAbstractNode instanceof OutputFileNode) {

				visitedAbstractNodes.add(curAbstractNode);

				resultTail = resultTail + "\n" + curAbstractNode.getCommandLineCall("${SOFTWAREDIR}/", "${RESULTDIR}/");	
				outputVisited = outputVisited + curAbstractNode.getName();
			} else {
				unvisitedAbstractNodes.add(curAbstractNode);
			}
		}

		resultHead = resultHead + "\n";
		resultHead = resultHead + "\n";
		resultHead = resultHead + "# ----- Execute Jobs ----- \n";

		// Then create calls for all normal jobs
		while (unvisitedAbstractNodes.size() > 0) {
			GraphNode curAbstractNode = this.findAbstractNodeWithAllPredecessorsVisited(visitedAbstractNodes, unvisitedAbstractNodes);
			visitedAbstractNodes.add(curAbstractNode);
			unvisitedAbstractNodes.remove(curAbstractNode);
			resultHead = resultHead + curAbstractNode.getCommandLineCall("${SOFTWAREDIR}/", "${RESULTDIR}/") + "\n";		
		}

		resultHead = resultHead + "\n";
		resultHead = resultHead + "\n";
		resultHead = resultHead + "# ----- Convert XML-Files to external outputs ----- \n";

		// Create, display and output final batch
		String result = resultHead + resultTail;
		File targetFile = new File(targetFileName); 
		FileWriter writer = new FileWriter(targetFile); 
		BufferedWriter buffer = new BufferedWriter(writer); 
		buffer.write(result);
		buffer.close();
		//System.out.println("Created Workflow-Script " + targetFileName);

		targetFile.setExecutable(true);
	}

	/**
	 * Sorts this graphs node according to their position value.
	 */
	private void sortNodes() {
		Vector<GraphNode> newNodes = new Vector<GraphNode>();
		while (nodes.size() > 0) {
			GraphNode minNode = null;
			for (GraphNode curNode : nodes) {
				if ((minNode == null) || (curNode.getOrderPosition() < minNode.getOrderPosition()))
					minNode = curNode;
			}
			
			nodes.remove(minNode);
			newNodes.add(minNode);
		}
		
		this.nodes = newNodes;
	}

	/**
	 * This method generates the batch header used when creating scripts.
	 */
	private String generateBatchHeader() {
		String result = "#!/bin/sh\n";
		result = result + "# -----------------------------\n";
		result = result + "# Export of workflow: " + name + "\n";
		result = result + "# -----------------------------\n";
		result = result + "\n";
		result = result + "# This workflow was created with " + AboutFrame.NAME + " " + HelpFrame.PROFIDO_VERSION + "\n";
		result = result + "\n";
		result = result + "\n";
		return result;
	}

	/**
	 * This method determines the unvisited nodes of which all predecessor were visited.
	 * This method is called by createWorkflowScript to create the calls for the job nodes.
	 */
	private GraphNode findAbstractNodeWithAllPredecessorsVisited(Vector<GraphNode> visitedAbstractNodes, Vector<GraphNode> unvisitedAbstractNodes) {
		if (visitedAbstractNodes.size() == 0) {
			return unvisitedAbstractNodes.get(0);
		} else {
			for (GraphNode curAbstractNode : unvisitedAbstractNodes) {
				if (curAbstractNode.allPredecessorsVisited(visitedAbstractNodes)) 
					return curAbstractNode;
			}

		}	
		return null;
	}

	/**
	 * Add a new action to the history.
	 */
	public void addNewAction(HistoryObject newAction) {
		history.addNewAction(newAction);
	}

	/**
	 * Perform the last action added to the histroy.
	 */
	public void performLastAction() {
		saved = false;
		history.performLastAction();		
	}

	public String getHistoryText() {
		return history.toString();
	}

	/**
	 * Undo the last change applied to the graph.
	 */
	public void undo() {
		history.undo(1);
	}

	/**
	 * Redo the last change applied to the graph.
	 */
	public void redo() {
		history.redo(1);	
	}

	public boolean isSaved() {
		return saved;
	}

	public void setSaved(boolean saved) {
		this.saved = saved;
	}

	public void clearHistory() {
		this.history.clear();
	}

	/** 
	 * This method creates a string used to store the graph in an XML file 
	 * and returns it. The elements of the workflow are saved in the string 
	 * step by step in the following order input files, output files, job nodes 
	 * and finally edges.
	 */
	public String getXMLRepresentationOfGraph(){
		String savefile = "";
		savefile += "<?xml version= \"1.0\" ?>"+"\n";
		savefile += "<workflow>"+"\n";
		savefile += "\t<general>"+"\n";
		savefile += "\t\t<name> "+ getName() +" </name>"+"\n"; 
		savefile += "\t\t<folder> "+ getOutputFolder() +" </folder>"+"\n"; 
		savefile += "\t\t<path> "+ getExpFolder() +" </path>"+"\n"; 
		savefile += "\t\t<settings> "+ Graph.DISPLAY_NODE_INFORMATIONS + " " + Graph.DISPLAY_EDGE_INFORMATIONS + " " + Graph.ALWAYS_DISPLAY_ORDER_KEYS + " " + Graph.DISPLAY_ALL_NODE_INFORMATIONS + " " + Graph.DISPLAY_ALL_EDGE_INFORMATIONS + "</settings>"+"\n"; 
		savefile += "\t\t<version> "+ AboutFrame.PROFIDO_VERSION +  "</version>"+"\n"; 
		savefile += "\t</general>"+"\n";
		savefile += "\t<nodes>"+"\n";

		for (GraphNode aN : nodes) {
			if(aN instanceof InputFileNode){
				savefile += aN.getXMLRepresentationOfNode(nodes.indexOf(aN))+"\n";
			}
		}
		for (GraphNode aN : nodes) {
			if(aN instanceof OutputFileNode){
				savefile += aN.getXMLRepresentationOfNode(nodes.indexOf(aN))+"\n";
			}
		}
		for (GraphNode aN : nodes) {
			if(aN instanceof JobNode){
				savefile += aN.getXMLRepresentationOfNode(nodes.indexOf(aN))+"\n";
			}
		}
		savefile += "\t</nodes>"+"\n";

		if (edges.size()>0){

			savefile += "\t<edges>"+"\n";
			for (Edge e : edges) {
				GraphNode ef = e.getFrom();
				GraphNode et = e.getTo();
				int a = nodes.indexOf(ef);
				int b = nodes.indexOf(et);

				savefile += e.getXMLRepresentationOfEdge(a, b )+"\n";
			}
			savefile += "\t</edges>"+"\n";

		}
		savefile += "</workflow>"+"\n";

		return savefile;
	}

	public History getHistory() {
		return history;
	}

	/**
	 * This method checks if the workflow is build correctly. For a correct
	 * workflow every node has to be connected and at least one input and 
	 * output node have to be present. 
	 * If any error is detected, a detailed message is returned.
	 * 
	 * An input node has to have at least one outgoing edge.
	 * An output node has to have at least one incoming edge.
	 * A job node has to have at least one incoming and one outgoing edge.
	 */
	public String checkBuild() {
		String result = "";
		boolean outputNodePresent = false;
		boolean inputNodePresent = false;

		for(GraphNode v : nodes){
			if (v instanceof InputFileNode) {
				inputNodePresent = true;
				if(v.getOutgoingEdges().isEmpty())
					result = result + "Input node \"" + v.getName() + "\" has to have at least one outgoing edge.\n";
			}

			else if (v instanceof OutputFileNode ) {
				outputNodePresent = true;
				if (v.getIncomingEdges().isEmpty())
					result = result + "Output node \"" + v.getName() + "\" has to have at least one incoming edge.\n";		
			}

			else if (v instanceof JobNode ) {
				if (v.getIncomingEdges().isEmpty()){ 
					result = result + "Job node \"" + v.getName() + "\" has to have at least one incoming edge.\n";
				} 
				if (v.getOutgoingEdges().isEmpty()){ 
					result = result + "Job node \"" + v.getName() + "\" has to have at least one outgoing edge.\n";
				} 
			}

		}

		if (!inputNodePresent)
			result = result + "The workflow needs at least one input node.\n";

		if (!outputNodePresent)
			result = result + "The workflow needs at least one output node.\n";

		return result;
	}

	/**
	 * This method checks if the graph contains a cycle and returns a
	 * detailed message containing further informations if any is found.
	 * @return
	 */
	public String checkForCycle() {
		String result = "";

		for (GraphNode curNodeToCheck : nodes) {
			Vector<GraphNode> markedNodes = new Vector<GraphNode>();
			Vector<GraphNode> nodesToVisit = new Vector<GraphNode>();
			nodesToVisit.add(curNodeToCheck);
			boolean cycle = false;

			while ((nodesToVisit.size() > 0) && (!cycle)) {
				GraphNode curNode = nodesToVisit.remove(0);
				markedNodes.add(curNode);
				for (Edge curEdge : curNode.getOutgoingEdges()) {
					GraphNode nextNode = curEdge.getTo();

					if (nextNode == curNodeToCheck) {
						cycle = true;
						result = result + "\"" + curNodeToCheck.getName() + "\", ";
					} else if (markedNodes.indexOf(nextNode) == -1) {
						nodesToVisit.add(nextNode);						
					}		
				}			
			}	
		}

		if (!result.equals(""))
			result = "The nodes " + result.substring(0, result.length() - 2) + " are part of a cycle.\n";

		return result;
	}

	/**
	 * Returns a bounding box completely enclosing this graph.
	 * @param grid 
	 */
	public Rectangle getBoundingBox(int cellWidth, int cellHeight) {
		int ulX = -1;
		int ulY = -1;
		int brX = -1;
		int brY = -1;
		
		for (GraphNode curNode : nodes) {
			int curX = curNode.getX();
			int curY = curNode.getY();
			if ((ulX == -1) || (curX < ulX)) 
				ulX = curX;
			if ((ulY == -1) || (curY < ulY)) 
				ulY = curY;
			if ((brX == -1) || (curX > brX)) 
				brX = curX;
			if ((brY == -1) || (curY > brY)) 
				brY = curY;
		}


		return new Rectangle(ulX - (cellWidth / 2), ulY - (cellHeight / 2), brX - ulX + cellWidth, brY - ulY + cellHeight);

//		return new Rectangle(ulX - space.width / 2, ulY - space.height / 2, brX - ulX + space.width, brY - ulY + space.height);
	}

	public Vector<Point> getJobNodePositions() {
		Vector<Point> result = new Vector<Point>();
		for (BasicNode curNode : nodes)
			result.add(curNode.getPosition());
		return result;
	}

	public Vector<Point> getEdgePositions() {
		Vector<Point> result = new Vector<Point>();
		for (Edge curEdge : edges)
			result.add(curEdge.getLastPoint());
		return result;
	}




}
