/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package core.graph;

import gui.helper.Grid;
import gui.helper.TextManager;
import helper.SmallHelpers;


import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;
import java.io.Serializable;
import java.util.Vector;

import core.TypedFile;
import core.storage.ConfigurationXMLReader;


/**
 * This abstract class defines the properties of nodes as used in the workflow 
 * graph. The subclasses of this class represent the specific node types such 
 * as file and job node.
 * 
 * It stores and controls most of the informations relevant for managing the 
 * node within the graph and for drawing it on the canvas.
 * 
 * Each node has a position within the canvas and a set of incoming and outgoing 
 * edges.
 * 
 */

public abstract class GraphNode extends BasicNode implements Serializable {
	private static final long serialVersionUID = 1L;
	protected static final Color DEFAULT_COLOR = Color.BLACK;
	protected static final int BORDER_SIZE = 1;
	private static final int DEFAULT_ORDER_KEY = 10;
	
	private static int nextFreeOrderPosition = 1;

	/**
	 * Name of this node.
	 */
	protected String name;

	/**
	 * ID explicitly identifying this nodes assigned tool. 
	 */
	protected int referenceID;

	/**
	 * Color of this node used when drawing it on the canvas.
	 */
	protected Color color;

	/**
	 * Storage for the edges connected to this node
	 */
	protected Vector<Edge> incomingEdges, outgoingEdges;

	/**
	 * Indicates if this nodes edit dialog is currently visible in the gui.
	 */
	private boolean editDialogVisible = false;

	/**
	 * Indicates if this node is currently edited in order to draw it in 
	 * a different color.
	 */
	protected boolean currentlyEdited = false;

	/**
	 * Reference to the XML reader used for reading the configurations.
	 */
	protected ConfigurationXMLReader reader;

	/**
	 * File representing the xml output of this node. 
	 */	
	protected TypedFile xmlOutputFile;
	
	/**
	 * Position of this node within the graphs node list.
	 */
	private int positionInGraphNodes;

	/**
	 * Generate and return an information text of this node to display it in the
	 * workflow.
	 */
	protected abstract Vector<String> getCaptionText(boolean ignoreVisibilty, boolean markEnabled, boolean sortEnabled);

	@Override
	public String getTooltipText() {
		Vector<String> caption = this.getCaptionText(true, true, true);
		String result = "<html>";
		for (String curLine : caption)
			result = result + curLine + "<br>";
		return result + "</html>";
	}
	
	/**
	 * Returns a vector of string containing all types this
	 * node in its current state can accept as input.
	 */
	public abstract Vector<String> getInputFileTypes();


	/**
	 * Returns a vector of string containing all types this
	 * node in its current state will return as output.
	 */
	public abstract Vector<String> getOutputFileTypes();


	/**
	 * This method generates and returns the command line call represented by 
	 * this node.
	 * @param softwarePath Path where the software binaries are located.
	 * @param resultPath Path where the output of this node should be stored.
	 */
	public abstract String getCommandLineCall(String softwarePath, String resultPath);	

	/**
	 * Calculates and returns all intersections of the passed edge with the 
	 * shape of this node.
	 */
	public abstract Point calculateIntersection(Edge pEdge);

	/** 
	 * This method generates and returns a string representing all informations 
	 * of this node needed to store this node in an xml-file.
	 */
	public abstract String getXMLRepresentationOfNode(int id);

	/**
	 * Creates the node according to the passed values. 
	 */
	public GraphNode(int pX, int pY, ConfigurationXMLReader reader, Grid pGrid, int id, Graph graph) {
		super(pX, pY, pGrid, graph);
		this.referenceID = id;
		this.reader = reader;
		this.incomingEdges = new Vector<Edge>();
		this.outgoingEdges = new Vector<Edge>(); 
		this.color = DEFAULT_COLOR;				
		this.xmlOutputFile = null;
		this.positionInGraphNodes = GraphNode.nextFreeOrderPosition;
		GraphNode.nextFreeOrderPosition++;
	}
		
	public int getOrderPosition() {
		return positionInGraphNodes;
	}

	/**
	 * Creates the node according to the passed values. 
	 */
	public GraphNode(int pX,  int pY){
		super(pX, pY, null, null);
	}

	public void setColor(Color pColor) {
		color = pColor;		
	}

	public Color getColor() {
		return color;		
	}

	/**
	 * Adds the passed edges to the vector storing the incoming edges
	 * and triggers the resorting this nodes incoming edges.
	 */
	public void addIncomingEdge(Edge k) {
		this.incomingEdges.add(k);	
		this.sortIncomingEdgesAccordingToOrderKey();
	}

	/**
	 * Adds the passed edges to the vector storing the outgoing edges.
	 */
	public void addOutgoingEdge(Edge k) {
		this.outgoingEdges.add(k);		
	}

	/**
	 * Removes the passed edge from either edge vectors of this node.
	 */
	public void removeEdge(Edge k) {
		this.incomingEdges.remove(k);	
		this.outgoingEdges.remove(k);
	}

	/**
	 * Removes all edges from the edge vectors of this node.
	 */
	public void removeAllEdges() {
		incomingEdges = new Vector<Edge>();
		outgoingEdges = new Vector<Edge>(); 
	}

	public Vector<Edge> getIncomingEdges() {		
		return incomingEdges;
	}

	public Vector<Edge> getOutgoingEdges() {		
		return outgoingEdges;
	}

	/**
	 * This method determines the point within the passed vector of points which 
	 * has the smallest distance to the additional passed location and returns it.
	 */
	protected Point getClosestIntersection(Vector<Point> points, double px, double py) {
		// TODO Improve calculations
		Point closestIntersection = null;
		double closestDist = 99999;
		double curDist;
		for (Point curIntersection : points) {
			if (curIntersection != null) {
				curDist = curIntersection.distance(px,py);
				if (curDist < closestDist) {
					closestDist = curDist;
					closestIntersection = curIntersection;
				}					
			}
		}
		return closestIntersection;
	}


	public void setCurrentlyEdited(boolean b) {
		currentlyEdited = b;
	}

	public int getReferenceID() {
		return referenceID;
	}

	public void setName(String newName) {
		name = newName;
	}

	public TypedFile getXMLOutputFile() {
		return xmlOutputFile;
	}

	/**
	 * This method checks if any of this nodes predecessors are not part of the
	 * passed vector.
	 */
	public boolean allPredecessorsVisited(Vector<GraphNode> visitedAbstractNodes) {
		boolean allVisited = true;
		for (Edge curEdge : this.incomingEdges) {
			if (visitedAbstractNodes.indexOf(curEdge.getFrom()) == -1) {
				allVisited = false;
				break;
			}						
		}

		return allVisited;
	}

	@Override
	public void moveToPixelCoordinates(int pX, int pY, boolean pDoRealign) {
		if (pDoRealign) {
			Point gridPosition = grid.getNearestCell(pX, pY);
			this.gridX = gridPosition.x;
			this.gridY = gridPosition.y;
			realignToGrid();
		}
		else {
			this.x = pX;
			this.y = pY;			
		}

		// Notify edges that this node has moved in order to realign PointNodes on the edges
		if (outgoingEdges != null) {
			for (Edge curEdge : outgoingEdges)
				curEdge.notifiyThatNodeHasMoved();			
		}

		if (incomingEdges != null) {
			for (Edge curEdge : incomingEdges)
				curEdge.notifiyThatNodeHasMoved();			
		}
	}

	@Override
	public void moveToGridCoordinates(Point newPosition) {
		this.gridX = newPosition.x;
		this.gridY = newPosition.y;
		this.realignToGrid();

		if (outgoingEdges != null) {
			for (Edge curEdge : outgoingEdges)
				curEdge.notifiyThatNodeHasMoved();			
		}

		if (incomingEdges != null) {
			for (Edge curEdge : incomingEdges)
				curEdge.notifiyThatNodeHasMoved();			
		}
	}

	/**
	 * This method draws the outline of this node. 
	 * 
	 * @param pGraphics Graphics object to draw the outline on.
	 * @param pShape The bounding box of this node. Does not determine the 
	 * 		   		 exact outline of this node, but its outer boundaries.
	 * 				 The exact outline of the node determined within the
	 * 				 particular implementation within the subclasses.
	 */
	protected abstract void drawOutline(Graphics pGraphics, Rectangle pShape);


	@Override
	public void draw(Graphics pGraphics) {
		// Set color according to the state of this node
		if (currentlyEdited) {
			pGraphics.setColor(Color.red);				
		} else
			pGraphics.setColor(color);

		// Fill outer shape of node in nodes color
		Rectangle outterShape = getBoundingBox();
		drawOutline(pGraphics, outterShape);

		// Clear inner shape of node 
		int borderSize = BORDER_SIZE;
		if (marked)
			// If marked, draw border in bold
			borderSize = 2 * borderSize;

		Rectangle innerShape = new Rectangle(outterShape.x + borderSize, outterShape.y + borderSize, outterShape.width - (2 * borderSize), outterShape.height - (2 * borderSize));
		pGraphics.setColor(Color.white);
		drawOutline(pGraphics, innerShape);

		// Write name inside the node
		pGraphics.setColor(this.getColor());

		String toolTip = this.getName();		
		String displayName = toolTip;

		// Cut name if to long
		if(displayName.length() > 8) {
			displayName = displayName.substring(0, 7);
			displayName += "...";
		}

		TextManager.writeLargeText(displayName, (int) (outterShape.getCenterX()), (int) (outterShape.getCenterY()), pGraphics, true);

		// Write further informations (parameters, etc.) above the node
		if (Graph.DISPLAY_ALL_NODE_INFORMATIONS || Graph.DISPLAY_NODE_INFORMATIONS) {
			Vector<String> tooltipLines = this.getCaptionText(Graph.DISPLAY_ALL_NODE_INFORMATIONS, Graph.DISPLAY_ALL_NODE_INFORMATIONS, Graph.DISPLAY_ALL_NODE_INFORMATIONS);
			TextManager.writeSmallText(tooltipLines, (int) (outterShape.getCenterX() - (outterShape.getWidth() / 2)) + 7, (int) (y - (outterShape.getHeight() / 2)) - 7, pGraphics, false);
		}
	}

	/**
	 * This method intersects this nodes output types with the input
	 * types of the passed node and returns a vector containing all
	 * common types.
	 */
	public Vector<String> intersectOutputTypesWithNodesInputTypes(GraphNode to) {
		return SmallHelpers.intersect(this.getOutputFileTypes(), to.getInputFileTypes());
	}

	public void setXMLFileName(String newName) {
		Vector<String> types = xmlOutputFile.getTypes();
		xmlOutputFile = new TypedFile(newName, types);
	}

	/**
	 * Returns the next free order key.
	 */
	public String getNextFreeOrderKey() {
		String maxKey = "";
		String result = Integer.toString(GraphNode.DEFAULT_ORDER_KEY);
		
		// Find maximum key on edge
		for (Edge curEdge : incomingEdges) 
			if ((maxKey == "") || (curEdge.getOrderKey().compareTo(maxKey) > 0))
				maxKey = curEdge.getOrderKey();

		if (maxKey.length() > 0) {
			// Determine next key according to the nature of the maxkey
			try {
				// If number increment by default value
				int maxKeyValue = Integer.valueOf(maxKey);
				result = Integer.toString(maxKeyValue + GraphNode.DEFAULT_ORDER_KEY);
			} catch (NumberFormatException e) {
				// If no number, try to increment last char
				char lastChar = maxKey.charAt(maxKey.length() - 1);
				char newLastChar = (char) (lastChar + 1);
				if (Character.isLetter(newLastChar)) {
					String oldPrefix = "";
					if (maxKey.length() > 1)
						oldPrefix = maxKey.substring(0, maxKey.length() - 1);
					result = oldPrefix + newLastChar;
				}
				else
					result = maxKey + "a";
			}
		}
		return result;
	}

	/**
	 * Sorts this nodes incoming edges according
	 * to their oder key.
	 */
	public void sortIncomingEdgesAccordingToOrderKey() {
		Vector<Edge> newIncomingEdges = new Vector<Edge>();
		while (incomingEdges.size() > 0) {
			// Find minimum
			Edge minKeyEdge = null;
			for (Edge curEdge : incomingEdges) 
				if ((minKeyEdge == null) || (curEdge.getOrderKey().compareTo(minKeyEdge.getOrderKey()) < 0))
					minKeyEdge = curEdge;

			// Remove minimum
			incomingEdges.remove(minKeyEdge);
			newIncomingEdges.add(minKeyEdge);
		}
		 
		incomingEdges = newIncomingEdges;	
	}


}