/*
* 
* ProFiDo - The Processes Fitting Toolkit Dortmund
* Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License along
* with this program; if not, write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*
*/

package core.graph;

import gui.helper.Grid;

import java.awt.Graphics;
import java.awt.Point;
import java.awt.Color;
import java.awt.Rectangle;
import java.io.Serializable;
import java.util.Vector;

import core.storage.ConfigurationXMLReader;

/**
 * This class is a specific node that creates the bases (green points) of the 
 * edges used to allow more flexible alignment in order to avoid crossings 
 * and thus to enable a more readable graph layout.
 */

public class PointNode extends BasicNode implements Serializable{
	private static final long serialVersionUID = 1L;
	private static final int RADIUS = 4;
	
	/**
	 * Indicates if this node is used positioned or if
	 * its position is calculated according to its neighboring nodes.
	 */
	private boolean isUserPositioned = false;

	/**
	 * Reference to the left neighbouring node of this node.
	 */
	private BasicNode leftNeighbour;

	/**
	 * Reference to the right neighbouring node of this node.
	 */
	private BasicNode rightNeighbour;

	private Edge edge;

	/**
	 * Creates the node according to the passed values. 
	 */
	public PointNode(Grid grid, BasicNode leftNeighbour, BasicNode rightNeighbour, boolean userPositioned, Edge edge) {
		super(-1, -1, grid, null);
		this.leftNeighbour = leftNeighbour;
		this.rightNeighbour = rightNeighbour;
		this.isUserPositioned = userPositioned;
		this.edge = edge;
		this.determinePositionAccordingToNeighbours();

		if (userPositioned) {
			moveToPixelCoordinates(x, y, true);
		}
	}
	
	@Override
	public void moveToGridCoordinates(Point newPosition) {
		if (isUserPositioned) {
			this.gridX = newPosition.x;
			this.gridY = newPosition.y;
			this.realignToGrid();			
		
			edge.notifiyThatNodeHasMoved();
		}
	}

	@Override
	public void moveToPixelCoordinates(int pX, int pY, boolean pDoRealign) {
		if (isUserPositioned) {
			if (pDoRealign) {
				Point gridPosition = grid.getNearestCell(pX, pY);
				this.gridX = gridPosition.x;
				this.gridY = gridPosition.y;
				
				realignToGrid();
			}
			else {
				this.x = pX;
				this.y = pY;			
			}
			
			edge.notifiyThatNodeHasMoved();
		}
	}
	
	/**
	 * This method determines this nodes position by calculating
	 * the center point between its two neighbours.
	 */
	private void determinePositionAccordingToNeighbours() {
		BasicNode left = leftNeighbour;
		BasicNode right = rightNeighbour;
		
		// Search until suitable end and start points are found
		while (!((left instanceof GraphNode) || ((left instanceof PointNode) && (((PointNode)left).isUserPositioned())))) 
			left = ((PointNode)left).getLeftNeighbour();
	
		while (!((right instanceof GraphNode) || ((right instanceof PointNode) && (((PointNode)right).isUserPositioned())))) 
			right = ((PointNode)right).getRightNeighbour();
				
		int distX = Math.round((left.getX() - right.getX()) / 2);
		int distY = Math.round((left.getY() - right.getY()) / 2);
		
		this.x = right.getX() + distX;
		this.y = right.getY() + distY;
	}

	public boolean isUserPositioned() {
		return isUserPositioned;
	}

	public void setUserPositioned(boolean userPositioned) {
		this.isUserPositioned = userPositioned;
		if (userPositioned)
			moveToPixelCoordinates(x, y, true);
		else {
			this.determinePositionAccordingToNeighbours();
			edge.notifiyThatNodeHasMoved();			
		}
	}

	public BasicNode getLeftNeighbour() {
		return leftNeighbour;
	}

	public BasicNode getRightNeighbour() {
		return rightNeighbour;
	}

	/**
	 * Draws the node on the passed graphics object.
	 */
	public void draw(Graphics pGraphics){		
		Rectangle shape = this.getBoundingBox();
		pGraphics.setColor(Color.BLACK);
		pGraphics.fillOval(shape.x , shape.y, shape.width, shape.height);
		pGraphics.setColor(Color.GREEN);
		pGraphics.fillOval(shape.x + 1 , shape.y + 1, shape.width - 2, shape.height - 2);
	}

	@Override
	protected Rectangle getBoundingBox() {
		Rectangle space = new Rectangle(x - PointNode.RADIUS, y - PointNode.RADIUS, 2 * PointNode.RADIUS, 2 * PointNode.RADIUS);
		return space;
	}

	@Override
	public boolean isInside(int pX, int pY) {
			Rectangle shape = getBoundingBox();
			
			double xdist = pX - this.x;
			double ydist = pY - this.y;
	
			double xdist2 = Math.pow(xdist, 2);
			double ydist2 = Math.pow(ydist, 2);
	
			// Note: The correct mathematical calculation if
			// the passed coordinates are within this node require
			// a division by 2 to determine this nodes radius. 
			// As point nodes are very very small, it would be very 
			// hard to hit them with the mouse, so the radius
			// is multiplied by 1.8 in order to enlarge the
			// click area.
			return (Math.sqrt(xdist2 + ydist2) <= 1.8 * (shape.width / 2)); 
	}

	public void notifiyThatNodeOrEdgeHasMoved() {
		if (!isUserPositioned)
			determinePositionAccordingToNeighbours();
	}

	public void setRightNeighbour(BasicNode newNode) {
		this.rightNeighbour = newNode;
	}

	public void setLeftNeighbour(BasicNode newNode) {
		this.leftNeighbour = newNode;
		
	}
	
	@Override
	public void realignToGrid() {
		if ((gridX != -1) && (gridY != -1) && (isUserPositioned)) {
			Point newLocation = grid.getCellCenter(gridX, gridY);
			this.x = newLocation.x;
			this.y = newLocation.y;
		}	
	}
	
	@Override
	public Point getGridPosition() {
		if (isUserPositioned)
			return new Point(this.gridX, this.gridY);
		else
			return new Point(-1, -1);
	}

	@Override
	public String getTooltipText() {
		return edge.getTooltipText();
	}

	public Edge getEdge() {
		return edge;
	}
	
	public boolean isOnlyPointNodeOfEdge() {
		return (edge.getLastPointNode() == this) && (edge.getFirstPointNode() == this);
	}

	@Override
	public String getName() {
		return "EdgePoint on " + edge.getDisplayFriendlyName();
	}

	/*
	@Override
	public String getName() {
		return "Edge from " + edge.getFrom().getName() + " to " + edge.getTo().getName();
	}*/
	

}
