/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package core.history;

import gui.ShowHistoryFrame;
import gui.WorkflowPanel;
import java.io.Serializable;
import java.util.Vector;
import javax.swing.DefaultListModel;
import javax.swing.JList;

/**
 * This class defines the properties of the history.
 * 
 * The history stores a stack of changes done to the workspace or the 
 * elements on it and provides interfaces to manage these changes. 
 * 
 * The last action on top of the history can be undone (and redone). 
 * If a new action is performed after several other action have been 
 * undone, the undone actions can not be redone anymore, as the history 
 * does not branch.
 */

public class History implements Serializable {
	private static final long serialVersionUID = 1L;

	/**
	 * Stores all history actions that have been done.
	 */
	Vector<HistoryObject> storage;

	/**
	 * Stores the current position within the history.
	 * A value of -1 indicates that the history is empty.
	 */
	int curHistoryPosition;

	/**
	 * Reference to the JList used to display this history.
	 * If the reference is set, the JList is updated each time
	 * the state of the history changes.
	 */
	private JList historyList;

	/**
	 * Reference to the workflowPanel. It is used to cause a repaint
	 * of the workflow each a change has been performed by the history.
	 */
	private WorkflowPanel workflowPanel;

	/**
	 * Creates the history.
	 */
	public History(WorkflowPanel workflowPanel, ShowHistoryFrame historyFrame) {
		historyFrame.setHistory(this);
		this.historyList = historyFrame.getHistoryList();
		this.workflowPanel = workflowPanel;
		this.clear();
	}

	/**
	 * This Method adds a new History object to the history.
	 */
	public void addNewAction(HistoryObject action) {
		if (action != null) {
			while (storage.size() - 1 > curHistoryPosition)
				storage.remove(storage.size() - 1);
			storage.add(action);
			curHistoryPosition = storage.size() - 1;	
		}		
	}

	/**
	 * This method performs the last action added to the history.
	 */
	public void performLastAction() {	
		HistoryObject curAction = storage.get(storage.size() - 1);
		//System.out.println("Performing Action: " + curAction.getInformationString());
		curAction.performAction();			
		updateHistoryListAndRepaintGraph();
	}

	/**
	 * Moves the current history position to the specified index by
	 * performing the needed number of un- or redos.
	 */
	public void moveToSelectedIndex(int targetIndex) {
		if ((targetIndex >= 0) && (targetIndex < storage.size())) {

			// Determie which direction to move
			if (targetIndex > curHistoryPosition)
				redo(targetIndex - curHistoryPosition);
			else if (targetIndex < curHistoryPosition) 
				undo(curHistoryPosition - targetIndex);
		}
	}

	/**
	 * Undoes the passed amount of actions starting from the current 
	 * history position. 
	 */
	public void undo(int numberOfSteps) {	
		while ((storage.size() > 0) && (curHistoryPosition != -1) && (numberOfSteps > 0)) {
			HistoryObject lastAction = storage.get(curHistoryPosition);
			HistoryObject undoAction = lastAction.createUndoObject();
			undoAction.performAction();

			// Move undo position
			if (curHistoryPosition >= 0)
				curHistoryPosition--;	

			numberOfSteps--;
		}

		updateHistoryListAndRepaintGraph();
	}

	/**
	 * Redoes the passed amount of actions starting from the current 
	 * history position. 
	 */
	public void redo(int numberOfSteps) {
		while ((storage.size() > 0) && (curHistoryPosition < storage.size() - 1) && (numberOfSteps > 0)) {

			HistoryObject lastAction = storage.get(curHistoryPosition + 1);
			lastAction.performAction();

			curHistoryPosition++;		
			numberOfSteps--;
		}				
		updateHistoryListAndRepaintGraph();
	}

	/**
	 * Writes the content of the history in the historyList (if present)
	 * and repaints the workflow.
	 */
	public void updateHistoryListAndRepaintGraph() {
		DefaultListModel historyListModel = (DefaultListModel) historyList.getModel();
		historyListModel.removeAllElements();

		int markedIndex = curHistoryPosition;

		for (int curIndex = 0; curIndex < storage.size(); curIndex++) {
			HistoryObject curObject = storage.get(curIndex); 
			String curElement = curObject.getInformationString();
			if (curIndex == markedIndex)
				curElement += " <--\n";
			else
				curElement += "\n";

			historyListModel.addElement(curElement);
		}

		historyList.setSelectedIndex(markedIndex);			

		workflowPanel.repaint();
	}

	public void clear() {
		storage = new Vector<HistoryObject>();
		curHistoryPosition = -1; 
		updateHistoryListAndRepaintGraph();
	}


}
