/*
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
package core.storage;

import java.io.File;
import java.io.IOException;
import java.util.Vector;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * This class provides basic interface to access a xml file
 * via dom and xpath. 
 */
public class BasicXMLReader {
	/**
	 * Memory representation of the xml file of this xml reader.
	 */
	protected Document document;

	/**
	 * Reference to the xpath object used to adress nodes within this
	 * readers document.
	 */
	private XPath xpath;

	public BasicXMLReader(String filename) throws XPathExpressionException, ParserConfigurationException, SAXException, IOException {
		openFile(filename);
	}
	
	/**
	 * This method opens the passed xml file and reads its content into
	 * the document field.
	 */
	protected void openFile(String filename)
			throws ParserConfigurationException, SAXException, IOException,
			XPathExpressionException {
		File file = new File(filename);
		if (file.exists()) {
			DocumentBuilderFactory domFactory = DocumentBuilderFactory.newInstance();
			domFactory.setNamespaceAware(true); 
			DocumentBuilder builder = domFactory.newDocumentBuilder();
			document = builder.parse(filename);

			XPathFactory xpathFactory = XPathFactory.newInstance();
			xpath = xpathFactory.newXPath();
		} else 
			throw new IOException("File " + file + " does not exist.");
	}

	/**
	 * This method returns a node list containing the result of the execution of
	 * the passed XPath expression string. 
	 */
	public NodeList getNodeListOfXPathExpression(Node root, String xpathString) {
		try {
			XPathExpression xpathExpression = xpath.compile(xpathString);
			NodeList result = (NodeList)xpathExpression.evaluate(root, XPathConstants.NODESET);
			return result;
		} catch (XPathExpressionException e) {
			System.out.println("Error while reading xml file. Please check if input file is valid.\nUsed XPath: " + xpathString);
			return null;
		}
	}

	/**
	 * This method returns a string containing the result of the execution of
	 * the passed XPath expression string. 
	 */
	public String getStringOfXPathExpression(Node root, String xpathString) {
		try {
			XPathExpression xpathExpression = xpath.compile(xpathString);
			String result = (String)xpathExpression.evaluate(root, XPathConstants.STRING);
			return result;
		} catch (XPathExpressionException e) {
			System.out.println("Error while reading xml file. Please check if input file is valid.\nUsed XPath: " + xpathString);
			return null;
		}
	}

	/**
	 * This method collects all child nodes (of type text) of the nodes specified by 
	 * the passed xpathString. Afterwards it reads their values, stores them in 
	 * vector and returns it.
	 * Please note:
	 * 		The term child node can be misleading. In the following example xml:
	 * 		<foo>
	 * 			<bar>
	 * 				Test
	 * 			</bar>
	 * 		</foo>
	 * 		bar is obviously the child node of foo. But according to the specification
	 * 		the atomic value "Test" is also a node and child of bar. Therefore, the 
	 * 		child node of the node specified by the xpath "/foo/bar" is the text node 
	 * 		"Test".
	 */
	public Vector<String> getValuesOfAllTextNodesBelowXPathNodes(Node root,
			String xpathString) {
		Vector<String> result = new Vector<String>();
		NodeList nodes = getNodeListOfXPathExpression(root, xpathString + "/child::text()");
		for (int i = 0; i < nodes.getLength(); i++) {
			result.add(nodes.item(i).getNodeValue()); 
		}
		return result;	
	}

	/**
	 * This method collects the first child node (of type text) of the nodes specified by 
	 * the passed xpathString and returns them as a string.
	 * Please note:
	 * 		The term child node can be misleading. In the following example xml:
	 * 		<foo>
	 * 			<bar>
	 * 				Test
	 * 			</bar>
	 * 		</foo>
	 * 		bar is obviously the child node of foo. But according to the specification
	 * 		the atomic value "Test" is also a node and child of bar. Therefore, the 
	 * 		child node of the node specified by the xpath "/foo/bar" is the text node 
	 * 		"Test".
	 */
	public String getValuesOfFirstTextNodeBelowXPathNodes(Node root,
			String xpathString) {
		return getStringOfXPathExpression(root, xpathString + "/child::text()[1]");
	}

}