/*
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
package core.storage;

import helper.ConverterSignature;
import helper.SmallHelpers;

import java.io.File;
import java.io.IOException;
import java.util.Dictionary;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Vector;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathExpressionException;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import core.BooleanParameterGroup;
import core.DynamicTextParameter;
import core.FileNameParameterGroup;
import core.ParameterGroup;
import core.SelectParameterGroup;
import core.StaticTextParameter;
import core.TextParameter;
import core.TextParameterGroup;

import sun.security.action.GetLongAction;

/**
 * This xml reader is used to read the applications configuration file
 * which specifies all possible jobs of a workflow and furhter settings
 * of the gui.
 */
public class ConfigurationXMLReader extends BasicXMLReader {	
	/**
	 * Reference to a node list storing all <program> nodes of the config 
	 * file. The position within this list corresponds to the program id.
	 */
	NodeList programXMLNodes;

	/**
	 * Reference to a node list storing all <converter> nodes of type "input"
	 * within the config file. The position within this list corresponds to 
	 * the program id.
	 */
	NodeList inputConverterXMLNodes;

	/**
	 * Reference to a node list storing all <converter> nodes of type "output"
	 * within the config file. The position within this list corresponds to 
	 * the program id.
	 */
	NodeList outputConverterXMLNodes;

	@Override
	protected void openFile(String filename) throws XPathExpressionException, ParserConfigurationException, SAXException, IOException {
		super.openFile(filename);
		programXMLNodes = getNodeListOfXPathExpression(document, "/settings/software/program");
		inputConverterXMLNodes = getNodeListOfXPathExpression(document, "/settings/tools/converter[@type='input']");
		outputConverterXMLNodes = getNodeListOfXPathExpression(document, "/settings/tools/converter[@type='output']");		
	}

	public ConfigurationXMLReader(String filename) throws XPathExpressionException, ParserConfigurationException, SAXException, IOException {
		super(filename);
	}

	/** 
	 * This method returns the names of all the programs in the xml file..
	 */
	public Vector<String> getProgrammNames() {
		return getValuesOfAllTextNodesBelowXPathNodes(document, "/settings/software/program/general[1]/name[1]");
	}

	/**
	 * This method returns the id of the first job matching the passed name.
	 */
	public int getProgrammIDByName(String name) {
		Vector<String> names = getProgrammNames();
		for (int curProgramIndex = 0; curProgramIndex < names.size(); curProgramIndex++)
			if (names.get(curProgramIndex).equals(name))
				return curProgramIndex;
		return -1;
	}

	/**
	 * This method returns a dictionary containing the signatures (Input Type, Input Extension, 
	 * Output Type, Output Extension) of any matching input converter using the 
	 * converter ids as keys. An input converter is included in the dictionary if it outputs 
	 * a type listed in the additionally passed vector. If null or an empty vector is passed, 
	 * all input converters are included in the dictionary. 
	 */
	public HashMap<Integer, ConverterSignature> getInputConverterSignatures(Vector<String> outputTypeFilter) {
		HashMap<Integer, ConverterSignature> result = new HashMap<Integer, ConverterSignature>();

		for (int id = 0; id < inputConverterXMLNodes.getLength(); id++) {
			String curOutputType = this.getInputConverterOutputTypeByID(id);
			if (isInVector(curOutputType, outputTypeFilter)) {
				ConverterSignature curSignatrue = new ConverterSignature(id, this.getInputConverterInputTypeByID(id), 
																			 this.getInputConverterInputSuffixByID(id),
																			 curOutputType,
																			 this.getInputConverterOutputSuffixByID(id));
				result.put(id, curSignatrue);			
			}
		}

		return result;
	}

	/**
	 * Determines if the passed string is an element of the passed vector.
	 */
	private boolean isInVector(String string,
			Vector<String> vector) {
		if ((vector == null) || (vector.size() == 0)) {
			return true;
		} else {
			boolean found = false;
			for (String curElement : vector) {
				if (curElement.equals(string)) {
					found = true;
					break;
				}
			}
			return found;
		}
	}

	/**
	 * This method returns a dictionary containing the signatures (Input Type, Input Extension, 
	 * Output Type, Output Extension) of any matching output converter using the 
	 * converter ids as keys. An output converter is included in the dictionary if it accepts 
	 * a type listed in the additionally passed vector. If null or an empty vector is passed, 
	 * all output converters are included in the dictionary. 
	 */
	public HashMap<Integer, ConverterSignature> getOutputConverterSignatures(Vector<String> inputTypeFilter) {
		HashMap<Integer, ConverterSignature> result = new HashMap<Integer, ConverterSignature>();

		for (int id = 0; id < outputConverterXMLNodes.getLength(); id++) {
			String curInputType = this.getOutputConverterInputTypeByID(id);
			if (isInVector(curInputType, inputTypeFilter)) {
				ConverterSignature curSignatrue = new ConverterSignature(id, curInputType, 
																			 this.getOutputConverterInputSuffixByID(id),
																			 this.getOutputConverterOutputTypeByID(id),
																			 this.getOutputConverterOutputSuffixByID(id));
				result.put(id, curSignatrue);
				
			}
		}

		return result;
	}


	/**
	 * This method determines the id of the first output converter excepting the
	 * passed type as its input.
	 */
	public int getOutputConverterIDByOutputType(String type) {
		Vector<String> types = getValuesOfAllTextNodesBelowXPathNodes(document, "/settings/tools/converter[@type='output']/output[1]/type[1]");
		for (int curConverterIndex = 0; curConverterIndex < types.size(); curConverterIndex++)
			if (types.get(curConverterIndex).equals(type))
				return curConverterIndex;
		return -1;
	}

	/**
	 * This method determines the id of the first input converter excepting the
	 * passed type as its input.
	 */
	public int getInputConverterIDByInputType(String type) {
		Vector<String> types = getValuesOfAllTextNodesBelowXPathNodes(document, "/settings/tools/converter[@type='input']/input[1]/type[1]");
		for (int curConverterIndex = 0; curConverterIndex < types.size(); curConverterIndex++)
			if (types.get(curConverterIndex).equals(type))
				return curConverterIndex;
		return -1;
	}


	/**
	 * This method returns the requirements of the specified program.
	 */
	public Vector<String> getProgrammRequirementsByID(int id) {
		Vector<String> allRequirements = getValuesOfAllTextNodesBelowXPathNodes(programXMLNodes.item(id), "general/requires");
		allRequirements.addAll(getValuesOfAllTextNodesBelowXPathNodes(document, "/settings/software/requires"));
		return flattenRequirements(allRequirements);
	}

	/**
	 * This method returns the requirements of the specified input converter.
	 */
	public Vector<String> getInputConverterRequirementsByID(int id) {
		Vector<String> allRequirements = getValuesOfAllTextNodesBelowXPathNodes(inputConverterXMLNodes.item(id), "requires");
		allRequirements.addAll(getValuesOfAllTextNodesBelowXPathNodes(document, "/settings/tools/requires"));
		return flattenRequirements(allRequirements);
	}

	/**
	 * This method returns the requirements of the specified output converter.
	 */
	public Vector<String> getOutputConverterRequirementsByID(int id) {
		Vector<String> allRequirements = getValuesOfAllTextNodesBelowXPathNodes(outputConverterXMLNodes.item(id), "requires");
		allRequirements.addAll(getValuesOfAllTextNodesBelowXPathNodes(document, "/settings/tools/requires"));

		return flattenRequirements(allRequirements);
	}

	/**
	 * This methods flattens the passed vector of string into one vector
	 * of string containing each element only once.
	 */
	public Vector<String> flattenRequirements(Vector<String> requirements) {
		Vector<String> result = new Vector<String>();
		for (String curLine : requirements) {
			String[] curLineRequirements = curLine.split(",");
			for (String curRequirement : curLineRequirements) {
				String curRequirementTrimmed = curRequirement.trim();
				// Check if already existing
				boolean found = false;
				for (String curResultRequirement : result) {
					if (curRequirementTrimmed.equals(curResultRequirement)) {
						found = true;
						break;
					}
				}
				if (!found)
					result.add(curRequirementTrimmed);
			}
		}

		return result;
	}


	/**
	 * This method returns the name of the program identified by the 
	 * passed id.
	 */
	public String getProgrammNameByID(int id) {
		return getValuesOfFirstTextNodeBelowXPathNodes(programXMLNodes.item(id), "general[1]/name[1]");
	}

	/**
	 * This method returns the output types of the specified program.
	 */
	public Vector<String> getProgramOutputTypesByID(int id) {
		return getValuesOfAllTextNodesBelowXPathNodes(programXMLNodes.item(id), "output/type");
	}

	/**
	 * This method returns a vector of string containing all input types 
	 * the program specified by the passed id accepts.
	 */
	public Vector<String> getProgramInputTypesByID(int id) {
		return getValuesOfAllTextNodesBelowXPathNodes(programXMLNodes.item(id), "input/type[1]");
	}	

	/**
	 * This method returns the input parameter for the specified file type of 
	 * the program specified by the passed id. 
	 */
	public String getProgrammInputParameterByFileType(int id, String type) {
		return getValuesOfFirstTextNodeBelowXPathNodes(programXMLNodes.item(id), "input[type='" +  type + "'][1]/parameter[1]");		
	}

	/**
	 * This method returns the converter binary that should be run on inputs
	 * of the specified file type before the program specified by the passed id
	 * can be executed.
	 */	
	public String getProgrammInputConverterBinaryByFileType(int id, String type) {
		return getValuesOfFirstTextNodeBelowXPathNodes(programXMLNodes.item(id), "input[type='" +  type + "'][1]/converter[1]");
	}

	/**
	 * This method returns the converter binary that will be run on inputs of 
	 * the program specified by the passed id.
	 */	
	public String getProgrammOutputConverterBinaryByProgrammID(int id) {
		return getValuesOfFirstTextNodeBelowXPathNodes(programXMLNodes.item(id), "output[1]/converter[1]");
	}

	/**
	 * This method determines the name of the binary (only the name, not the path)
	 * of the program specified by the passed id.
	 */
	public String getProgrammBinaryNameByID(int id) {
		File binary = new File(getProgrammBinaryByID(id));
		return binary.getName();
	}

	/**
	 * This method determines the path of the binary (only the path, not the name)
	 * of the program specified by the passed id.
	 */
	public String getProgrammBinaryPathByID(int id) {
		String name = getProgrammBinaryByID(id);
		return name.substring(0, name.lastIndexOf("/") + 1);
	}

	/**
	 * This method determines the path of the binary of the program specified 
	 * by the passed id.
	 */
	private String getProgrammBinaryByID(int id) {
		return getValuesOfFirstTextNodeBelowXPathNodes(programXMLNodes.item(id), "general[1]/binary[1]");
	}

	/**
	 * This method returns the name of the binary of the input parameter
	 * by the passed id.
	 */
	public String getInputConverterBinaryByID(int id) {
		return getValuesOfFirstTextNodeBelowXPathNodes(inputConverterXMLNodes.item(id), "binary[1]");
	}

	/**
	 * This method returns the name of the binary of the input parameter
	 * by the passed id.
	 */
	public String getOutputConverterBinaryByID(int id) {
		return getValuesOfFirstTextNodeBelowXPathNodes(outputConverterXMLNodes.item(id), "binary[1]");
	}

	/**
	 * This method return the input type of the output converter specified
	 * by the passed id. 
	 */
	public String getOutputConverterInputTypeByID(int id) {
		return getValuesOfFirstTextNodeBelowXPathNodes(outputConverterXMLNodes.item(id), "input[1]/type[1]");
	}

	private String getOutputConverterInputSuffixByID(int id) {
		return getValuesOfFirstTextNodeBelowXPathNodes(outputConverterXMLNodes.item(id), "input[1]/suffix[1]");
	}

	/**
	 * This method return the output type of the output converter specified
	 * by the passed id. 
	 */
	public String getOutputConverterOutputTypeByID(int id) {
		return getValuesOfFirstTextNodeBelowXPathNodes(outputConverterXMLNodes.item(id), "output[1]/type[1]");
	}

	public String getOutputConverterOutputSuffixByID(int id) {
		return getValuesOfFirstTextNodeBelowXPathNodes(outputConverterXMLNodes.item(id), "output[1]/suffix[1]");
	}

	/**
	 * This method return the output type of the input converter specified
	 * by the passed id. 
	 */
	public String getInputConverterOutputTypeByID(int id) {
		return getValuesOfFirstTextNodeBelowXPathNodes(inputConverterXMLNodes.item(id), "output[1]/type[1]");
	}

	public String getInputConverterOutputSuffixByID(int id) {
		return getValuesOfFirstTextNodeBelowXPathNodes(inputConverterXMLNodes.item(id), "output[1]/suffix[1]");
	}

	/**
	 * This method return the input type of the input converter specified
	 * by the passed id. 
	 */
	public String getInputConverterInputTypeByID(int id) {
		return getValuesOfFirstTextNodeBelowXPathNodes(inputConverterXMLNodes.item(id), "input[1]/type[1]");
	}

	public String getInputConverterInputSuffixByID(int id) {
		return getValuesOfFirstTextNodeBelowXPathNodes(inputConverterXMLNodes.item(id), "input[1]/suffix[1]");
	}

	/**
	 * This method reads out the configuration value specified by the 
	 * passed tag.
	 */
	public Vector<String> getConfigValue(String tag) {
		return getValuesOfAllTextNodesBelowXPathNodes(document, "/settings[1]/config[1]/" + tag);
	}	

	/**
	 * This method returns the name of the parameter the program specified by
	 * the passed id expects its output file name in.
	 */
	public String getProgrammOutputParameterByID(int id) {
		return getValuesOfFirstTextNodeBelowXPathNodes(programXMLNodes.item(id), "output[1]/parameter[1]");
	}

	/**
	 * This method generates and returns all parameter group objects for the
	 * job specified by the passed id.
	 */
	public Vector<ParameterGroup> generateProgramParameterGroups(int id) {
		Vector<ParameterGroup> result = new Vector<ParameterGroup>();

		NodeList programParameterGroups = getNodeListOfXPathExpression(programXMLNodes.item(id), "parameterlist/parametergroup");
		for (int curParameterGroupPos = 0; curParameterGroupPos < programParameterGroups.getLength(); curParameterGroupPos++) {
			result.add(generateParameterGroup(programParameterGroups.item(curParameterGroupPos)));
		}
		createParameterGroupReferences(result);

		return result;
	}

	/**
	 * This method generates and returns the text parameter represented by the
	 * passed node.
	 */
	private TextParameter generateTextParameter(Node curNode) {
		String type = getStringOfXPathExpression(curNode, "@type[1]");
		String name = getValuesOfFirstTextNodeBelowXPathNodes(curNode, "name[1]");
		String description = getValuesOfFirstTextNodeBelowXPathNodes(curNode, "description[1]");
		String successor = getValuesOfFirstTextNodeBelowXPathNodes(curNode, "successor[1]");

		if (type.equals("dynamic")) {
			String dynamicpart = getValuesOfFirstTextNodeBelowXPathNodes(curNode, "dynamicpart[1]");
			String dynamicref = getValuesOfFirstTextNodeBelowXPathNodes(curNode, "dynamicref[1]");
			return new DynamicTextParameter(name, description, successor, dynamicpart, dynamicref);
		} else //if (type.equals("static")) {
			return new StaticTextParameter(name, description, successor);
	}

	/**
	 * This method generates and returns the parameter group represented by the
	 * passed node.
	 */
	private ParameterGroup generateParameterGroup(Node curNode) {
		ParameterGroup result = null;

		// Read values from xml
		String successor = getValuesOfFirstTextNodeBelowXPathNodes(curNode, "successor[1]");
		String name = getValuesOfFirstTextNodeBelowXPathNodes(curNode, "name[1]");
		String description = getValuesOfFirstTextNodeBelowXPathNodes(curNode, "description[1]");
		String key = getValuesOfFirstTextNodeBelowXPathNodes(curNode, "key[1]");
		boolean isVisible = getStringOfXPathExpression(curNode, "@visible[1]").equals("true");
		String type = getStringOfXPathExpression(curNode, "@type[1]");

		//bool
		if (type.equals("bool")) {
			boolean defaultValue = getValuesOfFirstTextNodeBelowXPathNodes(curNode, "default").equals("1");
			result = new BooleanParameterGroup(name, key, defaultValue, description, successor, isVisible);
		} else {
			boolean isDisabled = (getStringOfXPathExpression(curNode, "@disabled[1]").equals("true"));
			//filename
			if (type.equals("filename")) {
				boolean isMulti = (getStringOfXPathExpression(curNode, "@multi[1]").equals("true"));
				result = new FileNameParameterGroup(name, key, description, successor, isVisible, isMulti);				
			} 
			//select 
			else if (type.equals("select")){
				Vector<String> options = getValuesOfAllTextNodesBelowXPathNodes(curNode, "values/value");
				result = new SelectParameterGroup(name, key, description, successor, isVisible,!isDisabled, options);
			}
			//text
			else {
				Vector<TextParameter> parameters = new Vector<TextParameter>();
				NodeList parameterNodes = this.getNodeListOfXPathExpression(curNode, "parameter");
				for (int curParameterPos = 0; curParameterPos < parameterNodes.getLength(); curParameterPos++) {
					Node curParameterNode = parameterNodes.item(curParameterPos);
					parameters.add(generateTextParameter(curParameterNode));			
				}
				String defaultValue = getValuesOfFirstTextNodeBelowXPathNodes(curNode, "default");
				result = new TextParameterGroup(name, key, description, successor, isVisible, !isDisabled, parameters);

				((TextParameterGroup) result).createReferences();
				((TextParameterGroup) result).setDefaultValue(defaultValue);			

			}

			result.setEnable(!isDisabled);
		}
		return result;
	}	

	/**
	 * This method sets the successor reference of the passed parameter groups
	 * according to the textual value of the corresponding field.
	 */
	private void createParameterGroupReferences(Vector<ParameterGroup> groups) {
		for (ParameterGroup curParameterGroup : groups) {
			ParameterGroup curSuccessor = SmallHelpers.findParameterGroupByName(curParameterGroup.getSuccessorName(), groups);
			curParameterGroup.setSuccessor(curSuccessor);
		}

	}

	public Vector<String> getRequirements(Vector<Integer> usedProgrammIDs, Vector<Integer> usedInputConverterIDs, Vector<Integer> usedOutputConverterIDs) {
		Vector<String> result = new Vector<String>();
		for (int curID : usedProgrammIDs)
			result.addAll(this.getProgrammRequirementsByID(curID));
		for (int curID : usedInputConverterIDs)
			result.addAll(this.getInputConverterRequirementsByID(curID));
		for (int curID : usedOutputConverterIDs)
			result.addAll(this.getOutputConverterRequirementsByID(curID));

		return this.flattenRequirements(result);
	}




}
