/*
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package core.storage;
import gui.helper.DialogManager;
import gui.helper.Grid;

import gui.WorkflowPanel;
import helper.SmallHelpers;

import java.awt.Point;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Vector;
import java.util.HashMap;

import javax.swing.JOptionPane;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathExpressionException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import core.BooleanParameterGroup;
import core.FileNameParameterGroup;
import core.ParameterGroup;
import core.SelectParameterGroup;
import core.TextParameterGroup;
import core.TypedFile;
import core.graph.Edge;
import core.graph.Graph;
import core.graph.InputFileNode;
import core.graph.JobNode;
import core.graph.OutputFileNode;
import core.graph.PointNode;
import core.graph.GraphNode;
import core.history.ChangeTextParameterGroupValuesHistoryObject;
import core.history.HistoryObject;
import core.history.ChangeSelectParameterGroupValuesHistoryObject;

/**
 * This class provides methods to load a previously stored 
 * workflow in the specified xml format. 
 * 
 * The menu item "Load Workflow" allows to load a workflow 
 * description from a file in XML Format.
 * 
 *
 */

public class GraphLoader extends BasicXMLReader {
	private WorkflowPanel workflowPanel;
	private ConfigurationXMLReader configurationXmlReader;
	private NodeList nodeXMLNodes;
	private NodeList edgeXMLNodes;

	public GraphLoader(String filename, ConfigurationXMLReader configurationXmlReader, WorkflowPanel workflowPanel) throws ParserConfigurationException, SAXException, IOException, XPathExpressionException {	
		super(filename);		
		this.workflowPanel = workflowPanel;
		this.configurationXmlReader = configurationXmlReader;
	}

	@Override
	protected void openFile(String filename) throws XPathExpressionException, ParserConfigurationException, SAXException, IOException {
		super.openFile(filename);

		nodeXMLNodes = getNodeListOfXPathExpression(document, "/workflow/nodes/node");
		edgeXMLNodes = getNodeListOfXPathExpression(document, "/workflow/edges/edge");
	}

	/**
	 * This method returns the name of the workflow. 
	 */
	public String getWorkflowName(){
		return getValuesOfFirstTextNodeBelowXPathNodes(document, "/workflow[1]/general[1]/name[1]");
	}

	/**
	 * This method returns the value of the folder.
	 */
	public String getWorkflowFolder(){
		return getValuesOfFirstTextNodeBelowXPathNodes(document, "/workflow[1]/general[1]/folder[1]");
	}

	/**
	 * This method returns the folder where the script should be
	 * exported.
	 */
	public String getExportFolder(){
		return getValuesOfFirstTextNodeBelowXPathNodes(document, "/workflow[1]/general[1]/path[1]");		
	}

	/**
	 * This method loads the graph from this readers xml file.
	 * @param checkFilesOnLoad 
	 */
	public Graph loadGraph(boolean checkFilesOnLoad, String currentWorkingDir) {
		// Create a temporary grid for backwards compatibility. As older version of profido only store
		// the pixel coordinates of a node, it is necessary to re-calculate their grid coordinates using a 
		// grid matching the default grids cell size.
		Grid oldGrid = new Grid(1, 1, 100, 50, false);
		oldGrid.setSize(2000, 2000);

		// Create graph
		String newWorkflowFolder = currentWorkingDir;
		if (!this.getWorkflowFolder().trim().equals(""))
			newWorkflowFolder = this.getWorkflowFolder().trim();
		
		Graph graph = new Graph(this.getWorkflowName().trim(), newWorkflowFolder, this.getExportFolder().trim(), configurationXmlReader, workflowPanel, workflowPanel.getShowHistoryFrame());

		// Store nodes and their corresponding IDs in has map
		HashMap<Integer, GraphNode> idToNodeMap = new HashMap<Integer, GraphNode>();

		// Load nodes
		for (int i = 0; i < nodeXMLNodes.getLength(); i++) {
			int nodeID = Integer.valueOf((getValuesOfFirstTextNodeBelowXPathNodes(nodeXMLNodes.item(i), "id[1]")).trim());
			try {
				GraphNode curNode = this.createAbstractNode(nodeXMLNodes.item(i), graph, oldGrid, checkFilesOnLoad);
				idToNodeMap.put(nodeID, curNode);
				graph.addNode(curNode);
			} catch (NoJobException e) {
				JOptionPane.showMessageDialog(workflowPanel, e.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
			}
		}

		// Load edges
		for (int i = 0; i < edgeXMLNodes.getLength(); i++) {
			try {
				Edge edge = this.createEdge(edgeXMLNodes.item(i), idToNodeMap, oldGrid);
				graph.addEdge(edge);

			} catch (NoNodeException e) {
				JOptionPane.showMessageDialog(workflowPanel, e.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
			} 
		}

		return graph;
	}

	/**
	 * This method loads the job node from the passed xml node and 
	 * returns it.
	 * @param checkFilesOnLoad 
	 */
	private GraphNode createAbstractNode(Node xmlNode, Graph graph, Grid oldGrid, boolean checkFilesOnLoad) throws NoJobException {
		GraphNode resultNode = null;
		// Read basic informations
		String name = getValuesOfFirstTextNodeBelowXPathNodes(xmlNode, "name[1]").trim();
		String coordinates = getValuesOfFirstTextNodeBelowXPathNodes(xmlNode, "coords[1]").trim();
		String gridCoordinates = getValuesOfFirstTextNodeBelowXPathNodes(xmlNode, "gridcoords[1]").trim();

		// Read node specific informations		
		if (this.getStringOfXPathExpression(xmlNode, "count(input[1])").equals("1")) {
			// input nodes
			String filename = getValuesOfFirstTextNodeBelowXPathNodes(xmlNode, "input[1]/file[1]/name[1]").trim();
			String filepath = normalizePath(getValuesOfFirstTextNodeBelowXPathNodes(xmlNode, "input[1]/file[1]/path[1]").trim());
			File fileWithoutType = new File(filepath + filename);
			
			String filetype = getValuesOfFirstTextNodeBelowXPathNodes(xmlNode, "input[1]/file[1]/type[1]").trim();
			// TODO Explicitly store and load converter id
			int converterID = configurationXmlReader.getInputConverterIDByInputType(filetype);

			// Check if file exists and allow user to relocate it
			if ((checkFilesOnLoad) && (!SmallHelpers.fileReadable(fileWithoutType))) {
				int dialogResult = JOptionPane.showConfirmDialog(workflowPanel, "The loaded workflow uses the file " +  fileWithoutType.getPath() + "\nas input file. It does not exist or you don't have the rights to access it.\nWould you like to select another file?", 
						"Warning", JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE,null);
				if (dialogResult == JOptionPane.YES_OPTION) {
					File alternateFile = DialogManager.showOpenDialog(filepath, filename, ".*", workflowPanel);
					if (alternateFile != null)
						fileWithoutType = alternateFile;
				}
			}
			resultNode = new InputFileNode(-1, -1, configurationXmlReader, workflowPanel.getGrid(), converterID, graph, new TypedFile(fileWithoutType.getPath(), filetype));
			resultNode.setName(name);
		} else if (this.getStringOfXPathExpression(xmlNode, "count(output[1])").equals("1")) {
			// output nodes
			String filename = getValuesOfFirstTextNodeBelowXPathNodes(xmlNode, "output[1]/file[1]/name[1]").trim();
			String filepath = normalizePath(getValuesOfFirstTextNodeBelowXPathNodes(xmlNode, "output[1]/file[1]/path[1]").trim());
			File fileWithoutType = new File(filepath + filename);
			
			String filetype = getValuesOfFirstTextNodeBelowXPathNodes(xmlNode, "output[1]/file[1]/type[1]").trim();
			// TODO Explicitly store and load converter id
			int converterID = configurationXmlReader.getOutputConverterIDByOutputType(filetype);
		
			// Check if file exists and allow user to relocate it
			if ((checkFilesOnLoad) && (!SmallHelpers.fileWriteable(fileWithoutType))) {
				int dialogResult = JOptionPane.showConfirmDialog(workflowPanel, "The loaded workflow uses the file " +  fileWithoutType.getPath() + "\nas output file. You don't have the rights to access the specified file or the directory it is located in does not exist.\nWould you like to select another file?", 
						"Warning - " + fileWithoutType.getPath(), JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE,null);
				if (dialogResult == JOptionPane.YES_OPTION) {
					File alternateFile = DialogManager.showSaveDialog(filepath, filename, ".*", workflowPanel, false);
					if (alternateFile != null)
						fileWithoutType = alternateFile;
				}
			}
			resultNode = new OutputFileNode(-1, -1, configurationXmlReader, workflowPanel.getGrid(), converterID, graph, new TypedFile(fileWithoutType.getPath(), filetype));
			resultNode.setName(name);
		} else if (this.getStringOfXPathExpression(xmlNode, "count(job[1])").equals("1")) {
			// job node

			// find ID of job
			int jobID = configurationXmlReader.getProgrammIDByName(getValuesOfFirstTextNodeBelowXPathNodes(xmlNode, "job[1]/name[1]").trim());
			if (jobID == -1) 
				throw new NoJobException(jobID);

			resultNode = new JobNode(-1, -1, configurationXmlReader, workflowPanel.getGrid(), jobID, graph);
			resultNode.setName(name);
			// read parameters
			String missingParameters = "";
			NodeList parameterNodes = getNodeListOfXPathExpression(xmlNode, "job[1]/parameterlist/parameter");
			for (int curParameterNodePos = 0; curParameterNodePos < parameterNodes.getLength(); curParameterNodePos++) {
				missingParameters = missingParameters + readParameterValues(parameterNodes.item(curParameterNodePos), (JobNode)resultNode);
			}
			if (!missingParameters.equals(""))
				JOptionPane.showMessageDialog(workflowPanel, "The following parameters could not be found: \n" + missingParameters, "Error", JOptionPane.ERROR_MESSAGE);
		}

		// Read and apply coordinates
		Point gridPosition = null;
		if (gridCoordinates.equals("")) {
			// If no grid coordinates are found, calculate grid coordinates from pixel coordinates
			// using the default sized grid
			int xCoord = Integer.valueOf(coordinates.split(" ")[0]);
			int yCoord = Integer.valueOf(coordinates.split(" ")[1]);
			gridPosition = oldGrid.getNearestCell(xCoord, yCoord);
		} else {
			gridPosition = new Point(Integer.valueOf(gridCoordinates.split(" ")[0]), Integer.valueOf(gridCoordinates.split(" ")[1]));			
		}
		resultNode.moveToGridCoordinates(gridPosition);

		
		return resultNode;
	}

	/**
	 * This method reads the values of one parameter group stored at the
	 * passed xmlNode and applies their values to the corresponding parameter
	 * group of the passed jobNode. If the corresponding parameter group cannot
	 * be found the name of the parameter group is returned to enable more
	 * specific error messages.
	 */
	private String readParameterValues(Node xmlNode, JobNode jobNode) {
		String missingGroups = "";
		Vector<ParameterGroup> parameterGroups = jobNode.getParameterGroups();

		String paramName = getValuesOfFirstTextNodeBelowXPathNodes(xmlNode, "name[1]").trim();
		String paramValue = getValuesOfFirstTextNodeBelowXPathNodes(xmlNode, "value[1]").trim();
		Boolean paramVisible = getValuesOfFirstTextNodeBelowXPathNodes(xmlNode, "visible[1]").trim().equalsIgnoreCase("true");
		Boolean paramEnabled = getValuesOfFirstTextNodeBelowXPathNodes(xmlNode, "enable[1]").trim().equalsIgnoreCase("true");

		ParameterGroup curGroup = SmallHelpers.findParameterGroupByName(paramName, parameterGroups);
		if (curGroup != null) {
			curGroup.setVisible(paramVisible);
			curGroup.setEnable(paramEnabled);

			if (curGroup instanceof BooleanParameterGroup) {
				// Nothing to do (Value has already been read from enable-Tag)
			} else if (curGroup instanceof FileNameParameterGroup) {
				// Nothing to do, as file names are specified by edges
			} else if (curGroup instanceof SelectParameterGroup) {
				((SelectParameterGroup)curGroup).setChoiceValue(paramValue);
			} else if (curGroup instanceof TextParameterGroup) {
				// Parse to temporary value
				((TextParameterGroup)curGroup).parseInputToTemporaryValues(paramValue);

				// Apply temporary values
				ChangeTextParameterGroupValuesHistoryObject historyObject = new ChangeTextParameterGroupValuesHistoryObject("", ((TextParameterGroup)curGroup), ((TextParameterGroup)curGroup).createHistoryObjectsToAdoptNewValues());
				historyObject.performAction();
			}
		} else 
			missingGroups = missingGroups + paramName + "\n";

		return missingGroups;
	}


	/**
	 * This method loads the edge from the passed xml node and 
	 * returns it.
	 */
	private Edge createEdge(Node edgeXmlNode, HashMap<Integer, GraphNode> idToNodeMap, Grid oldGrid) throws NoNodeException {
		// Load references to nodes that are connected by edges
		int fromId = Integer.valueOf((getValuesOfFirstTextNodeBelowXPathNodes(edgeXmlNode, "from[1]")).trim());			
		int toId = Integer.valueOf((getValuesOfFirstTextNodeBelowXPathNodes(edgeXmlNode,"to[1]")).trim());	
		GraphNode fromNode = idToNodeMap.get(fromId);
		GraphNode toNode = idToNodeMap.get(toId);			
		if (fromNode == null) 
			throw new NoNodeException(fromId);		
		if (toNode == null)
			throw new NoNodeException(toId);

		// Load or generate order key
		String orderkey = getValuesOfFirstTextNodeBelowXPathNodes(edgeXmlNode, "orderkey[1]").trim();
		if (orderkey.equals(""))
			orderkey = toNode.getNextFreeOrderKey();
		
		// Load file informations
		String filename = getValuesOfFirstTextNodeBelowXPathNodes(edgeXmlNode, "file[1]/name[1]").trim();
		String filepath = normalizePath(getValuesOfFirstTextNodeBelowXPathNodes(edgeXmlNode, "file[1]/path[1]").trim());
		String filetype = getValuesOfFirstTextNodeBelowXPathNodes(edgeXmlNode, "file[1]/type[1]").trim();

		// Load visiblty
		boolean fileNameVisible = getStringOfXPathExpression(edgeXmlNode, "file[1]/name[1]/@visible[1]").equals("true");	// Defaults to false
		boolean fileTypeVisible = !getStringOfXPathExpression(edgeXmlNode, "file[1]/type[1]/@visible[1]").equals("false");	// Defaults to true
		boolean orderKeyVisible = getStringOfXPathExpression(edgeXmlNode, "orderkey[1]/@visible[1]").equals("true");		// Defaults to false
		
		// Create edge
		Edge resulEdge = new Edge(fromNode, toNode, orderkey);
		resulEdge.setVisibilty(fileNameVisible, fileTypeVisible, orderKeyVisible);

		// Set type
		resulEdge.setEdgeFileType(filetype);

		// Set file name of from node
		fromNode.setXMLFileName(filepath + filename);
		
		// Load coordinates
		Vector<String> oldStyleCoordinates = getValuesOfAllTextNodesBelowXPathNodes(edgeXmlNode, "points/point");
		Vector<String> newStyleCoordinates = getValuesOfAllTextNodesBelowXPathNodes(edgeXmlNode, "points/gridpoint");		

		if (newStyleCoordinates.size() > 0) {
			// If any new style coordinates are found, only load these
			resulEdge.deleteAllPointNodes();
			for (String curPointNodeCoordinates : newStyleCoordinates) {
				int gridXPos = Integer.valueOf(curPointNodeCoordinates.trim().split(" ")[0]);
				int gridYPos = Integer.valueOf(curPointNodeCoordinates.trim().split(" ")[1]);
				if ((gridXPos == -1) || (gridYPos == -1))
					resulEdge.createNewPointNodeAfterLastPointNode(null);
				else
					resulEdge.createNewPointNodeAfterLastPointNode(new Point(gridXPos, gridYPos));
			}
		} else {
			// If no new style coordinates are found, process old style coordinates and transform them

			// Flag to indicate if point nodes where user positioned
			boolean nodesAreUserPositioned = true;

			if (oldStyleCoordinates.size() == 1) {
				// In order to detect if the old style nodes were uses positioned, the center point of the edge is needed
				String startNodeCoords = getValuesOfFirstTextNodeBelowXPathNodes(document, "/workflow/nodes/node[id=' " + fromId + " '][1]/coords[1]"); // Note: Space in ID is important!
				String endNodeCoords = getValuesOfFirstTextNodeBelowXPathNodes(document, "/workflow/nodes/node[id=' " + toId + " '][1]/coords[1]");
				int startX = Integer.valueOf(startNodeCoords.trim().split(" ")[0]);
				int startY = Integer.valueOf(startNodeCoords.trim().split(" ")[1]);		
				int endX = Integer.valueOf(endNodeCoords.trim().split(" ")[0]);
				int endY = Integer.valueOf(endNodeCoords.trim().split(" ")[1]);		
				int centerX = endX + Math.round((startX - endX) / 2);
				int centerY = endY + Math.round((startY - endY) / 2);
				
				String curPointNodeCoordinates = oldStyleCoordinates.get(0);
				int xCoord = Integer.valueOf(curPointNodeCoordinates.trim().split(" ")[0]);
				int yCoord = Integer.valueOf(curPointNodeCoordinates.trim().split(" ")[1]);
				if ((xCoord == centerX) && (yCoord == centerY)) {
					// If there is only one point node which coordinates match
					// match the edges center, assume it is a the standard point 
					// node which every new style edge has anyway.
					// So nothing to do here					
					nodesAreUserPositioned = false;
				}
			}

			// There are several PointNodes, assume they were positioned by the user.
			// So load them all and create a matching new style node for each of them
			if (nodesAreUserPositioned) {
				resulEdge.deleteAllPointNodes();
				for (String curPointNodeCoordinates : oldStyleCoordinates) {
					int xPos = Integer.valueOf(curPointNodeCoordinates.trim().split(" ")[0]);
					int yPos = Integer.valueOf(curPointNodeCoordinates.trim().split(" ")[1]);
					Point gridPosition = oldGrid.getNearestCell(xPos, yPos);
					resulEdge.createNewPointNodeAfterLastPointNode(gridPosition);
				}	
			}
		}

		return resulEdge;
	}
	
	/**
	 * This method normalizes the passed path by adding
	 * a path separator at the end of it if none was present 
	 * so far.
	 */
	private String normalizePath(String orgPath) {
		if ((orgPath.length() > 0) && (!orgPath.endsWith(File.separator)))
			return orgPath + File.separator;
		else
			return orgPath;
	}


}
