/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */


package gui;

import gui.helper.DialogManager;
import helper.SmallHelpers;

import java.awt.Choice;
import java.awt.Color;
import java.awt.Rectangle;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import javax.swing.JPanel;

import core.BooleanParameterGroup;
import core.FileNameParameterGroup;
import core.ParameterGroup;
import core.SelectParameterGroup;
import core.TextParameterGroup;
import core.TypedFile;
import core.graph.GraphNode;
import core.graph.Edge;
import core.graph.Graph;
import core.graph.InputFileNode;
import core.graph.JobNode;
import core.graph.OutputFileNode;
import core.history.RemoveConnectionHistoryObject;
import core.history.SetFileNameHistoryObject;
import core.history.SetEdgeTypeHistoryObject;
import core.storage.ConfigurationXMLReader;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JTextField;

/**
 * This class defines the properties of the "Plotter" panel.
 * 
 * This class provides direct visualization of characteristics of 
 * models (eg CDF, ACF) within the GUI.
 * 
 * Therefore the edge whose file should be plotted is passed when creating this
 * panel.
 * 
 * A click on the "OK" button causes the current edge to be used as input for the 
 * plotter. A click on the "Cancel" button closes this window. 
 */

public class EdgePlotterPanel extends JPanel {
	private static final long serialVersionUID = 1L;

	/**
	 * Frame this panel is on.
	 */
	private EdgePlotterFrame edgePlotterFrame;

	/**
	 * Edge this panel is assigned to.
	 */
	private Edge myEdge;
	
	/**
	 * Job this panel is assigned to.
	 */
	private JobNode myJob;
	
	/**
	 * Reference to the xml reader used to access the configuration file.
	 */
	private ConfigurationXMLReader myReader;

	/**
	 * Height which this panel needs in order to be displayed correctly. The
	 * value is depended on the amount and type of the subpanels.
	 */
	protected int desiredHeight;

	/**
	 * Reference to the workflow panel.
	 */
	private WorkflowPanel workflowPanel;
		
	private JButton okButton = null;
	private JButton cancelButton = null;
	private JLabel prefixStartTextField = null;
	private JLabel prefixEndTextField = null;
	private JLabel prefixChoiceField = null;
	private JTextField startTextField = null;
	private JTextField endTextField = null;

	private Choice choice;

	/**
	 * Creates and initializes the panel.
	 */
	public EdgePlotterPanel(ConfigurationXMLReader xmlReader, Graph graph, Edge edge, WorkflowPanel workflowPanel,
			EdgePlotterFrame edgePlotterFrame) {
		super();
		this.myReader = xmlReader;
		this.myEdge = edge;
		Vector<String> progNames = xmlReader.getProgrammNames();
		int pid = -1;
		for (String progName: progNames) {
			if (progName.trim().equals("Plot")) {
				pid = progNames.indexOf(progName);
				break;
			}
		}

		this.myJob = new JobNode(0, 0, xmlReader, workflowPanel.getGrid(), pid, graph);
		this.workflowPanel = workflowPanel;
		this.edgePlotterFrame = edgePlotterFrame;
		this.choice = new Choice();

		initialize();
		setParameterGroupValues();
	}

	/**
	 * Initializes all components used by this panel and adds them to 
	 * its content pane.
	 */
	private void initialize() {
		this.setSize(300, 500);
		this.setLayout(null);
		this.add(getOkButton(), null);
		this.add(getCancelButton(), null);
		this.add(getStartPrefixTextField(), null);
		this.add(getStartTextField(), null);
		this.add(getEndPrefixTextField(), null);
		this.add(getEndTextField(), null);
		this.add(getPrefixChoiceField(), null);
		this.add(getChoiceField(), null);

		this.addComponentListener(new ComponentAdapter() {
			public void componentResized(ComponentEvent evt) {
				updatePositions();
			}
		});

		updatePositions();
	}

	/**
	 * This method reads the default values of the parameter groups Start, End 
	 * and Type of the corresponding job and writes them on the panel.
	 */
	private void setParameterGroupValues() {
		Vector<ParameterGroup> parameterGroups = myJob.getParameterGroups();
		for (ParameterGroup curGroup : parameterGroups) {

			if (curGroup instanceof TextParameterGroup) {

				if (curGroup.getName().equals("Start")) {
					prefixStartTextField.setText("Start");
					startTextField.setText(((TextParameterGroup) curGroup)
							.getDefaultValue());
				}

				else if (curGroup.getName().equals("End")) {
					prefixEndTextField.setText("End");
					endTextField.setText(((TextParameterGroup) curGroup)
							.getDefaultValue());
				}

			}

			else if (curGroup instanceof SelectParameterGroup) {

				if (curGroup.getName().equals("Type")) {
					prefixChoiceField.setText("Type");
					choice.select(((SelectParameterGroup) curGroup)
							.getDefaultChoice());
				}
			}

		}
	}

	/**
	 * This method adds the permitted file types to the choice box.
	 */
	private Choice getChoiceField() {

		Vector<ParameterGroup> parameterGroups = myJob.getParameterGroups();
		for (ParameterGroup curGroup : parameterGroups) {

			if (curGroup instanceof SelectParameterGroup) {

				if (curGroup.getName().equals("Type")) {

					Vector<String> list = ((SelectParameterGroup) curGroup)
							.getChoices();

					for (int i = 0; i < list.size(); i++) {

						choice.add(list.get(i));
					}
				}

			}

		}
		return choice;

	}

	/**
	 * Updates the position of all components on this panel.
	 */
	public void updatePositions() {
		// int curTopPosition = 2;

		// Define Margins
		int topMargin = 4;
		int leftMargin = 5;
		int rightMargin = 5;
		int bottomMargin = 5;

		// Define component sizes
		int buttonLeftSpacing = 5;
		int textFieldLeftSpacing = 5;

		// int panelTopSpacing = 2;
		int buttonTopSpacing = 6;
		int buttonHeigth = 20;
		int textFieldHeight = 28;

		int contentWidth = this.getWidth() - (leftMargin + rightMargin);

		int textFieldWidth = (contentWidth - (2 * textFieldLeftSpacing)) / 4;

		if (prefixStartTextField != null) {
			prefixStartTextField.setBounds(new Rectangle(leftMargin, topMargin,
					textFieldWidth, textFieldHeight));
		}

		if (startTextField != null) {
			startTextField.setBounds(new Rectangle(leftMargin + textFieldWidth
					+ textFieldLeftSpacing, topMargin, 3 * textFieldWidth
					+ leftMargin, textFieldHeight));
		}

		int secondRowTopPosition = startTextField.getY()
				+ startTextField.getHeight() + buttonTopSpacing;

		if (prefixEndTextField != null) {
			prefixEndTextField.setBounds(new Rectangle(leftMargin,
					secondRowTopPosition, textFieldWidth, textFieldHeight));
		}

		if (endTextField != null) {
			endTextField.setBounds(new Rectangle(2 * leftMargin
					+ textFieldWidth, secondRowTopPosition, 3 * textFieldWidth
					+ leftMargin, textFieldHeight));
		}

		int thirdRowTopPosition = endTextField.getY()
				+ endTextField.getHeight() + buttonTopSpacing;

		if (prefixChoiceField != null) {
			prefixChoiceField.setBounds(new Rectangle(leftMargin,
					thirdRowTopPosition, textFieldWidth, textFieldHeight));
		}

		if (choice != null) {
			choice.setBounds(new Rectangle(2 * leftMargin + textFieldWidth,
					thirdRowTopPosition, 3 * textFieldWidth + leftMargin,
					textFieldHeight));
		}

		int fourthRowTopPosition = choice.getY() + choice.getHeight()
				+ buttonTopSpacing * 2;

		int buttonWidth = (contentWidth - (3 * buttonLeftSpacing)) / 4;

		if (okButton != null) {
			okButton.setBounds(new Rectangle(leftMargin, fourthRowTopPosition,
					buttonWidth, buttonHeigth));
		}

		if (cancelButton != null) {
			cancelButton.setBounds(new Rectangle(leftMargin + (3 * buttonWidth)
					+ (3 * buttonLeftSpacing), fourthRowTopPosition,
					buttonWidth, buttonHeigth));
		}

		desiredHeight = okButton.getY() + okButton.getHeight() + bottomMargin;
	}

	public int getDesiredHeight() {
		return desiredHeight;
	}

	/**
	 * Repaints the workflow.
	 */
	public void repaintGraph() {
		workflowPanel.repaint();
	}

	private JButton getOkButton() {
		if (okButton == null) {
			okButton = new JButton();
			okButton.setBounds(new Rectangle(9, 134, 75, 29));
			okButton.setText("OK");
			okButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {

					String targetFileName;

					// No export folder was specified, so select manually
					if (workflowPanel.getExpFolder().equals("")) {
						JOptionPane.showMessageDialog(workflowPanel, "Please select the directory where you have exported the script", "Choose Directory", JOptionPane.INFORMATION_MESSAGE);
						File newPath = DialogManager.showExportFolderDialog(
								workflowPanel.getOutputFolder(), workflowPanel);
						if (newPath != null) {

							targetFileName = newPath.getPath();

							try {
								createScriptForPlotting(targetFileName);
							} catch (IOException e1) {

								e1.printStackTrace();
							}

						}

					}

					else {

						targetFileName = workflowPanel.getExpFolder();

						try {
							createScriptForPlotting(targetFileName);
						} catch (IOException e1) {

							e1.printStackTrace();
						}
					}

				}

			});
		}
		return okButton;
	}

	/**
	 * This method creates the script which carry out a 'workflow' which 
	 * consists of an input (current edge) and the plotter.
	 * This script executes the plotter whose result is displayed in a 
	 * additional frame.
	 * 
	 * @param targetFileName the target where the script is stored
	 * @throws IOException
	 */
	public void createScriptForPlotting(String targetFileName)
			throws IOException {
		
		File targetFolder = new File(targetFileName + "/Test");
		targetFolder.mkdir();
		
				
		String resultHead = "#!/bin/sh\n";
		resultHead = resultHead
			+ "# Before the actual execution, variables are set. To adapt the\n";
		resultHead = resultHead
				+ "# batch to another location just change the BASEDIR variable.\n";
		resultHead = resultHead + "export BASEDIR=\""
				+ (new TypedFile(targetFileName + "/tmp.tmp")).getParent() + "\"";
		resultHead = resultHead + "\n";
		resultHead = resultHead + "\n";

		resultHead = resultHead + "# Derived folders: \n";
		resultHead = resultHead + "export TESTDIR=\"${BASEDIR}/Test\" \n";
		resultHead = resultHead
				+ "export SOFTWAREDIR=\"${BASEDIR}/Software\" \n";


		resultHead = resultHead + "\n";
		resultHead = resultHead + "\n";
		resultHead = resultHead + "# ----- Execute Jobs ----- \n";
		resultHead = resultHead + "# Execute " + myJob.getName() + "\n";
		resultHead = resultHead + "cd \"" + "${SOFTWAREDIR}/" + myJob.getName()
				+ "\"\n";


		Vector<FileNameParameterGroup> usedInputParameters = new Vector<FileNameParameterGroup>();
		String curType = myEdge.getEdgeFileType();

		String corresponingParameterName = myReader
				.getProgrammInputParameterByFileType(myJob.getReferenceID(),
						curType);

		FileNameParameterGroup currentInputFileParameterGroup = (FileNameParameterGroup) SmallHelpers
				.findParameterGroupByName(corresponingParameterName, myJob
						.getParameterGroups());

		currentInputFileParameterGroup.addEdge(myEdge);

		String converterBinary = myReader
				.getProgrammInputConverterBinaryByFileType(myJob
						.getReferenceID(), curType);

		currentInputFileParameterGroup.setInputConverterBinary(converterBinary);

		if (usedInputParameters.indexOf(currentInputFileParameterGroup) == -1)
			usedInputParameters.add(currentInputFileParameterGroup);

		// Create input converter calls
		for (FileNameParameterGroup curGroup : usedInputParameters) {

			if (curGroup.hasEdges()){
				resultHead = resultHead
						+ curGroup.createConverterCalls("${SOFTWAREDIR}/",
								"${BASEDIR}/Results/") + "\n";


			}
		}

		resultHead = resultHead
				+ "./"
				+ myReader.getProgrammBinaryNameByID(myJob.getReferenceID())
				.trim();



		ParameterGroup curParameterGroup = myJob.getParameterGroups().get(0);

		while (curParameterGroup != null) {

			if (curParameterGroup instanceof FileNameParameterGroup) {

				if (curParameterGroup.getName().equals("Output Filename")) {



					resultHead = resultHead + " " + curParameterGroup.getKey()
							+ "${TESTDIR}/" + "batch_Plot0.xml.org";


				}

				else {


					resultHead = resultHead
							+ " "
							+ ((FileNameParameterGroup) curParameterGroup)
							.getCommandlineCallString(
									"${BASEDIR}/Results/").trim();


				}	
			} else {
				if (curParameterGroup.getName().equals("Format")) {
					resultHead = resultHead + " " + curParameterGroup.getKey()
							+ "png";
				}

				else if (curParameterGroup.getName().equals("Start")) {
					resultHead = resultHead + " " + curParameterGroup.getKey()
							+ startTextField.getText();
				} else if (curParameterGroup.getName().equals("End")) {
					resultHead = resultHead + " " + curParameterGroup.getKey()
							+ endTextField.getText();
				} else if (curParameterGroup.getName().equals("Type")) {
					resultHead = resultHead + " " + curParameterGroup.getKey()
							+ choice.getSelectedItem();

				}

			}
			curParameterGroup = curParameterGroup.getSuccessor();
		}

		currentInputFileParameterGroup.clearEdges();

		// .sh erstellen

		File targetFile = new File(targetFileName + "/Test/sk.sh");
		FileWriter writer = new FileWriter(targetFile);
		BufferedWriter buffer = new BufferedWriter(writer);
		buffer.write(resultHead);
		buffer.close();
		System.out.println("Created Workflow-Script " + targetFile);

		targetFile.setExecutable(true);

		// .sh ausfuehren

		Process p = Runtime.getRuntime().exec("sh " + targetFile.getPath());

		try {
			p.waitFor();

		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		// .png im Frame anzeigen

		new ShowImageFrame(targetFileName
				+ "/Test/batch_Plot0.xml.org.png");

	}

	private JButton getCancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton();
			cancelButton.setBounds(new Rectangle(81, 131, 75, 29));
			cancelButton.setText("Cancel");
			cancelButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					edgePlotterFrame.finished();
				}
			});
		}
		return cancelButton;
	}

	private JLabel getStartPrefixTextField() {
		if (prefixStartTextField == null) {
			prefixStartTextField = new JLabel();

		}
		return prefixStartTextField;
	}

	private JTextField getStartTextField() {
		if (startTextField == null) {
			startTextField = new JTextField();
			startTextField.setBounds(new Rectangle(98, 63, 14, 28));
		}
		return startTextField;
	}

	private JLabel getEndPrefixTextField() {
		if (prefixEndTextField == null) {
			prefixEndTextField = new JLabel();

		}
		return prefixEndTextField;
	}

	private JTextField getEndTextField() {
		if (endTextField == null) {
			endTextField = new JTextField();
			endTextField.setBounds(new Rectangle(98, 63, 14, 28));
		}
		return endTextField;
	}

	private JLabel getPrefixChoiceField() {
		if (prefixChoiceField == null) {
			prefixChoiceField = new JLabel();

		}
		return prefixChoiceField;
	}

}
