/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package gui;

import java.awt.Choice;
import java.awt.Color;
import java.awt.Rectangle;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import javax.swing.JPanel;

import core.graph.Edge;
import core.graph.Graph;
import core.graph.JobNode;
import core.history.ChangeEdgeInformationVisibiltyHistoryObject;
import core.history.RemoveConnectionHistoryObject;
import core.history.SetEdgeOrderKeyHistoryObject;
import core.history.SetFileNameHistoryObject;
import core.history.SetEdgeTypeHistoryObject;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JTextField;

/**
 * This class defines the properties of the "Edit File" panel.
 * 
 * It is used to change file name and type of an edge. The file name can be 
 * entered in a text field while the type can be selected from a choice box.
 * The listed file types are the permitted types for a connection between two nodes.  
 * A click on the "OK" button confirms the changes, stores the new values, closes the window 
 * and updates the information over the edge.  
 * A click on the "Last Values" button reverts the changes to the last saved version.
 * A click on the "Cancel" button cancels the window without performing any kind of change.
 */

public class EditFilePanel extends JPanel {

	private static final long serialVersionUID = 1L;


	/**
	 * Height which this panel needs in order to be displayed correctly. The
	 * value is depended on the amount and type of the subpanels.
	 */
	protected int desiredHeight;

	/**
	 * Reference to the workflow panel.
	 */
	private WorkflowPanel workflowPanel;

	/**
	 * Reference to the frame this panel is on.
	 */
	private EditFileFrame editFileFrame;

	/**
	 * Checkbox used to choose if this edges file name is visible.
	 */
	protected JCheckBox fileNameVisibleCheckBox = null;
	
	/**
	 * Checkbox used to choose if this edges file type is visible.
	 */
	protected JCheckBox fileTypeVisibleCheckBox = null;
	
	/**
	 * Checkbox used to choose if this edges order key is visible.
	 */
	protected JCheckBox orderKeyVisibleCheckBox = null;

	private JButton okButton = null;
	private JButton cancelButton = null;
	private JButton defaultValueButton = null;
	private JButton lastValueButton = null;	
	private JLabel nameLabel = null;
	private JLabel typeLabel = null;
	private JLabel orderTextFieldLabel = null;
	private JTextField nameTextField = null;
	private JTextField orderTextField = null;
	
	/**
	 * Choice box displaying the possible file types of this panels edge.
	 */
	private Choice typeChoice;

	/**
	 * Edge this panel is assigned to.
	 */
	private Edge edge;

	/**
	 * Creates and initializes the panel.
	 */
	public EditFilePanel(Edge edge, WorkflowPanel workflowPanel, EditFileFrame editFileFrame) {
		super();	
		this.edge = edge;
		this.workflowPanel = workflowPanel;
		this.editFileFrame = editFileFrame;

		initialize();
		writeInformations();
	}

	/**
	 * Initializes all components used by this panel and adds them to 
	 * its content pane.
	 */
	private void initialize() {
		this.setSize(300, 500);
		this.setLayout(null);
		this.add(getOkButton(), null);
		this.add(getCancelButton(), null);
		this.add(getDefaultValueButton(), null);
		this.add(getLastValueButton(), null);
		this.add(getPrefixTextField(), null);
		this.add(getOrderTextField(), null);
		this.add(getNameTextField(), null);
		this.add(getPrefixChoiceField(), null);
		this.add(getOrderTextFieldLabel(), null);
		this.add(getChoiceField(),null);
		this.add(getFileNameVisibleCheckBox(), null);
		this.add(getFileTypeVisibleCheckBox(), null);
		this.add(getOrderKeyVisibleCheckBox(), null);

		this.addComponentListener(new ComponentAdapter() {
			public void componentResized(ComponentEvent evt) {
				updatePositions();
			}
		});

		updatePositions();
	}

	private Choice getChoiceField() {
		if (typeChoice == null) {
			typeChoice = new Choice();
			for (String curType : edge.getPossibleTypes())
				typeChoice.add(curType);
		}
		return typeChoice;		

	}

	/**
	 * Updates the position of all components on this panel.
	 */
	public void updatePositions() {
		//int curTopPosition = 2;

		// Define Margins
		int topMargin = 4;
		int leftMargin = 5;
		int rightMargin = 5;
		int bottomMargin = 5;

		// Define component sizes
		int buttonLeftSpacing = 5;
		int labelWidth = 66;
		int checkBoxWidth = 80;

		//int panelTopSpacing = 2;
		int buttonTopSpacing = 6; 
		int buttonHeigth = 20;	
		int textFieldHeight = 28;

		int checkBoxHeight = 23;
		int componentLeftSpacing = 2;

		int contentWidth = this.getWidth() - (leftMargin + rightMargin);

		int textFieldWidth = contentWidth - labelWidth - checkBoxWidth - 2 * componentLeftSpacing;
		
		if (nameLabel != null) {
			nameLabel.setBounds(new Rectangle(leftMargin, topMargin, labelWidth, textFieldHeight));
		}

		if (nameTextField != null) {
			nameTextField.setBounds(new Rectangle(leftMargin + labelWidth + componentLeftSpacing, topMargin, textFieldWidth, textFieldHeight));
		}
		
		if (fileNameVisibleCheckBox != null) {
			fileNameVisibleCheckBox.setBounds(new Rectangle(leftMargin + labelWidth + textFieldWidth + (2 * componentLeftSpacing), topMargin, checkBoxWidth, checkBoxHeight));
		}
		
		int secondRowTopPosition = nameTextField.getY() + nameTextField.getHeight() + buttonTopSpacing;	

		if (typeLabel != null) {
			typeLabel.setBounds(new Rectangle(leftMargin, secondRowTopPosition, labelWidth, textFieldHeight));
		}

		if (typeChoice!= null){
			typeChoice.setBounds(new Rectangle(leftMargin + labelWidth + componentLeftSpacing, secondRowTopPosition, textFieldWidth, textFieldHeight));
		}
		
		if (fileTypeVisibleCheckBox != null) {
			fileTypeVisibleCheckBox.setBounds(new Rectangle(leftMargin + labelWidth + textFieldWidth + (2 * componentLeftSpacing), secondRowTopPosition, checkBoxWidth, checkBoxHeight));
		}
		
		int thirdRowTopPosition = typeChoice.getY() + typeChoice.getHeight() + buttonTopSpacing;	

		if (orderTextFieldLabel != null) {
			orderTextFieldLabel.setBounds(new Rectangle(leftMargin, thirdRowTopPosition, textFieldWidth, textFieldHeight));
		}

		if (orderTextField != null){
			orderTextField.setBounds(new Rectangle(leftMargin + labelWidth + componentLeftSpacing, thirdRowTopPosition, textFieldWidth, textFieldHeight));
		}
		
		if (orderKeyVisibleCheckBox != null) {
			orderKeyVisibleCheckBox.setBounds(new Rectangle(leftMargin + labelWidth + textFieldWidth + (2 * componentLeftSpacing), thirdRowTopPosition, checkBoxWidth, checkBoxHeight));
		}
				
		
		int fourthRowTopPosition = orderTextField.getY() + orderTextField.getHeight() + buttonTopSpacing;			
		int buttonWidth = (contentWidth - (3 * buttonLeftSpacing)) / 4;

		if (okButton != null) {
			okButton.setBounds(new Rectangle(leftMargin, fourthRowTopPosition, buttonWidth, buttonHeigth));
		}

		if (lastValueButton != null) {
			lastValueButton.setBounds(new Rectangle(leftMargin + buttonWidth + buttonLeftSpacing, fourthRowTopPosition, buttonWidth, buttonHeigth));
		}

		if (defaultValueButton != null) {
			defaultValueButton.setBounds(new Rectangle(leftMargin + (2 * buttonWidth) + (2 * buttonLeftSpacing), fourthRowTopPosition, buttonWidth, buttonHeigth));
		}

		if (cancelButton != null) {
			cancelButton.setBounds(new Rectangle(leftMargin + (3 * buttonWidth) + (3 * buttonLeftSpacing), fourthRowTopPosition, buttonWidth, buttonHeigth));
		}


		desiredHeight = okButton.getY() + okButton.getHeight() + bottomMargin;
	}

	public int getDesiredHeight() {
		return desiredHeight;
	}

	private JButton getOkButton() {
		if (okButton == null) {
			okButton = new JButton();
			okButton.setBounds(new Rectangle(9, 134, 75, 29));
			okButton.setText("OK");
			okButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					String oldTyp = edge.getEdgeFileType();
					String oldName= edge.getFileName().getPath();
					String oldOrderKey = edge.getOrderKey();

					if(!(oldName.equals(nameTextField.getText()))){
						workflowPanel.getGraph().addNewAction(new SetFileNameHistoryObject(edge, nameTextField.getText()));
						workflowPanel.getGraph().performLastAction();
					}

					if(!(oldTyp.equals(typeChoice.getSelectedItem()))){
						workflowPanel.getGraph().addNewAction(new SetEdgeTypeHistoryObject(edge, typeChoice.getSelectedItem()));
						workflowPanel.getGraph().performLastAction();
					}

					if(!(oldOrderKey.equals(orderTextField.getText()))){
						workflowPanel.getGraph().addNewAction(new SetEdgeOrderKeyHistoryObject(edge, orderTextField.getText()));
						workflowPanel.getGraph().performLastAction();
					}
									
					if ((fileNameVisibleCheckBox.isSelected() != edge.isFileNameVisible()) || (fileTypeVisibleCheckBox.isSelected() != edge.isFileTypeVisible()) || (orderKeyVisibleCheckBox.isSelected() != edge.isOrderKeyVisible())) {
						workflowPanel.getGraph().addNewAction(new ChangeEdgeInformationVisibiltyHistoryObject(edge, fileNameVisibleCheckBox.isSelected(), fileTypeVisibleCheckBox.isSelected(), orderKeyVisibleCheckBox.isSelected()));
						workflowPanel.getGraph().performLastAction();				
					}
						
					editFileFrame.finished();
				}
			});
		}
		return okButton;
	}

	private JButton getCancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton();
			cancelButton.setBounds(new Rectangle(81, 131, 75, 29));
			cancelButton.setText("Cancel");
			cancelButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					editFileFrame.finished();
				}
			});
		}
		return cancelButton;
	}

	private JButton getDefaultValueButton() {
		if (defaultValueButton == null) {
			defaultValueButton = new JButton();
			defaultValueButton.setBounds(new Rectangle(152, 131, 75, 29));
			defaultValueButton.setText("Generate");
			defaultValueButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					// TODO Implement function
				}
			});
		}
		return defaultValueButton;
	}

	private JButton getLastValueButton() {
		if (lastValueButton == null) {
			lastValueButton = new JButton();
			lastValueButton.setText("Last Values");
			lastValueButton.setBounds(new Rectangle(187, 170, 75, 29));
			lastValueButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					writeInformations();
				}
			});
		}
		return lastValueButton;
	}

	/**
	 * This method writes the file name and file type on this panel.
	 */
	private void writeInformations() {
		orderTextField.setText(edge.getOrderKey());
		nameTextField.setText(edge.getFileName().getPath());
		typeChoice.select(edge.getEdgeFileType());
		orderKeyVisibleCheckBox.setSelected(edge.isOrderKeyVisible());
		fileTypeVisibleCheckBox.setSelected(edge.isFileTypeVisible());
		fileNameVisibleCheckBox.setSelected(edge.isFileNameVisible());
	}

	private JLabel getPrefixTextField() {
		if (nameLabel == null) {
			nameLabel = new JLabel();
			nameLabel.setText("File Name");
		}
		return nameLabel;
	}

	private JTextField getNameTextField() {
		if (nameTextField == null) {
			nameTextField = new JTextField();
			nameTextField.setBounds(new Rectangle(98, 63, 14, 28));
		}
		return nameTextField;
	}


	private JTextField getOrderTextField() {
		if (orderTextField == null) {
			orderTextField = new JTextField();
			orderTextField.setBounds(new Rectangle(98, 63, 14, 28));
		}
		return orderTextField;
	}
	
	private JLabel getOrderTextFieldLabel() {
		if (orderTextFieldLabel == null) {
			orderTextFieldLabel = new JLabel();
			orderTextFieldLabel.setText("Order Key");
		}
		return orderTextFieldLabel;
	}
	
	private JLabel getPrefixChoiceField() {
		if (typeLabel == null) {
			typeLabel = new JLabel();
			typeLabel.setText("File Type");
		}
		return typeLabel;
	}
	

	private JCheckBox getFileNameVisibleCheckBox() {
		if (fileNameVisibleCheckBox == null) {
			fileNameVisibleCheckBox = new JCheckBox();
			fileNameVisibleCheckBox.setText("Visible");
			fileNameVisibleCheckBox.setSelected(edge.isFileNameVisible());
			fileNameVisibleCheckBox.setBounds(new Rectangle(307, 35, 76, 23));
		}
		return fileNameVisibleCheckBox;
	}
	

	private JCheckBox getFileTypeVisibleCheckBox() {
		if (fileTypeVisibleCheckBox == null) {
			fileTypeVisibleCheckBox = new JCheckBox();
			fileTypeVisibleCheckBox.setText("Visible");
			fileTypeVisibleCheckBox.setSelected(edge.isFileTypeVisible());
			fileTypeVisibleCheckBox.setBounds(new Rectangle(307, 35, 76, 23));
		}
		return fileTypeVisibleCheckBox;
	}
	

	private JCheckBox getOrderKeyVisibleCheckBox() {
		if (orderKeyVisibleCheckBox == null) {
			orderKeyVisibleCheckBox = new JCheckBox();
			orderKeyVisibleCheckBox.setText("Visible");
			orderKeyVisibleCheckBox.setSelected(edge.isOrderKeyVisible());
			orderKeyVisibleCheckBox.setBounds(new Rectangle(307, 35, 76, 23));
		}
		return orderKeyVisibleCheckBox;
	}
	
	
}
