/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package gui;

import java.awt.Rectangle;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.util.Vector;

import javax.swing.JPanel;

import core.BooleanParameterGroup;
import core.FileNameParameterGroup;
import core.ParameterGroup;
import core.SelectParameterGroup;
import core.TextParameterGroup;
import core.graph.JobNode;
import core.history.ChangeParameterGroupEnable;
import core.history.HistoryObject;

import javax.swing.JButton;

/**
 * This class defines the properties of the "Edit Job" panel.
 * 
 * This panel displays all parameter groups of the corresponding job. It 
 * consists of a panel for each of the jobs parameter groups. The panels are
 * of type EditOneParameterPanel, EditOneTextParameterPanel or EditOneBooleanParameterPanel,
 * depending on the type of the corresponding ParameterGroup.
 * 
 * A click on the "OK" button confirms the changes, stores the new values, closes the window 
 * and updates the information on the canvas.  
 * A click on the "Last Values" button reverts the changes to the last saved version.
 * A click on the "Cancel" button cancels the current operation without performing any kind 
 * of change.
 */

public class EditJobPanel extends JPanel {

	private static final long serialVersionUID = 1L;

	/**
	 * Vector of all subpanels representing the nodes parameter groups.
	 */
	private Vector<EditOneParameterPanel> parameterPanels; 

	/**
	 * Job node which this frame is assigned to.
	 */
	private JobNode correspondingJob;

	/**
	 * Height which this panel needs in order to be displayed correctly. The
	 * value is depended on the amount and type of the subpanels.
	 */
	protected int desiredHeight;

	/**
	 * Reference to the workflow panel.
	 */
	private WorkflowPanel workflowPanel;

	private JButton okButton = null;
	private JButton cancelButton = null;
	private JButton defaultValueButton = null;
	private JButton lastValueButton = null;
	
	/**
	 * Reference to the frame this panel is on.
	 */
	private EditJobFrame editJobFrame;

	/**
	 * Creates and initializes the panel.
	 */
	public EditJobPanel(JobNode myJob, WorkflowPanel workflowPanel, EditJobFrame editJobFrame) {
		super();	
		this.correspondingJob = myJob;
		this.workflowPanel = workflowPanel;
		this.editJobFrame = editJobFrame;

		initialize();
	}

	/**
	 * This method creates the panels for the parameter groups of the JobNode.
	 **/
	private Vector<EditOneParameterPanel> createParameterPanels() {
		if (parameterPanels == null) {
			parameterPanels = new Vector<EditOneParameterPanel>();
			Vector<ParameterGroup> parameterGroups = correspondingJob.getParameterGroups();
			for (ParameterGroup curGroup : parameterGroups) {
				if (!(curGroup instanceof FileNameParameterGroup)) {		
					EditOneParameterPanel curPanel = null;
					if (curGroup instanceof TextParameterGroup)
						curPanel = new EditOneTextParameterPanel(correspondingJob, (TextParameterGroup) curGroup, this,  workflowPanel);
					else if (curGroup instanceof BooleanParameterGroup) 
						curPanel = new EditOneBooleanParameterPanel(correspondingJob, (BooleanParameterGroup) curGroup, this,workflowPanel);
					else if (curGroup instanceof SelectParameterGroup)
						curPanel = new EditOneSelectParameterPanel(correspondingJob, (SelectParameterGroup) curGroup, this, workflowPanel);

					curPanel.setBounds(new Rectangle(42, 22, 198, 22));
					this.add(curPanel, null);
					parameterPanels.add(curPanel);	
				}

			}
		}
		updatePositions();
		return parameterPanels;
	}

	/**
	 * Initializes all components used by this panel and adds them to 
	 * its content pane.
	 */
	private void initialize() {
		this.setSize(300, 200);
		this.setLayout(null);
		this.add(getOkButton(), null);
		this.add(getCancelButton(), null);
		this.add(getDefaultValueButton(), null);
		this.add(getLastValueButton(), null);
		this.addComponentListener(new ComponentAdapter() {
			public void componentResized(ComponentEvent evt) {
				updatePositions();
			}
		});

		createParameterPanels();
	}

	/**
	 * Updates the position of all components on this panel.
	 */
	public void updatePositions() {
		int curTopPosition = 2;

		int panelTopSpacing = 2;
		int buttonTopSpacing = 6; 
		int leftMargin = 5;
		int rightMargin = 5;
		int bottomMargin = 5;

		int buttonLeftSpacing = 5;
		int buttonHeigth = 20;	

		int contentWidth = this.getWidth() - (leftMargin + rightMargin);

		int curLeftPosition = leftMargin;

		for (EditOneParameterPanel curPanel : parameterPanels) {
			curPanel.setBounds(new Rectangle(curLeftPosition, curTopPosition, contentWidth, curPanel.getDesiredHeight()));
			curTopPosition = curTopPosition + panelTopSpacing + curPanel.getDesiredHeight();
		}

		int secondRowTopPosition;
		if (parameterPanels.size() > 0) {
			EditOneParameterPanel lastPanel = parameterPanels.get(parameterPanels.size() - 1);

			secondRowTopPosition = lastPanel.getY() + lastPanel.getDesiredHeight() + buttonTopSpacing;			
		} else
			secondRowTopPosition = buttonTopSpacing;

		int buttonWidth = (contentWidth - (3 * buttonLeftSpacing)) / 4;

		if (okButton != null) {
			okButton.setBounds(new Rectangle(leftMargin, secondRowTopPosition, buttonWidth, buttonHeigth));
		}

		if (lastValueButton != null) {
			lastValueButton.setBounds(new Rectangle(leftMargin + buttonWidth + buttonLeftSpacing, secondRowTopPosition, buttonWidth, buttonHeigth));
		}

		if (defaultValueButton != null) {
			defaultValueButton.setBounds(new Rectangle(leftMargin + (2 * buttonWidth) + (2 * buttonLeftSpacing), secondRowTopPosition, buttonWidth, buttonHeigth));
		}

		if (cancelButton != null) {
			cancelButton.setBounds(new Rectangle(leftMargin + (3 * buttonWidth) + (3 * buttonLeftSpacing), secondRowTopPosition, buttonWidth, buttonHeigth));
		}

		desiredHeight = okButton.getY() + okButton.getHeight() + bottomMargin;
	}

	public int getDesiredHeight() {
		return desiredHeight;
	}

	private JButton getOkButton() {
		// TODO Improve implementation
		if (okButton == null) {
			okButton = new JButton();
			okButton.setBounds(new Rectangle(9, 134, 75, 29));
			okButton.setText("OK");
			okButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {	
					for (EditOneParameterPanel curPanel : parameterPanels) {
						if(curPanel instanceof EditOneBooleanParameterPanel) {
							// check if change occoured
							Vector <HistoryObject> curPanelHistoryObject = ((EditOneBooleanParameterPanel) curPanel).createHistoryObjectToInheritTemporaryBooleanParameterCheckBoxValues();
							if (curPanelHistoryObject.size()>0) {
								if (curPanelHistoryObject.size() == 2) {
									workflowPanel.getGraph().addNewAction(curPanelHistoryObject.firstElement());
									workflowPanel.getGraph().performLastAction();

									workflowPanel.getGraph().addNewAction(curPanelHistoryObject.lastElement());
									workflowPanel.getGraph().performLastAction();
								}

								// one check box changed
								else if (curPanelHistoryObject.size() == 1) {
									//System.out.println("==1");

									workflowPanel.getGraph().addNewAction(curPanelHistoryObject.firstElement());
									workflowPanel.getGraph().performLastAction();
								}
							}

						}
						//selectParameter

						else if(curPanel instanceof EditOneSelectParameterPanel){

							//check if selection changed

							HistoryObject curPanelHistoryObject1 = curPanel.createHistoryObjectToInheritTemporaryValues();

							if (curPanelHistoryObject1 != null) {


								workflowPanel.getGraph().addNewAction(curPanelHistoryObject1);
								workflowPanel.getGraph().performLastAction();
							}

							//check if check boxes changed

							Vector <HistoryObject> curPanelHistoryObject2 = ((EditOneSelectParameterPanel) curPanel).createHistoryObjectToInheritTemporarySelectParameterCheckBoxValues();
							if (curPanelHistoryObject2.size()>0) {

								//System.out.println(">0");

								// both check boxes changed

								if(curPanelHistoryObject2.size()==2){

									//System.out.println("==2");

									workflowPanel.getGraph().addNewAction(curPanelHistoryObject2.firstElement());
									workflowPanel.getGraph().performLastAction();

									workflowPanel.getGraph().addNewAction(curPanelHistoryObject2.lastElement());
									workflowPanel.getGraph().performLastAction();
								}

								//one check box changed

								else if(curPanelHistoryObject2.size()==1){

									//System.out.println("==1");

									workflowPanel.getGraph().addNewAction(curPanelHistoryObject2.firstElement());
									workflowPanel.getGraph().performLastAction();
								}
							}

						}

						//textParameter

						else if(curPanel instanceof EditOneTextParameterPanel){

							//check if text fields changed

							HistoryObject curPanelHistoryObject1 = curPanel.createHistoryObjectToInheritTemporaryValues();
							if (curPanelHistoryObject1 != null) {

								workflowPanel.getGraph().addNewAction(curPanelHistoryObject1);
								workflowPanel.getGraph().performLastAction();}

							//check if check boxes changed

							Vector <HistoryObject> curPanelHistoryObject2 = ((EditOneTextParameterPanel) curPanel).createHistoryObjectToInheritTemporaryTextParameterCheckBoxValues();
							if (curPanelHistoryObject2.size()>0) {

								//System.out.println(">0");

								// both check boxes changed

								if(curPanelHistoryObject2.size()==2){

									//System.out.println("==2");

									workflowPanel.getGraph().addNewAction(curPanelHistoryObject2.firstElement());
									workflowPanel.getGraph().performLastAction();

									workflowPanel.getGraph().addNewAction(curPanelHistoryObject2.lastElement());
									workflowPanel.getGraph().performLastAction();
								}

								//one check box changed

								else if(curPanelHistoryObject2.size()==1){

									//System.out.println("==1");

									workflowPanel.getGraph().addNewAction(curPanelHistoryObject2.firstElement());
									workflowPanel.getGraph().performLastAction();
								}
							}

						}


					}
					editJobFrame.finished();
				}
			});
		}
		return okButton;
	}

	private JButton getCancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton();
			cancelButton.setBounds(new Rectangle(81, 131, 75, 29));
			cancelButton.setText("Cancel");
			cancelButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					editJobFrame.finished();
				}
			});
		}
		return cancelButton;
	}

	private JButton getDefaultValueButton() {
		if (defaultValueButton == null) {
			defaultValueButton = new JButton();
			defaultValueButton.setBounds(new Rectangle(152, 131, 75, 29));
			defaultValueButton.setText("Defaults");
			defaultValueButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					for (EditOneParameterPanel curPanel : parameterPanels) {
						curPanel.showDefaultValues();
					}
				}
			});
		}
		return defaultValueButton;
	}

	private JButton getLastValueButton() {
		if (lastValueButton == null) {
			lastValueButton = new JButton();
			lastValueButton.setText("Last Values");
			lastValueButton.setBounds(new Rectangle(187, 170, 75, 29));
			lastValueButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					for (EditOneParameterPanel curPanel : parameterPanels) {
						curPanel.showLastValues();

					}

				}
			});
		}
		return lastValueButton;
	}

}