/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package gui;



import java.awt.Rectangle;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;

import core.BooleanParameterGroup;
import core.TextParameterGroup;
import core.graph.JobNode;
import core.history.ChangeBooleanParameterGroupValuesHistoryObject;
import core.history.ChangeParameterGroupVisibility;
import core.history.ChangeParameterGroupEnable;
import core.history.ChangeTextParameterGroupValuesHistoryObject;
import core.history.HistoryObject;

import javax.swing.JList;
import javax.swing.JTextField;
import javax.swing.JLabel;

import java.util.Vector;

/**
 * This class defines the properties of the panel for editing one TextParameterGroup.
 * It contains a text box in order to display and edit the current value.
 */
public class EditOneTextParameterPanel extends EditOneParameterPanel {

	private static final long serialVersionUID = 1L;

	/**
	 * Label for displaying additional informations.
	 */
	private JLabel infoLabel = null;

	/**
	 * Textfield containing the current value.
	 */
	private JTextField valueTextField = null;

	/**
	 * Creates and initializes the panel.
	 */
	public EditOneTextParameterPanel(JobNode node, TextParameterGroup group, EditJobPanel editJobPanel, WorkflowPanel workflowPanel) {
		super(node, editJobPanel,workflowPanel);	
		myParameterGroup = group;
		initialize();
	}

	/**
	 * Initializes all components used by this panel and adds them to 
	 * its content pane.
	 */
	public void initialize() {
		super.initialize();
		infoLabel = new JLabel();
		infoLabel.setBounds(new Rectangle(11, 14, 304, 16));
		updateInfoLabelText();
		this.add(infoLabel, null);
		this.add(getValueTextField(), null);

		enableCheckBox.setText("Enable");

		this.addComponentListener(new ComponentAdapter() {
			public void componentResized(ComponentEvent evt) {
				updatePositions();
			}
		});
		updatePositions();
	}

	/**
	 * Updates the text displayed on the info label
	 */
	private void updateInfoLabelText() {
		infoLabel.setText("Format: " + ((TextParameterGroup) myParameterGroup).getFormat() + " (Result: " + ((TextParameterGroup) myParameterGroup).getTemporaryValueString() + ")");	
	}

	@Override
	public void updatePositions() {
		int topMargin = 13;
		int bottomMargin = 4;

		int leftMargin = 6;
		int rightMargin = 6;
		int componentLeftSpacing = 2;
		int componentTopSpacing = 2;

		int contentWidth = this.getWidth() - leftMargin - rightMargin;
		int labelPositionCorrection = 4;

		// Define desired component sizes
		int helpLabelWidth = 10;
		int enableCheckBoxWidth = 74;
		int visibleCheckBoxWidth = 76;

		int labelHeight = 16;
		int textFieldHeight = 28;
		int checkBoxHeight = 23;

		// Calculate derived sizes and positions
		int checkBoxTopCorrection = (textFieldHeight - checkBoxHeight) / 2;

		if (infoLabel != null) {
			infoLabel.setBounds(new Rectangle(leftMargin + labelPositionCorrection, topMargin, contentWidth - componentLeftSpacing - helpLabelWidth, labelHeight));
		}

		if (helpLabel != null) {
			helpLabel.setBounds(new Rectangle(infoLabel.getX() + infoLabel.getWidth() + componentLeftSpacing, topMargin, helpLabelWidth, labelHeight));
		}

		int secondRowTopPosition = topMargin + labelHeight + componentTopSpacing;

		if (visibleCheckBox != null) {
			visibleCheckBox.setBounds(new Rectangle(this.getWidth() - rightMargin - visibleCheckBoxWidth, secondRowTopPosition + checkBoxTopCorrection, visibleCheckBoxWidth, checkBoxHeight));
		}

		if (enableCheckBox != null) {
			enableCheckBox.setBounds(new Rectangle(visibleCheckBox.getX() - componentLeftSpacing - enableCheckBoxWidth, secondRowTopPosition + checkBoxTopCorrection, enableCheckBoxWidth, checkBoxHeight));
		}

		if (valueTextField != null) {
			valueTextField.setBounds(new Rectangle(leftMargin, secondRowTopPosition, contentWidth - (2 * componentLeftSpacing) - enableCheckBoxWidth - visibleCheckBoxWidth, textFieldHeight));
		}			

		desiredHeight = valueTextField.getY() + valueTextField.getHeight() + bottomMargin;
	}

	private JTextField getValueTextField() {
		if ((valueTextField == null) && (myParameterGroup instanceof TextParameterGroup)) {
			valueTextField = new JTextField();
			valueTextField.setBounds(new Rectangle(22, 37, 174, 28));
			valueTextField.setText(myParameterGroup.getDisplayFriendlyValueString());
			valueTextField.addCaretListener(new javax.swing.event.CaretListener() {
				public void caretUpdate(javax.swing.event.CaretEvent e) {
					proceedTextfieldInput();
				}
			});
			proceedTextfieldInput();
		}
		return valueTextField;
	}

	/**
	 * This method passes the changed value of the text field as temporary value 
	 * to the parameter group and updates the info label afterwards.
	 */
	private void proceedTextfieldInput() {
		((TextParameterGroup) myParameterGroup).parseInputToTemporaryValues(valueTextField.getText());
		updateInfoLabelText();

	}

	/**
	 * This method sets the values of the text field and the check boxes 
	 * to the default values.
	 */
	public void showDefaultValues() {
		valueTextField.setText(((TextParameterGroup) myParameterGroup).getDefaultValue());
		enableCheckBox.setSelected(((TextParameterGroup) myParameterGroup).getDefaultValueEnabled());
		visibleCheckBox.setSelected(((TextParameterGroup) myParameterGroup).getDefaultVisibility());
	}

	/**
	 * This method sets the values of the text field and the check boxes to the 
	 * last stored values.
	 */
	public void showLastValues() {
		valueTextField.setText(myParameterGroup.getDisplayFriendlyValueString());
		enableCheckBox.setSelected(((TextParameterGroup) myParameterGroup).isEnabled());
		visibleCheckBox.setSelected(((TextParameterGroup) myParameterGroup).isVisible());

	}

	/**
	 * This method determines the check boxes whose values has been changed and returns
	 * and creates a history object to inherit these changes.
	 */
	public Vector <HistoryObject> createHistoryObjectToInheritTemporaryTextParameterCheckBoxValues() {
		Vector <HistoryObject> curPanelHistoryObject = new Vector<HistoryObject>()  ;

		if (enableCheckBox.isSelected() != ((TextParameterGroup)myParameterGroup).isEnabled() && visibleCheckBox.isSelected()!= ((TextParameterGroup)myParameterGroup).isVisible()){

			curPanelHistoryObject.add(new ChangeParameterGroupEnable(node.getName(), (TextParameterGroup)myParameterGroup, enableCheckBox.isSelected()));
			curPanelHistoryObject.add(new ChangeParameterGroupVisibility(node.getName(), (TextParameterGroup)myParameterGroup, visibleCheckBox.isSelected()));
		} else if (enableCheckBox.isSelected() != ((TextParameterGroup)myParameterGroup).isEnabled() && !(visibleCheckBox.isSelected()!= ((TextParameterGroup)myParameterGroup).isVisible())){

			curPanelHistoryObject.add(new ChangeParameterGroupEnable(node.getName(), (TextParameterGroup)myParameterGroup, enableCheckBox.isSelected()));
			//System.out.println("E & !V");
		} else if (!(enableCheckBox.isSelected() != ((TextParameterGroup)myParameterGroup).isEnabled()) && visibleCheckBox.isSelected()!= ((TextParameterGroup)myParameterGroup).isVisible()){

			curPanelHistoryObject.add(new ChangeParameterGroupVisibility(node.getName(), (TextParameterGroup)myParameterGroup, visibleCheckBox.isSelected()));
			//System.out.println("!E & V");
		}

		return curPanelHistoryObject;
	}

	@Override
	public HistoryObject createHistoryObjectToInheritTemporaryValues() {		
		Vector<HistoryObject> changes = ((TextParameterGroup) myParameterGroup).createHistoryObjectsToAdoptNewValues();
		if (changes.size() > 0)
			return new ChangeTextParameterGroupValuesHistoryObject(node.getName(), (TextParameterGroup) myParameterGroup, changes);
		else
			return null;

	}




}  