/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.File;
import java.util.Vector;

import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JMenuItem;

import gui.helper.DialogManager;
import core.graph.Graph;
import core.storage.ConfigurationXMLReader;

import javax.swing.JCheckBoxMenuItem;


/**
 * This class defines the properties of main frame of the gui.
 * 
 * The MainWindowPanel consists of the MainWindowPanel and the 
 * applications menu bar with all its items e.g. File, Batch, Edit... 
 * Each menu item has an action listener which are defined in this class
 */

public class MainWindowFrame extends JFrame {

	private static final long serialVersionUID = 1L;


	/**
	 * Reference to this frames main panel.
	 */
	private MainWindowPanel mainWindowPanel = null; 

	/**
	 * Reference to the panel showing the workflow.
	 */
	private WorkflowPanel workflowPanel;

	/**
	 * Reference to the configuration xml reader.
	 */
	private ConfigurationXMLReader configurationXMLReader;

	/**
	 * Main menu bar.
	 */
	private JMenuBar MainMenuBar = null;

	private MainWindowFrame me;

	private JMenu batchMenu = null;
	private JMenu AddJobMenu;
	private JMenu helpMenu;
	private JMenuItem helpContentItem;
	private JMenuItem aboutItem;
	private JCheckBoxMenuItem ResizeCheckBoxM = null;
	private JMenu settingsMenu;
	private JCheckBoxMenuItem ShowGridCheckBoxM;
	private JMenu fileMenu;
	private JMenuItem loadGraphMenuItem;
	private JMenuItem saveGraphMenuItem;
	private JCheckBoxMenuItem displayEdgeInformationsCheckBox;
	private JCheckBoxMenuItem displayNodeInformationsCheckBox;
	private JCheckBoxMenuItem alwaysDisplayOrderKeyCheckBox;	
	private JCheckBoxMenuItem displayAllEdgeInformationsCheckBox;
	private JCheckBoxMenuItem displayAllNodeInformationsCheckBox;
	private JMenu debugMenu;
	private JMenuItem editNameMenuItem;
	private JMenu editMenu;
	private JMenuItem redoMenuItem;
	private JMenuItem undoMenuItem;
	private JMenuItem newOutputFileMenuItem;
	private JMenuItem newInputFileMenuItem;

	/**
	 * Menu item to export the workflow as a shell script.
	 */
	private JMenuItem exportScriptMenuItem;
	private JMenuItem setOutputFolderMenuItem;

	/**
	 * Menu item to create a new workflow.
	 */
	private JMenuItem newBatchMenuItem;
	private JMenuItem showHistoryMenuItem;
	private JMenuItem quitMenuItem;
	private JMenuItem clearHistoryMenuItem;
	private JMenuItem exportScreenShotMenuItem;
	private JCheckBoxMenuItem displayOrderKeyCheckBox;


	/**
	 * Creates and initializes the panel.
	 */	
	public MainWindowFrame(ConfigurationXMLReader reader) {
		super();
		me = this;
		this.setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
		this.configurationXMLReader = reader;

		initialize();
		mainWindowPanel.updatePositions();
	}
	

	/**
	 * This method close the window and the entire application
	 * after verifying if the graph was saved.
	 * @param performCheckIfSaved 
	 */
	public void exitApplication(boolean performCheckIfSaved) {
		boolean exitApplication = true;
		
		// If checks are enabled and the graph is not saved yet, ask user
		if ((performCheckIfSaved) && !workflowPanel.isGraphSaved()) {
			int dialogResult = JOptionPane.showOptionDialog(this, "You are about to close the application without saving the current Workflow.\nWould you like to do so now?", "Workflow not saved.", JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.WARNING_MESSAGE, null, null, null);
			if (dialogResult == JOptionPane.YES_OPTION) {
				// Get file name to save
				File targetName = DialogManager.showSaveDialog(workflowPanel.getOutputFolder(), mainWindowPanel.getWorkflowName(), ".xml", me, true);
				if (targetName != null)	
					workflowPanel.saveGraph(targetName.getPath());						
				else
					// Only close, if a valid file name was selected
					exitApplication = false;
			} else if (dialogResult == JOptionPane.CANCEL_OPTION) {
				// If user canceled the dialog, don't close
				exitApplication = false;
			}
		} 
		
		if (exitApplication)
			System.exit(0);
			
	}

	/**
	 * Initializes all components used by this panel and adds them to 
	 * its content pane.
	 */
	private void initialize() {
		this.setSize(600, 600);
		this.setContentPane(getJContentPane());
		this.setJMenuBar(getMainMenuBar());
		updateTitle("");
		this.addWindowListener(new WindowListener() {
			public void windowOpened(WindowEvent e) {	
			}
			public void windowIconified(WindowEvent e) {
			}
			public void windowDeiconified(WindowEvent e) {
			}
			public void windowDeactivated(WindowEvent e) {
			}
			public void windowClosing(WindowEvent e) {
				exitApplication(true);
			}
			public void windowClosed(WindowEvent e) {				
			}
			public void windowActivated(WindowEvent e) {				
			}
		});
		this.addComponentListener(new ComponentAdapter() {
	        public void componentResized(ComponentEvent evt) {

	        }
	      });
	}

	private JPanel getJContentPane() {
		if (mainWindowPanel == null) {
			mainWindowPanel = new MainWindowPanel(configurationXMLReader, this);
			workflowPanel = mainWindowPanel.getWorkflowPanel(configurationXMLReader, this);
		}
		return mainWindowPanel;
	}

	private JMenuBar getMainMenuBar() {
		if (MainMenuBar == null) {
			MainMenuBar = new JMenuBar();
			MainMenuBar.add(getFileMenu());
			MainMenuBar.add(getBatchMenu());
			MainMenuBar.add(getEditMenu());
			MainMenuBar.add(getSettingsMenu());
			MainMenuBar.add(getHelpMenu());
		}
		return MainMenuBar;
	}
	private JMenu getEditMenu() {	
		if (editMenu == null) {
			editMenu = new JMenu();
			editMenu.setText("Edit");
			editMenu.add(getUndoMenuItem());
			editMenu.add(getRedoMenuItem());
			editMenu.addSeparator();
			editMenu.add(getShowHistoryMenuItem());
			editMenu.add(getClearHistoryMenuItem());
		}
		return editMenu;
	}

	private JMenu getBatchMenu() {
		if (batchMenu == null) {
			batchMenu = new JMenu();
			batchMenu.setText("Workflow");
			batchMenu.add(getAddJobMenu());
			batchMenu.add(getNewOutputFileMenuItem());
			batchMenu.add(getNewInputFileMenuItem());
			batchMenu.add(getEditNameMenuItem());
			batchMenu.add(getSetOutputFolderMenuItem());
			batchMenu.addSeparator();
			batchMenu.add(getExportScriptMenuItem());
			batchMenu.add(getExportScreenShotMenuItem());
		}
		return batchMenu;
	}

	private JMenu getFileMenu() {
		if (fileMenu == null) {
			fileMenu = new JMenu();
			fileMenu.setText("File");

			fileMenu.add(getNewBatchMenuItem());
			fileMenu.addSeparator();
			fileMenu.add(getLoadGraphMenuItem());
			fileMenu.add(getSaveGraphMenuItem());
			fileMenu.addSeparator();
			fileMenu.add(getQuitMenuItem());
		}
		return fileMenu;
	}

	private JMenuItem getRedoMenuItem() {
		if (redoMenuItem == null) {
			redoMenuItem = new JMenuItem();
			redoMenuItem.setText("Redo");
			redoMenuItem.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					workflowPanel.redo();		
					workflowPanel.repaint();		
				}
			});
		}
		return redoMenuItem;
	}

	public JMenuItem getUndoMenuItem() {
		if (undoMenuItem == null) {
			undoMenuItem = new JMenuItem();
			undoMenuItem.setText("Undo");
			undoMenuItem.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					workflowPanel.undo();		
					workflowPanel.repaint();
				}
			});
		}
		return undoMenuItem;
	}

	private JMenuItem getClearHistoryMenuItem() {
		if (clearHistoryMenuItem == null) {
			clearHistoryMenuItem = new JMenuItem();
			clearHistoryMenuItem.setText("Clear History");
			clearHistoryMenuItem.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					workflowPanel.clearHistory();
				}
			});
		}
		return clearHistoryMenuItem;
	}

	private JMenuItem getShowHistoryMenuItem() {
		if (showHistoryMenuItem == null) {
			showHistoryMenuItem = new JMenuItem();
			showHistoryMenuItem.setText("Show History");
			showHistoryMenuItem.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					workflowPanel.showHistory();					
				}
			});
		}
		return showHistoryMenuItem;
	}

	private JMenuItem getExportScriptMenuItem() {
		if (exportScriptMenuItem == null) {
			exportScriptMenuItem = new JMenuItem();
			exportScriptMenuItem.setText("Export Workflow as Script");
			exportScriptMenuItem.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					if (workflowPanel.check()==true){
						File targetName = DialogManager.showSaveDialog(workflowPanel.getOutputFolder(), mainWindowPanel.getWorkflowName() + ".sh", ".sh", me, true);
						if (targetName != null)	
							workflowPanel.exportWorkflowScript(targetName.getPath());						
					}}
			});
		}
		return exportScriptMenuItem;
	}

	private JMenuItem getExportScreenShotMenuItem() {
		if (exportScreenShotMenuItem == null) {
			exportScreenShotMenuItem = new JMenuItem();
			exportScreenShotMenuItem.setText("Save Screenshot");
			exportScreenShotMenuItem.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {

					File targetName = DialogManager.showSaveDialog(workflowPanel.getOutputFolder(), mainWindowPanel.getWorkflowName() + ".png", ".png", me, true);
					if (targetName != null)	
						workflowPanel.saveScreenshot(targetName.getPath());						
				}
			});
		}
		return exportScreenShotMenuItem;
	}

	private JMenuItem getLoadGraphMenuItem() {
		if (loadGraphMenuItem == null) {
			loadGraphMenuItem = new JMenuItem();
			loadGraphMenuItem.setText("Load Workflow");
			loadGraphMenuItem.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					File targetName = DialogManager.showOpenDialog(workflowPanel.getOutputFolder(), mainWindowPanel.getWorkflowName(), ".xml", me);
					if (targetName != null)
						try {
							me.openGraph(targetName.getPath(), true);
						} catch (Exception e1) {
							e1.printStackTrace();
						}						
				}
			});
		}
		return loadGraphMenuItem;
	}

	private JMenuItem getSaveGraphMenuItem() {
		if (saveGraphMenuItem == null) {
			saveGraphMenuItem = new JMenuItem();
			saveGraphMenuItem.setText("Save Workflow");
			saveGraphMenuItem.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					File targetName = DialogManager.showSaveDialog(workflowPanel.getOutputFolder(), mainWindowPanel.getWorkflowName(), ".xml", me, true);
					if (targetName != null)	
						workflowPanel.saveGraph(targetName.getPath());						
				}

			});
		}
		return saveGraphMenuItem;
	}

	private JMenu getSettingsMenu() {
		if (settingsMenu == null) {
			settingsMenu = new JMenu();
			settingsMenu.setText("Settings");
			settingsMenu.add(getResizeCheckBoxM());
			settingsMenu.add(getShowGridCheckBoxM());
			settingsMenu.add(getDisplayEdgeInformationsCheckBox());
			settingsMenu.add(getDisplayNodeInformationsCheckBox());
			settingsMenu.add(getDebugMenu());
		}
		workflowPanel.setPrintInformations(displayEdgeInformationsCheckBox.getState(), displayNodeInformationsCheckBox.getState(), alwaysDisplayOrderKeyCheckBox.getState(), displayAllEdgeInformationsCheckBox.getState(), displayAllNodeInformationsCheckBox.getState());
		return settingsMenu;
	}

	private JMenu getDebugMenu() {
		if (debugMenu == null) {
			debugMenu = new JMenu();
			debugMenu.setText("Debug");
			debugMenu.add(getAlwaysDisplayOrderKeyCheckBox());
			debugMenu.add(getDisplayAllEdgeInformationsCheckBox());
			debugMenu.add(getDisplayAllNodeInformationsCheckBox());
		}
		return debugMenu;
	}

	
	private JMenu getAddJobMenu() {
		if (AddJobMenu == null) {
			AddJobMenu = new JMenu();
			AddJobMenu.setText("Add Job");
			createAddJobMenu(AddJobMenu);
		}
		return AddJobMenu;
	}

	private JMenuItem getNewInputFileMenuItem() {
		if (newInputFileMenuItem == null) {
			newInputFileMenuItem = new JMenuItem();
			newInputFileMenuItem.setText("Add Input File");
			newInputFileMenuItem.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					workflowPanel.newInputFile();
				}
			});
		}
		return newInputFileMenuItem;
	}

	private JMenuItem getQuitMenuItem() {
		if (quitMenuItem == null) {
			quitMenuItem = new JMenuItem();
			quitMenuItem.setText("Quit");
			quitMenuItem.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					exitApplication(true);
				}

			});
		}
		return quitMenuItem;
	}

	private JMenuItem getNewOutputFileMenuItem() {
		if (newOutputFileMenuItem == null) {
			newOutputFileMenuItem = new JMenuItem();
			newOutputFileMenuItem.setText("Add Output File");
			newOutputFileMenuItem.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					workflowPanel.newOutputFile();
				}
			});
		}
		return newOutputFileMenuItem;
	}

	private JMenuItem getEditNameMenuItem() {
		if (editNameMenuItem == null) {
			editNameMenuItem = new JMenuItem();
			editNameMenuItem.setText("Change Workflow Name");
			editNameMenuItem.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					String newName = (String)JOptionPane.showInputDialog(workflowPanel, "Enter new Name:", "Edit Batch Name",JOptionPane.PLAIN_MESSAGE, null, null, workflowPanel.getBatchName());
					if ((newName != null) && (newName.length() > 0)) {
						workflowPanel.setBatchName(newName);
					}


				}
			});
		}
		return editNameMenuItem;
	}	

	private JMenuItem getSetOutputFolderMenuItem() {
		if (setOutputFolderMenuItem == null) {
			setOutputFolderMenuItem = new JMenuItem();
			setOutputFolderMenuItem.setText("Set Working Directory");
			setOutputFolderMenuItem.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					File newPath = DialogManager.showSelectFolderDialog(workflowPanel.getOutputFolder(), me);
					if (newPath != null){
						workflowPanel.setOutputFolder(newPath.getPath());
					}
				}
			});
		}
		return setOutputFolderMenuItem;
	}	

	private JMenuItem getNewBatchMenuItem() {
		if (newBatchMenuItem == null) {
			newBatchMenuItem = new JMenuItem();
			newBatchMenuItem.setText("New Workflow");
			newBatchMenuItem.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					boolean createNewWorkflow = true;
					
					int dialogResult = JOptionPane.showOptionDialog(me, "You are about to create a new workflow without saving the current Workflow.\nWould you like to do so now?", "Workflow not saved.", JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE, null, null, null);
					if (dialogResult == JOptionPane.YES_OPTION) {
						// Get file name to save
						File targetName = DialogManager.showSaveDialog(workflowPanel.getOutputFolder(), mainWindowPanel.getWorkflowName(), ".xml", me, true);
						if (targetName != null)	
							workflowPanel.saveGraph(targetName.getPath());						
						else
							// Only create new workflow, if a valid file name was selected
							createNewWorkflow = false;
					} 
								
					if (createNewWorkflow) {
						String newName = (String)JOptionPane.showInputDialog(workflowPanel, "Enter Batch Name:", "Enter Batch Name",JOptionPane.PLAIN_MESSAGE, null, null, "batch");
						if ((newName != null) && (newName.length() > 0)) {
							workflowPanel.newWorkflow(newName, "");
						}						
					}
				}
			});
		}
		return newBatchMenuItem;
	}	

	public JMenu getHelpMenu() {
		if(helpMenu==null){
			helpMenu = new JMenu("Help");
			helpMenu.add(getHelpContentItem());
			helpMenu.addSeparator();
			helpMenu.add(getAboutItem());
		}
		return helpMenu;
	}

	/**
	 * This method updates the title of the main window frame. It is called after changing / setting the batch name. 
	 * @param newName the new name of the batch
	 */
	protected void updateTitle(String newName) {
		if (!newName.equals(""))
			this.setTitle(AboutFrame.NAME + " - " + newName);	
		else
			this.setTitle(AboutFrame.NAME );	
	} 

	private void createAddJobMenu(JMenu pAddJobMenu) {
		Vector<String> items = configurationXMLReader.getProgrammNames();

		for (int curPos = 0; curPos < items.size(); curPos++) {
			String curProgrammNodeName = items.get(curPos);
			TaggedJMenuItem tmpProgrammMenuItem = new TaggedJMenuItem();
			tmpProgrammMenuItem.setText(curProgrammNodeName);
			tmpProgrammMenuItem.setID(curPos);
			tmpProgrammMenuItem.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					workflowPanel.newJob(((TaggedJMenuItem)e.getSource()).getID());
				}
			});		
			pAddJobMenu.add(tmpProgrammMenuItem);
		}
	}

	private JCheckBoxMenuItem getResizeCheckBoxM() {
		if (ResizeCheckBoxM == null) {
			ResizeCheckBoxM = new JCheckBoxMenuItem();
			ResizeCheckBoxM.setState(workflowPanel.getScaleOnResize());
			ResizeCheckBoxM.setText("Scale on Resize");
			ResizeCheckBoxM.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					workflowPanel.setScaleOnResize(ResizeCheckBoxM.getState());
				}
			});
		}
		return ResizeCheckBoxM;
	}
	
	private JCheckBoxMenuItem getDisplayEdgeInformationsCheckBox() {
		if (displayEdgeInformationsCheckBox == null) {
			displayEdgeInformationsCheckBox = new JCheckBoxMenuItem();
			displayEdgeInformationsCheckBox.setText("Display Edge Information");
			displayEdgeInformationsCheckBox.setState(true);
			displayEdgeInformationsCheckBox.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					workflowPanel.setPrintInformations(displayEdgeInformationsCheckBox.getState(), displayNodeInformationsCheckBox.getState(), alwaysDisplayOrderKeyCheckBox.getState(), displayAllEdgeInformationsCheckBox.getState(), displayAllNodeInformationsCheckBox.getState());
				}
			});
		}
		return displayEdgeInformationsCheckBox;
	}
	
	private JCheckBoxMenuItem getDisplayNodeInformationsCheckBox() {
		if (displayNodeInformationsCheckBox == null) {
			displayNodeInformationsCheckBox = new JCheckBoxMenuItem();
			displayNodeInformationsCheckBox.setText("Display Node Information");
			displayNodeInformationsCheckBox.setState(true);
			displayNodeInformationsCheckBox.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {					
					workflowPanel.setPrintInformations(displayEdgeInformationsCheckBox.getState(), displayNodeInformationsCheckBox.getState(), alwaysDisplayOrderKeyCheckBox.getState(), displayAllEdgeInformationsCheckBox.getState(), displayAllNodeInformationsCheckBox.getState());
					
				}
			});
		}
		return displayNodeInformationsCheckBox;
	}
	
	private JCheckBoxMenuItem getAlwaysDisplayOrderKeyCheckBox() {
		if (alwaysDisplayOrderKeyCheckBox == null) {
			alwaysDisplayOrderKeyCheckBox = new JCheckBoxMenuItem();
			alwaysDisplayOrderKeyCheckBox.setText("Always Display Order Keys");
			alwaysDisplayOrderKeyCheckBox.setState(false);
			alwaysDisplayOrderKeyCheckBox.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					workflowPanel.setPrintInformations(displayEdgeInformationsCheckBox.getState(), displayNodeInformationsCheckBox.getState(), alwaysDisplayOrderKeyCheckBox.getState(), displayAllEdgeInformationsCheckBox.getState(), displayAllNodeInformationsCheckBox.getState());			
				}
			});
		}
		return alwaysDisplayOrderKeyCheckBox;
	}
	
	private JCheckBoxMenuItem getDisplayAllEdgeInformationsCheckBox() {
		if (displayAllEdgeInformationsCheckBox == null) {
			displayAllEdgeInformationsCheckBox = new JCheckBoxMenuItem();
			displayAllEdgeInformationsCheckBox.setText("Display All Edge Information");
			displayAllEdgeInformationsCheckBox.setState(false);
			displayAllEdgeInformationsCheckBox.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					workflowPanel.setPrintInformations(displayEdgeInformationsCheckBox.getState(), displayNodeInformationsCheckBox.getState(), alwaysDisplayOrderKeyCheckBox.getState(), displayAllEdgeInformationsCheckBox.getState(), displayAllNodeInformationsCheckBox.getState());
		
				}
			});
		}
		return displayAllEdgeInformationsCheckBox;
	}
	
	private JCheckBoxMenuItem getDisplayAllNodeInformationsCheckBox() {
		if (displayAllNodeInformationsCheckBox == null) {
			displayAllNodeInformationsCheckBox = new JCheckBoxMenuItem();
			displayAllNodeInformationsCheckBox.setText("Display All Node Information");
			displayAllNodeInformationsCheckBox.setState(false);
			displayAllNodeInformationsCheckBox.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					workflowPanel.setPrintInformations(displayEdgeInformationsCheckBox.getState(), displayNodeInformationsCheckBox.getState(), alwaysDisplayOrderKeyCheckBox.getState(), displayAllEdgeInformationsCheckBox.getState(), displayAllNodeInformationsCheckBox.getState());
			
				}
			});
		}
		return displayAllNodeInformationsCheckBox;
	}


	
	private JCheckBoxMenuItem getShowGridCheckBoxM() {
		if (ShowGridCheckBoxM == null) {
			ShowGridCheckBoxM = new JCheckBoxMenuItem();
			ShowGridCheckBoxM.setText("Display Grid");
			ShowGridCheckBoxM.setState(true);
			ShowGridCheckBoxM.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					workflowPanel.setShowGrid(ShowGridCheckBoxM.getState());
				}
			});
			workflowPanel.setShowGrid(ShowGridCheckBoxM.getState());
		}
		return ShowGridCheckBoxM;
	}

	/**
	 * This method returns if the checkbox "Show Grid" is marked or not.
	 */
	public boolean getShowGridStatus(){
		return ShowGridCheckBoxM.getState();
	}

	/**
	 * This method opens the workflow specified by the passed filename.
	 * @param checkFilesOnLoad 
	 */
	public void openGraph(String pFileName, boolean checkFilesOnLoad) {
		try {
			workflowPanel.loadGraph(pFileName, checkFilesOnLoad);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private JMenuItem getHelpContentItem() {
		if(helpContentItem==null){
			helpContentItem = new JMenuItem("Help");
			helpContentItem.addActionListener(new ActionListener() {

				@Override
				public void actionPerformed(ActionEvent e) {
					HelpFrame helpFrame = new HelpFrame(450, 300);
					helpFrame.setVisible(true);
				}
			});
		}
		return helpContentItem;
	}

	private JMenuItem getAboutItem() {
		if(aboutItem==null){
			aboutItem=new JMenuItem("About ProFiDo");
			aboutItem.addActionListener(new ActionListener() {

				public void actionPerformed(ActionEvent e) {
					AboutFrame aboutFrame = new AboutFrame(700, 710);
					aboutFrame.setVisible(true);

				}
			});
		}
		return aboutItem;
	}

	public void exportWorkflowScript(String exportFilenName) {
		workflowPanel.exportWorkflowScript(exportFilenName);

	}

	public WorkflowPanel getWorkflowPanel() {
		return workflowPanel;
	}	


} 
