/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package gui;

import gui.helper.DialogManager;
import gui.helper.Grid;
import gui.helper.RestoreToolTipTimingsTimerTask;
import gui.helper.status;
import helper.ConverterSignature;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Timer;
import java.util.Vector;

import javax.imageio.ImageIO;
import javax.swing.AbstractButton;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.SwingUtilities;
import javax.swing.ToolTipManager;

import com.sun.corba.se.impl.orbutil.graph.Node;

import core.TypedFile;
import core.graph.BasicNode;
import core.graph.Edge;
import core.graph.FileNode;
import core.graph.Graph;
import core.graph.GraphNode;
import core.graph.InputFileNode;
import core.graph.JobNode;
import core.graph.OutputFileNode;
import core.graph.PointNode;
import core.history.ChangeNodeNameHistoryObject;
import core.history.ChangePointNodeUserPositioned;
import core.history.ConnectNodeHistoryObject;
import core.history.MoveNodeHistoryObject;
import core.history.NewNodeHistoryObject;
import core.history.NewPointNodeHistoryObject;
import core.history.RemoveConnectionHistoryObject;
import core.history.RemoveNodeHistoryObject;
import core.history.SetConverterIDHistoryObject;
import core.history.SetExternalFileNameHistoryObject;
import core.storage.ConfigurationXMLReader;
import core.storage.GraphLoader;

/**
 * This class defines the properties of the workspace. It displays
 * the workflow and manages all mouse interactions with it.
 */

public class WorkflowPanel extends JPanel {
	private static final long serialVersionUID = 1L;

	private int mousePressedX;
	private int mousePressedY;

	/**
	 * Timer used to restore tooltip settings
	 * when needed.
	 */
	private Timer toolTipRestoreTimer;

	/** 
	 * Reference to a JLabel where status informations can be written on.
	 */
	private JLabel statusjLabel;

	/**
	 * Reference to a JLabel where informations can be written on.
	 */
	private JLabel graphInfojLabel;


	/**
	 * Reference to the graph representing the workflow.
	 */
	private Graph graph; 

	/**
	 * Reference to the currently marked node.
	 */
	private BasicNode markedNode;

	/**
	 * Reference to the node located at the current mouse position.
	 */
	private BasicNode nodeAtMouse;

	/**
	 * Manages the status of this panel in order to allow more 
	 * sophisticated and multi-click user interactions. 
	 */
	private status curStatus = status.FREE;

	/**
	 * Reference to the grid used to align this graph.
	 */
	private Grid grid;

	/**
	 * Reference to the xmlReader used for accessing the configuration file.
	 */
	private ConfigurationXMLReader xmlReader;

	/**
	 * Stores the font used for output. 
	 */
	private Font font;

	/**
	 * Stores the current selection rectangle.
	 */
	private Rectangle selectionRectangle;

	private WorkflowPanel me;
	private JPopupMenu edgeAktionsPopupMenu;
	private JMenuItem deleteEdgeMenuItem;
	private JCheckBoxMenuItem pointNodeAutoPositionCheckBoxMenuItem;
	private JMenuItem deletePointNodeMenuItem;
	private JMenuItem editEdgeMenuItem;
	private JMenuItem setPointNodeMenuItem;
	private JMenuItem plotMenuItem;
	private JPopupMenu jobNodePopupMenu;
	private JMenuItem deleteFileNodeMenuItem;
	private JMenuItem connectFileNodeMenuItem;
	private JPopupMenu fileNodePopupMenu;
	private JMenuItem changeFileNodeFileMenuItem;
	private JMenuItem editJobNodeMenuItem;
	private JMenuItem changeJobNodeNameMenuItem;
	private JMenuItem deleteJobNodeMenuItem;
	private JMenuItem connectJobNodeMenuItem;
	private JMenuItem showJobNodeCommandLineCallMenuItem;

	private ShowHistoryFrame showHistoryFrame;

	private int toolTipDismissDelay;
	private int toolTipInitialDelay;
	private int toolTipReshowDelay;


	private MainWindowFrame mainFrame;

	private JMenuItem changeFileNodeFileTypeMenuItem;

	private boolean tooltipVisible;


	/**
	 * Creates and initializes the panel.
	 * 
	 * @param pStatusjLabel the status label
	 * @param pGraphInfojLabel the graph information label
	 * @param pXMLReader the XML reader
	 */
	public WorkflowPanel(JLabel pStatusjLabel, JLabel pGraphInfojLabel, ConfigurationXMLReader xmlReader, MainWindowFrame mainFrame) {
		super();
		initialize();
		me = this;

		this.setSize(300, 300);

		statusjLabel = pStatusjLabel;
		this.mainFrame = mainFrame;
		this.graphInfojLabel = pGraphInfojLabel;
		this.xmlReader = xmlReader;
		this.newWorkflow("untitled", "/home/");
		grid = new Grid(6, 12, 100, 50, false);
		toolTipDismissDelay = ToolTipManager.sharedInstance().getDismissDelay();
		toolTipInitialDelay = ToolTipManager.sharedInstance().getInitialDelay();
		toolTipReshowDelay = toolTipInitialDelay;
		ToolTipManager.sharedInstance().setReshowDelay(toolTipReshowDelay); // ToolTipManager.sharedInstance().getReshowDelay();
		ToolTipManager.sharedInstance().registerComponent(this);

		toolTipRestoreTimer = new Timer();
	}

	@Override
	public String getToolTipText(MouseEvent e) {
		e = normalizeMouseEvent(e);
		BasicNode tmpNode = graph.getBasicNodeAtPosition(e.getX(), e.getY(), grid);

		if ((tmpNode != null) && (!tmpNode.getTooltipText().equals(""))) {
			tooltipVisible = true;
			return tmpNode.getTooltipText();					
		} else {		
			return null;
		}
	}

	private MouseEvent normalizeMouseEvent(MouseEvent e) {
		int mouseX = e.getX();
		int mouseY = e.getY();

		// If outside, move to border
		if (mouseX < 0)
			mouseX = 0;
		if (mouseX > getWidth())
			mouseX = getWidth();
		if (mouseY < 0)
			mouseY = 0;
		if (mouseY > getHeight())
			mouseY = getHeight();

		return new MouseEvent((Component)e.getSource(), e.getID(), e.getWhen(), e.getModifiers(), mouseX, mouseY, e.getXOnScreen(), e.getYOnScreen(), e.getClickCount(), false, e.getButton());
	}

	private void changeMarkedNode(BasicNode newMarkedNode) {
		if (markedNode != null)
			markedNode.setMarked(false);
		this.markedNode = newMarkedNode;
		if (markedNode != null)
			markedNode.setMarked(true);
		repaint();
	}

	/**
	 * Initializes all components used by this panel and adds them to 
	 * its content pane.
	 */
	private void initialize() {
		showHistoryFrame = new ShowHistoryFrame(null, this);			

		font = new Font("Arial", Font.PLAIN, 12);

		edgeAktionsPopupMenu = getEdgeAktionsPopupMenu();
		jobNodePopupMenu = getJobNodePopupMenu();

		this.setLayout(null);

		this.addComponentListener(new ComponentAdapter() {
			public void componentResized(ComponentEvent evt) {
				if (grid != null) {
					grid.setSize(getWidth(), getHeight());
					graph.realignToGrid(grid);
					me.repaint();
				}
			}
		});

		// ---------------------------------------------------------------------------------------
		// * All action listeners for clicks and movements follow
		// ---------------------------------------------------------------------------------------

		this.addMouseListener(new java.awt.event.MouseAdapter() { 
			/**
			 * This method reacts on pressing the mouse button.
			 * By pressing the left mouse button a new node can be placed at the 
			 * clicked position. 
			 * If the left button of the mouse is pressed and its position is in 
			 * the click area of a node then this node is marked.
			 * If the right button of the mouse is pressed and its position is in 
			 * the click area of an workflow element (node, edge) a pop up menu is 
			 * displayed for this element. 
			 */

			public void mousePressed(java.awt.event.MouseEvent e) {
				e = normalizeMouseEvent(e);

				if (e.getButton() == java.awt.event.MouseEvent.BUTTON1) {
					if (curStatus == status.FREE) {
						// Currently no action is taking place
						// -> click marks node
						me.changeMarkedNode(graph.getBasicNodeAtPosition(e.getX(), e.getY(), grid));
						//selectionRectangle = new Rectangle(0, 0, 0, 0);
						mousePressedX = e.getX();
						mousePressedY = e.getY();
					} else if (curStatus == status.NEW) {
						// Currently waiting for positioning of a new node
						// -> Click set position of new node and finishes the adding of it
						markedNode.moveToPixelCoordinates(e.getX(), e.getY(), true);
						repaint();

						graph.addNewAction(new NewNodeHistoryObject(
								(GraphNode)markedNode, null, graph));
						graph.performLastAction();

						curStatus = status.FREE;
						statusjLabel.setText("");
						if (markedNode instanceof JobNode)
							editJob((JobNode)markedNode);
					}
				} else if (e.getButton() == java.awt.event.MouseEvent.BUTTON3) {
					nodeAtMouse = graph.getBasicNodeAtPosition(e.getX(), e.getY(),
							grid);
					if (nodeAtMouse != null) {
						if (nodeAtMouse instanceof JobNode)
							me.getJobNodePopupMenu()
							.show(me, e.getX(), e.getY());
						else if (nodeAtMouse instanceof FileNode)
							me.getFileNodePopupMenu().show(me, e.getX(),
									e.getY());
						else if (nodeAtMouse instanceof PointNode) {
							me.updateEdgePopupMenu(((PointNode)nodeAtMouse));
							me.getEdgeAktionsPopupMenu().show(me, e.getX(), e.getY());
						}
					}
				}

				repaint();
			}

			/**
			 * This method reacts on releasing the mouse button.
			 * The node which is dropped is placed at the position where the mouse 
			 * button is released.
			 */
			public void mouseReleased(java.awt.event.MouseEvent e) {
				e = normalizeMouseEvent(e);

				if ((e.getButton() == java.awt.event.MouseEvent.BUTTON1) && (curStatus == status.FREE)) {
					if (markedNode != null) {
						// Move selected node
						Point newGridPosition = grid.getNearestCell(e.getX(), e.getY());
						Point oldGridPosition = markedNode.getGridPosition();
						if ((!newGridPosition.equals(oldGridPosition)) && (!(markedNode instanceof PointNode) || (((PointNode) markedNode).isUserPositioned()))) {
							graph.addNewAction(new MoveNodeHistoryObject(graph, markedNode, oldGridPosition, newGridPosition));
							graph.performLastAction();								
						} else {
							// Node was not move to another cell but might be not in center anymore, so move back
							markedNode.realignToGrid();
						}
					} else {
						//selectionRectangle = null;
					}
					repaint();
				}
			}
		});

		this.addMouseMotionListener(new java.awt.event.MouseMotionAdapter() {

			/**
			 * This method reacts on moving the mouse while the key is pressed.
			 * The node which is dragged can be moved on the workspace.
			 */
			public void mouseDragged(java.awt.event.MouseEvent e) {
				e = normalizeMouseEvent(e);
				// BUTTON1_MASK is needed instead of BUTTON1 in order
				// to work properly under windows.
				if ((e.getModifiers() & MouseEvent.BUTTON1_MASK) != 0) {
					if ((markedNode != null) ) {
						// Move node without realignen it to the grid
						markedNode.moveToPixelCoordinates(e.getX(), e.getY(), false);
					} /*else if (selectionRectangle != null) {
						int startX = mousePressedX;
						int startY = mousePressedY;
						int width = Math.abs(e.getX() - startX);
						int heigth = Math.abs(e.getY() - startY);

						if (e.getX() < startX) 
							startX = e.getX();					
						if (e.getY() < startY) 
							startY = e.getY();				

						selectionRectangle.setBounds(startX, startY, width, heigth);
					}*/
					repaint();
				}
			}

			public void mouseMoved(java.awt.event.MouseEvent e) {
				e = normalizeMouseEvent(e);

				// Ensure that all currently visible tooltip are dismissed, 
				// when the mouse is moved, in order to preserve correct
				// timing when displaying further tooltips.
				if (tooltipVisible) {
					tooltipVisible = false;		
					ToolTipManager.sharedInstance().setDismissDelay(1);	// Dismiss tooltip instantly
					// Restore tooltip timing
					toolTipRestoreTimer.schedule(new RestoreToolTipTimingsTimerTask(
							me.toolTipInitialDelay, me.toolTipReshowDelay, me.toolTipDismissDelay), 5);  
				}
			}
		});


	}

	/**
	 * Updates the menu items of the edge popup menu according to the
	 * properties of the passed point node.
	 */
	protected void updateEdgePopupMenu(PointNode pointNode) {
		if ((pointNodeAutoPositionCheckBoxMenuItem != null) && (pointNode != null)) {
			this.pointNodeAutoPositionCheckBoxMenuItem.setSelected(!pointNode.isUserPositioned());
		}
	}

	private JPopupMenu getEdgeAktionsPopupMenu() {
		if (edgeAktionsPopupMenu == null) {
			edgeAktionsPopupMenu = new JPopupMenu();

			deleteEdgeMenuItem = new JMenuItem();
			deleteEdgeMenuItem.setText("Delete Connection");
			deleteEdgeMenuItem
			.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					me.deleteEdge();
				}
			});

			editEdgeMenuItem = new JMenuItem();
			editEdgeMenuItem.setText("Edit Connection");
			editEdgeMenuItem
			.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					me.editEdge();
				}
			});

			setPointNodeMenuItem = new JMenuItem();
			setPointNodeMenuItem.setText("Add EdgePoint");
			setPointNodeMenuItem
			.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					me.addPointNode();

				}
			});

			deletePointNodeMenuItem = new JMenuItem();
			deletePointNodeMenuItem.setText("Delete EdgePoint");
			deletePointNodeMenuItem
			.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					me.deleteNode();
				}
			});

			pointNodeAutoPositionCheckBoxMenuItem = new JCheckBoxMenuItem();
			pointNodeAutoPositionCheckBoxMenuItem.setText("Auto Position");
			pointNodeAutoPositionCheckBoxMenuItem.setSelected(false);
			pointNodeAutoPositionCheckBoxMenuItem
			.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					me.changeAutoPositionOfPointNode(pointNodeAutoPositionCheckBoxMenuItem.isSelected());
				}
			});

			plotMenuItem = new JMenuItem();
			plotMenuItem.setText("Plotter");
			plotMenuItem
			.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {

					me.openPlotter();

				}
			});

			edgeAktionsPopupMenu.add(deleteEdgeMenuItem);
			edgeAktionsPopupMenu.add(editEdgeMenuItem);
			edgeAktionsPopupMenu.add(setPointNodeMenuItem);
			edgeAktionsPopupMenu.add(deletePointNodeMenuItem);
			edgeAktionsPopupMenu.add(pointNodeAutoPositionCheckBoxMenuItem);

			edgeAktionsPopupMenu.add(plotMenuItem);

		}
		return edgeAktionsPopupMenu;
	}

	protected void changeAutoPositionOfPointNode(boolean selected) {
		if (nodeAtMouse instanceof PointNode) {
			boolean curState = ((PointNode) nodeAtMouse).isUserPositioned();
			// If value changed
			if (curState == selected) {
				graph.addNewAction(new ChangePointNodeUserPositioned((PointNode) nodeAtMouse, !selected));
				graph.performLastAction();
			}
			repaint();			
		}
	}

	private JPopupMenu getJobNodePopupMenu() {
		if (jobNodePopupMenu == null) {
			jobNodePopupMenu = new JPopupMenu();

			if (connectJobNodeMenuItem == null) {
				connectJobNodeMenuItem = new JMenuItem();
				connectJobNodeMenuItem
				.setText("Connect output from selected node to this");
				connectJobNodeMenuItem
				.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(
							java.awt.event.ActionEvent e) {
						me.connectNode();
						changeMarkedNode(nodeAtMouse);
					}
				});
			}

			if (deleteJobNodeMenuItem == null) {
				deleteJobNodeMenuItem = new JMenuItem();
				deleteJobNodeMenuItem.setText("Delete");
				deleteJobNodeMenuItem
				.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(
							java.awt.event.ActionEvent e) {
						me.deleteNode();
					}
				});
			}

			if (changeJobNodeNameMenuItem == null) {
				changeJobNodeNameMenuItem = new JMenuItem();
				changeJobNodeNameMenuItem.setText("Change Name");
				changeJobNodeNameMenuItem
				.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(
							java.awt.event.ActionEvent e) {
						changeMarkedNode(nodeAtMouse);
						me.changeNodeName();
					}
				});
			}

			if (editJobNodeMenuItem == null) {
				editJobNodeMenuItem = new JMenuItem();
				editJobNodeMenuItem.setText("Edit");
				editJobNodeMenuItem
				.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(
							java.awt.event.ActionEvent e) {
						if (nodeAtMouse != null)
							me.editJob((JobNode)nodeAtMouse);
					}
				});
			}

			if (showJobNodeCommandLineCallMenuItem == null) {
				showJobNodeCommandLineCallMenuItem = new JMenuItem();
				showJobNodeCommandLineCallMenuItem
				.setText("Show Commandline Call");
				showJobNodeCommandLineCallMenuItem
				.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(
							java.awt.event.ActionEvent e) {
						if ((nodeAtMouse != null) && (nodeAtMouse instanceof GraphNode)) {
							System.out.println(((GraphNode)nodeAtMouse)
									.getCommandLineCall("", ""));
						}
						changeMarkedNode(nodeAtMouse);
					}
				});
			}

			jobNodePopupMenu.add(connectJobNodeMenuItem);
			jobNodePopupMenu.add(deleteJobNodeMenuItem);
			jobNodePopupMenu.add(editJobNodeMenuItem);
			jobNodePopupMenu.add(changeJobNodeNameMenuItem);
			jobNodePopupMenu.add(showJobNodeCommandLineCallMenuItem);

		}
		return jobNodePopupMenu;
	}

	private JPopupMenu getFileNodePopupMenu() {
		if (fileNodePopupMenu == null) {
			fileNodePopupMenu = new JPopupMenu();

			if (connectFileNodeMenuItem == null) {
				connectFileNodeMenuItem = new JMenuItem();
				connectFileNodeMenuItem
				.setText("Connect output from selected node to this");
				connectFileNodeMenuItem
				.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(
							java.awt.event.ActionEvent e) {
						me.connectNode();
						changeMarkedNode(nodeAtMouse);
					}
				});
			}

			if (deleteFileNodeMenuItem == null) {
				deleteFileNodeMenuItem = new JMenuItem();
				deleteFileNodeMenuItem.setText("Delete");
				deleteFileNodeMenuItem
				.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(
							java.awt.event.ActionEvent e) {
						me.deleteNode();
					}
				});
			}

			if (changeFileNodeFileMenuItem == null) {
				changeFileNodeFileMenuItem = new JMenuItem();
				changeFileNodeFileMenuItem.setText("Change External File Name");
				changeFileNodeFileMenuItem
				.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(
							java.awt.event.ActionEvent e) {
						me.changeFileNodeFileName();
						changeMarkedNode(nodeAtMouse);
					}
				});
			}

			if (changeFileNodeFileTypeMenuItem == null) {
				changeFileNodeFileTypeMenuItem = new JMenuItem();
				changeFileNodeFileTypeMenuItem.setText("Change External File Type");
				changeFileNodeFileTypeMenuItem
				.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(
							java.awt.event.ActionEvent e) {
						me.changeFileNodeFileType(true, false);
						changeMarkedNode(nodeAtMouse);
					}
				});
			}

			/*
			if (changeFileNodeTypeMenuItem == null) {
				changeFileNodeTypeMenuItem = new JMenuItem();
				changeFileNodeTypeMenuItem.setText("Change Type");
				changeFileNodeTypeMenuItem
						.addActionListener(new java.awt.event.ActionListener() {
							public void actionPerformed(
									java.awt.event.ActionEvent e) {
								me.changeFileNodeType();
							}
						});
			}
			 */

			/*
			if (changeFileNodeNameMenuItem == null) {
				changeFileNodeNameMenuItem = new JMenuItem();
				changeFileNodeNameMenuItem.setText("Change name");
				changeFileNodeNameMenuItem
						.addActionListener(new java.awt.event.ActionListener() {
							public void actionPerformed(
									java.awt.event.ActionEvent e) {
								me.changeNodeName();
							}
						});
			}
			 */
			fileNodePopupMenu.add(connectFileNodeMenuItem);
			fileNodePopupMenu.add(deleteFileNodeMenuItem);
			fileNodePopupMenu.add(changeFileNodeFileMenuItem);
			fileNodePopupMenu.add(changeFileNodeFileTypeMenuItem);

		}
		return fileNodePopupMenu;
	}


	// -------------------------------------------------------------------------------
	// * Methods for drawing follow
	// -------------------------------------------------------------------------------

	protected void paintComponent(Graphics g) {
		super.paintComponent(g);

		if (g != null) {
			drawAll(g);
		}
	}

	/**
	 * This method draws the complete panel and the graph on it on the
	 * passed graphics object.
	 */
	private void drawAll(Graphics g) {
		g.setFont(font);

		// Clear
		g.setColor(Color.white);
		g.fillRect(0, 0, this.getWidth(), this.getHeight());

		// Draw graph and grid
		grid.draw(g);
		graph.draw(g, grid);
		/*
		// Draw selection rectangle
		if ((selectionRectangle != null) && (selectionRectangle.getWidth() > 0) && (selectionRectangle.getHeight() > 0)) {
			AlphaComposite ta = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.1f);
			Graphics2D g2d = (Graphics2D)g;
			g2d.setComposite(ta);
			g2d.setColor(Color.BLUE);
			g2d.fill(selectionRectangle);
			g2d.setComposite(AlphaComposite.SrcOver);
			g2d.setColor(Color.GRAY);
			g2d.draw(selectionRectangle);
		}
		 */
	}

	/**
	 * Notify this panel that a repaint is necessary.
	 */
	public void notify(Object caller) {
		// TODO Remove if possible
		this.repaint();
	}

	/**
	 * This method creates and displayes the file frame after a click on the menu 
	 * item "Edit Connection".
	 */
	protected void editEdge() {
		if (nodeAtMouse instanceof PointNode) {
			Edge pointNodesEdge = ((PointNode) nodeAtMouse).getEdge();

			JFrame editFileFrame = new EditFileFrame(pointNodesEdge, this);

			editFileFrame.setLocation(this.getX() + this.getWidth() + 20,
					this.getY());
			editFileFrame.setVisible(true);
		}
	}

	/**
	 * This method sets a further PointNode on the edge after a click on the menu item 
	 * "Set Edge Point" and adds this action to the history.
	 */
	protected void addPointNode() {
		if (nodeAtMouse instanceof PointNode) {
			Edge pointNodesEdge = ((PointNode)nodeAtMouse).getEdge();
			graph.addNewAction(new NewPointNodeHistoryObject(pointNodesEdge, (PointNode) nodeAtMouse, graph));
			graph.performLastAction();
		}
	}

	/**
	 * This method opens the plotter panel after a click on the "Plot" menu item.
	 */
	protected void openPlotter(){
		if (nodeAtMouse instanceof PointNode) {
			Edge pointNodesEdge = ((PointNode) nodeAtMouse).getEdge();			
			JFrame EdgePlotterFrame = new EdgePlotterFrame(xmlReader,graph,pointNodesEdge,this);

			EdgePlotterFrame.setLocation(this.getX() + this.getWidth() + 20,
					this.getY());
			EdgePlotterFrame.setVisible(true);			
		}
	}

	/**
	 * This method checks if the connection between two nodes is valid.
	 */
	private boolean isConnectionIsValid(GraphNode from, GraphNode to) {
		boolean result = false;
		String message = "Connection between " + from.getName() + " and "
				+ to.getName() + " is not valid.";

		if (from == to) {
			result = false;
			message = message + " A Node should not be connected to itself.";
		} else {
			if (from instanceof JobNode) {

				// Two job nodes are fine
				if (to instanceof JobNode) {
					result = true;

				}
				else if (to instanceof OutputFileNode) {

					if (to.getIncomingEdges().size()<1){
						result = true;
					}
					else {
						result = false;
						message = message
								+ " An OutputFileNode can only have one input.";
					}
				}
				else {
					result = false;
					message = message
							+ " A JobNode can only be connected to another JobNode or an OutputFileNode.";
				}
			}



			else if (from instanceof FileNode) {
				if (to instanceof JobNode) {
					if (from instanceof InputFileNode) {
						result = true;
					} else {
						result = false;
					}
				} else {
					result = false;
					message = message
							+ " A FileNode can only be connected to a JobNode.";
				}
			}	 
		}

		// Display error if any
		if (!result)
			JOptionPane.showMessageDialog(this, message, "Error", JOptionPane.ERROR_MESSAGE);


		return result;
	}

	// TODO Merge methods
	/**
	 * This methods shows a dialog prompting the user to select the input type (and
	 * the corresponding converter) for the passed fileName. By passing an additional
	 * filter the possible types can be restricted. If showAlways is unset, the 
	 * dialog is only showed if several types match the extension of the passed fileName.
	 */
	private int selectInputConverterID(Vector<String> possibleTypes, int oldConverterID, String fileName, boolean showAlways) {
		String fileExtension = getExtension(new File(fileName));
		HashMap<Integer, ConverterSignature> inputConverterSignatures = xmlReader.getInputConverterSignatures(possibleTypes);
		int idOfFirstInputConverterWithMatchingInputSuffix = -1;
		int matchingInputConverterCount = 0;
		for (int curID : inputConverterSignatures.keySet()) {
			if (inputConverterSignatures.get(curID).getInputExtension().equals(fileExtension)) {
				matchingInputConverterCount++;

				if (idOfFirstInputConverterWithMatchingInputSuffix == -1)
					idOfFirstInputConverterWithMatchingInputSuffix = curID;
			}
		}

		if ((idOfFirstInputConverterWithMatchingInputSuffix != -1) && (!showAlways) && (matchingInputConverterCount == 1)) {
			return idOfFirstInputConverterWithMatchingInputSuffix;
		} else {
			// Create entries for selection box
			Vector<String> inputTypes = new Vector<String>();
			for (int curID : inputConverterSignatures.keySet()) {
				inputTypes.add(inputConverterSignatures.get(curID).getConversionSignature());
			}

			// Determine default type
			String defaultType = "";
			if (oldConverterID != -1) {
				defaultType = inputConverterSignatures.get(oldConverterID).getConversionSignature();			
			} else {
				if (idOfFirstInputConverterWithMatchingInputSuffix != -1)
					defaultType = inputConverterSignatures.get(idOfFirstInputConverterWithMatchingInputSuffix).getConversionSignature();				
			}

			// Show dialog
			String selectedSignature = DialogManager.showSelectionDialog(inputTypes, defaultType, "Select Input Type","Please select type of external (input) file.", this);
			if (selectedSignature != null) {
				// Find id of selected type
				for (int curID : inputConverterSignatures.keySet()) {
					if (selectedSignature.equals(inputConverterSignatures.get(curID).getConversionSignature()))
						return curID;
				}						
			}

			return -1;

		}
	}


	/**
	 * This methods shows a dialog prompting the user to select the output type (and
	 * the corresponding converter) for the passed fileName. By passing an additional
	 * filter the possible types can be restricted. If showAlways is unset, the 
	 * dialog is only showed if several types match the extension of the passed fileName.
	 */
	private int selectOutputConverterID(Vector<String> possibleTypes, int oldConverterID, String fileName, boolean showAlways) {	
		String fileExtension = getExtension(new File(fileName));
		HashMap<Integer, ConverterSignature> outputConverterSignatures = xmlReader.getOutputConverterSignatures(possibleTypes);
		int idOfFirstOutputConverterWithMatchingOutputSuffix = -1;
		int matchingOutputConverterCount = 0;
		for (int curID : outputConverterSignatures.keySet()) {
			if (outputConverterSignatures.get(curID).getOutputExtension().equals(fileExtension)) {
				matchingOutputConverterCount++;

				if (idOfFirstOutputConverterWithMatchingOutputSuffix == -1)
					idOfFirstOutputConverterWithMatchingOutputSuffix = curID;
			}
		}

		if ((idOfFirstOutputConverterWithMatchingOutputSuffix != -1) && (!showAlways) && (matchingOutputConverterCount == 1)) {
			return idOfFirstOutputConverterWithMatchingOutputSuffix;
		} else {
			// Collect possible type
			Vector<String> outputTypes = new Vector<String>();
			for (int curID : outputConverterSignatures.keySet()) {
				outputTypes.add(outputConverterSignatures.get(curID).getConversionSignature());
			}

			// Determine default type
			String defaultType = "";
			if (oldConverterID != -1) {
				defaultType = outputConverterSignatures.get(oldConverterID).getConversionSignature();
			} else {
				if (idOfFirstOutputConverterWithMatchingOutputSuffix != -1)
					defaultType = outputConverterSignatures.get(idOfFirstOutputConverterWithMatchingOutputSuffix).getConversionSignature();				
			}

			// Show dialog
			String selectedSignature = DialogManager.showSelectionDialog(outputTypes, defaultType, "Select Output Type","Please select type of external (output) file.", this);
			if (selectedSignature != null) {
				// Find id of selected type
				for (int curID : outputConverterSignatures.keySet()) {
					if (selectedSignature.equals(outputConverterSignatures.get(curID).getConversionSignature()))
						return curID;
				}						
			}

			return -1;

		}
	}


	/**
	 * This method is responsible for the creation of an new InputNode. It is called after a click 
	 * on the main window frame menu item "Add Input File".
	 */
	public void newInputFile() {
		File selectedFile = DialogManager.showOpenDialog(me.getOutputFolder(), "", ".*", me);
		if (selectedFile != null) {
			int converterID = selectInputConverterID(null, -1, selectedFile.getName(), false);

			TypedFile externalFile = new TypedFile(selectedFile.getPath());
			if (converterID != -1) {
				newAbstractNode(new InputFileNode(0, 0, xmlReader, grid, converterID, graph, externalFile));
			}
		}
	}

	/**
	 * This method is responsible for the creation of an new OutputNode. It is called after a click 
	 * on the main window frame menu item "Add Output File".
	 */
	public void newOutputFile() {
		File selectedFile = DialogManager.showSaveDialog(me.getOutputFolder(), "", ".*", me, false);
		if (selectedFile != null) {
			int converterID = selectOutputConverterID(null, -1, selectedFile.getName(), false);

			TypedFile externalFile = new TypedFile(selectedFile.getPath());
			if(converterID != -1) {
				newAbstractNode(new OutputFileNode(0, 0, xmlReader, grid, converterID, graph, externalFile));
			}
		}
	}


	/**
	 * This method changes the file type of a FileNode.
	 */
	protected void changeFileNodeFileType(boolean showAlways, boolean ignoreOldConverterID) {
		if ((nodeAtMouse != null) && (nodeAtMouse instanceof FileNode)) {
			// Select new file name
			int newConverterID = -1;
			int curConverterID = -1;
			TypedFile curFileName = ((FileNode) nodeAtMouse).getExternalFileName();
			if (!ignoreOldConverterID)
				curConverterID = ((FileNode) nodeAtMouse).getConverterID();

			if (nodeAtMouse instanceof OutputFileNode)	
				newConverterID = selectOutputConverterID(((OutputFileNode) nodeAtMouse).getCommonIncomingTypes(), curConverterID, curFileName.getName(), showAlways);
			else if (nodeAtMouse instanceof InputFileNode) {
				newConverterID = selectInputConverterID(((InputFileNode) nodeAtMouse).getCommonOutgoingTypes(), curConverterID, curFileName.getName(), showAlways);
			}

			if ((newConverterID != -1) && (newConverterID != curConverterID)) {
				graph.addNewAction(new SetConverterIDHistoryObject((FileNode) nodeAtMouse,newConverterID, graph));
				graph.performLastAction();
			}
		}
	}

	/**
	 * This method changes the file of a FileNode.
	 */
	protected void changeFileNodeFileName() {
		if ((nodeAtMouse != null) && (nodeAtMouse instanceof FileNode)) {
			// Select new file name
			File newFileName = null;
			TypedFile curFileName = ((FileNode) nodeAtMouse).getExternalFileName();

			if (nodeAtMouse instanceof OutputFileNode)				
				newFileName = DialogManager.showSaveDialog(curFileName.getParent(), curFileName.getName(), xmlReader.getOutputConverterOutputSuffixByID(((FileNode)nodeAtMouse).getReferenceID()), me, false);

			else if (nodeAtMouse instanceof InputFileNode) {
				newFileName = DialogManager.showOpenDialog(curFileName.getParent(), curFileName.getName(), xmlReader.getInputConverterInputSuffixByID(((FileNode)nodeAtMouse).getReferenceID()), me);
			}

			// If a file was selected, check type
			if (newFileName != null) {
				graph.addNewAction(new SetExternalFileNameHistoryObject((FileNode) nodeAtMouse, newFileName.getPath()));
				graph.performLastAction();

				// If the extension changed, let the user select the new type
				if (!(getExtension(newFileName).equals(getExtension(curFileName)))) {
					changeFileNodeFileType(true, true);
				}
			}
		}

	}

	private String getExtension(File curFileName) {
		String result = "";
		int dotPos = curFileName.getName().lastIndexOf(".");
		if (dotPos > 0) {
			result = curFileName.getName().substring(dotPos, curFileName.getName().length());
		} 
		return result;
	}

	/**
	 * This method creates and displays the job frame after a click on the menu 
	 * item "Edit".
	 */
	protected void editJob(JobNode nodeToEdit) {
		if ((nodeToEdit != null) && (!nodeToEdit.hasEditDialog())) {
			changeMarkedNode(nodeToEdit);
			EditJobFrame editJobFrame = new EditJobFrame(
					(JobNode) nodeToEdit, this);
			editJobFrame.setLocation(this.getX() + this.getWidth() + 20,
					this.getY());
			editJobFrame.setVisible(true);
			nodeToEdit.setEditDialog(editJobFrame);
		}
	}


	/**
	 * This method deletes the selected edge.
	 * This method is called after a click on the menu item "Delete Connection".
	 */
	protected void deleteEdge() {
		if (nodeAtMouse instanceof PointNode) {
			Edge pointNodesEdge = ((PointNode) nodeAtMouse).getEdge();			
			graph.addNewAction(new RemoveConnectionHistoryObject(pointNodesEdge,
					graph));
			graph.performLastAction();
			repaint();
			pointNodesEdge = null;
		}
	}


	/**
	 * This method deletes a node with all its edges.
	 * This method is called after a click on the menu item "Delete".
	 */
	public void deleteNode() {
		// Delete node only if one is selected and, in case it is
		// a point node, it is not the only point node of the edge
		if ((nodeAtMouse != null) && ((!(nodeAtMouse instanceof PointNode) || (!((PointNode) nodeAtMouse).isOnlyPointNodeOfEdge())))){
			graph.addNewAction(new RemoveNodeHistoryObject(nodeAtMouse,
					graph));
			graph.performLastAction();
			
			if (nodeAtMouse == markedNode)
				me.changeMarkedNode(null);
			nodeAtMouse = null;

			repaint();
		}
	}

	/**
	 * This method changes the name of a node.
	 * This method is called after a click on the menu item "Change Name".
	 */
	protected void changeNodeName() {
		if ((nodeAtMouse != null) && (!(nodeAtMouse instanceof PointNode))) {
			String newName = (String) JOptionPane.showInputDialog(this,
					"Enter new Name:", "Edit Job Name",
					JOptionPane.PLAIN_MESSAGE, null, null,
					((GraphNode)nodeAtMouse).getName());
			if ((newName != null) && (newName.length() > 0)) {
				graph.addNewAction(new ChangeNodeNameHistoryObject(
						(GraphNode)nodeAtMouse, newName));
				graph.performLastAction();

				repaint();
			}
		}
	}

	/**
	 * This method is responsible for the creation of a new job. The jobs type
	 * is determined by the passed id.
	 * This method is called after a click on one of the job nodes in the main 
	 * window frame menu. The output types of the current job are read out from 
	 * the XML and displayed in the "Select Output" dialog window where the user 
	 * can determine the output type of the job.
	 */
	public void newJob(int pId) {
		/*
		Vector<String> types = xmlReader.getProgramOutputTypesByID(pId);
		String targetType = DialogManager.showSelectionDialog(types,
				types.get(0), "Select Output", "Select Output Type for Job "
				+ xmlReader.getProgrammNameByID(pId), this);
		 */
		newAbstractNode(new JobNode(0, 0, xmlReader, grid, pId, graph));
	}

	/**
	 * This method is called to change the state of the panel in order to
	 * place the new created node which is passed to this method.
	 */
	public void newAbstractNode(GraphNode newAbstractNode) {
		if (curStatus == status.FREE) {
			curStatus = status.NEW;

			// Save new node
			changeMarkedNode(newAbstractNode);

			// statusjLabel.setText("Node created, please klick to position.");
			repaint();
		}
	}

	/**
	 * This method connects two nodes with each other. Before the connection is created, 
	 * the method checks if the connection between these two nodes is valid.
	 */
	private void connectNode() {
		if ((markedNode != null) && (nodeAtMouse != null)) {
			GraphNode from = (GraphNode)markedNode;
			GraphNode to = (GraphNode)nodeAtMouse;

			if (isConnectionIsValid(from, to)) {
				Edge edge = new Edge(from, to, to.getNextFreeOrderKey());
				if (edge.getPossibleTypes().size() > 0) {
					graph.addNewAction(new ConnectNodeHistoryObject(edge, graph));
					graph.performLastAction();

					markedNode.setMarked(false);

					repaint();
				}
				else {
					String message = "Connection between " + from.getName() + " and " + to.getName() + " is not valid.\n";

					message += "\n";
					message += from.getName() + " has the following types as Output: \n";
					for(String curType : edge.getFrom().getOutputFileTypes()){
						message+= curType + "\n";
					}
					message += "\n";

					message += to.getName() + " expects the following types as Input: \n";

					for(String curType : edge.getTo().getInputFileTypes()){
						message+= curType + "\n";
					}

					JOptionPane.showMessageDialog(this, message, "Error", JOptionPane.ERROR_MESSAGE);

				}
			}

		}
	}


	/**
	 * This method compares the elements of the two delivered string vectors 
	 * and returns the common ones in a new vector.
	 */
	private Vector<String> getCommonElements(Vector<String> outputTypes,
			Vector<String> inputTypes) {
		Vector<String> result = new Vector<String>();

		for (String curOutputType : outputTypes) {
			for (String curInputType : inputTypes) {
				if (curInputType.equals(curOutputType))
					result.add(curInputType);
			}
		}
		return result;
	}

	/**
	 * Sets if the workflow should be scaled on resize.
	 */
	public void setScaleOnResize(boolean state) {
		grid.scaleOnResize(state);
	}

	/**
	 * Sets if the grid should be visible.
	 */
	public void setShowGrid(boolean state) {
		grid.setVisible(state);
		repaint();
	}

	/**
	 * This method saves the workflow under the passed name.
	 */
	public void saveGraph(String pFileName) {
		try {
			BufferedWriter bw = new BufferedWriter(new FileWriter(new File(pFileName)));
			bw.write(graph.getXMLRepresentationOfGraph());
			bw.close();
			graph.setSaved(true);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Saves a snaphot of the current graph in the users
	 * home folder.
	 */
	public void saveSnapshot() {
		SimpleDateFormat dateTimeFormat = new SimpleDateFormat("yyyy.MM.dd_HH.mm.ss");
		Date now = new Date();

		String usersHomeFolder = System.getProperty("user.home");
		String targetFolder = usersHomeFolder + File.separator + "ProFiDo";

		// Create folder if not existent
		File targetFolderFile = new File(targetFolder);
		if (!targetFolderFile.exists())
			targetFolderFile.mkdir();

		String fileName = "_snapshot_" + dateTimeFormat.format(now) + ".xml";

		if (graph.getName().equals(""))
			fileName = "untitled" + fileName;
		else
			fileName = graph.getName() + fileName;

		saveGraph(targetFolder + '/' + fileName);
	}

	/**
	 * This method loads the graph from the location specified by the
	 * passed filename.
	 * @param checkFilesOnLoad 
	 */
	public void loadGraph(String pFileName, boolean checkFilesOnLoad) throws Exception {
		try {
			String curWorkingDir = "";
			if (graph != null)
				curWorkingDir = graph.getOutputFolder();
			GraphLoader loader = new GraphLoader(pFileName, xmlReader, me);

			graph = loader.loadGraph(checkFilesOnLoad, curWorkingDir);
			graph.realignToGrid(grid);

			nodeAtMouse = null;
			markedNode = null;
			curStatus = status.FREE;

			Rectangle boundingBox = graph.getBoundingBox(grid.getCellWidth(), grid.getCellHeight());
			int topOffset = (int) SwingUtilities.convertPoint(this, new Point(0,0), mainFrame).getY();

			grid.dontScaleOnNextResize();
			mainFrame.setSize(2 * boundingBox.x + boundingBox.width + 20, 2 * boundingBox.y + boundingBox.height + topOffset);				
			mainFrame.updateTitle(graph.getName());
			repaint();

		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	public void setBatchName(String newName) {
		graph.setName(newName);
		mainFrame.updateTitle(graph.getName());
	}

	public String getBatchName() {
		return graph.getName();
	}

	public String getExpFolder(){
		return graph.getExpFolder();
	}

	public void setExpFolder(String outputFolder) {
		graph.setExpFolder(outputFolder);
	}

	/**
	 * This method checks if the workflow is consistent to export the script
	 * and shows a message box with further informations about any problem.
	 * 
	 * @return true if the workflow is consistent else false.
	 */
	public boolean check(){
		String allEdgesConnectedMessage = graph.checkBuild();
		String cycleMessage = graph.checkForCycle();

		String message = allEdgesConnectedMessage + cycleMessage;

		if (message.equals(""))
			return true;
		else {
			JOptionPane.showMessageDialog(this, "The current workflow cannont be exported. Please solve the following problems:\n" + message, "Incorrect Workflow", JOptionPane.ERROR_MESSAGE);
			return false;			
		}
	}

	/**
	 * This method exports the workflow to the passed location.
	 */
	public void exportWorkflowScript(String targetFile) {
		try {
			graph.setExpFolder(new TypedFile(targetFile).getParent());
			graph.createScripts(targetFile);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public String getOutputFolder() {
		return graph.getOutputFolder();
	}

	public void setOutputFolder(String outputFolder) {
		graph.setOutputFolder(outputFolder);
	}

	/**
	 * Creates a new workflow according to the passed values.
	 */
	public void newWorkflow(String name, String workingDir) {
		String curWorkingDir = workingDir;

		if ((curWorkingDir.equals("")) && (graph != null))
			curWorkingDir = graph.getOutputFolder();
		this.graph = new Graph(name, curWorkingDir, "", xmlReader, this, showHistoryFrame);
		mainFrame.updateTitle(graph.getName());
		me.repaint();
	}

	/**
	 * Undoes the last change done to the workflow.
	 */
	public void undo() {
		this.graph.undo();
	}

	/**
	 * Redoes the last change done to the workflow.
	 */
	public void redo() {
		this.graph.redo();
	}

	public Graph getGraph() {
		return graph;
	}

	public boolean isGraphSaved() {
		return graph.isSaved();
	}

	public void clearHistory() {
		graph.clearHistory();
	}

	public Grid getGrid() {
		return grid;
	}

	/**
	 * Shows the current history in a ShowHistoryFrame.
	 */
	public void showHistory() {
		if (!showHistoryFrame.isVisible()) {
			showHistoryFrame.setLocation(this.getX() + this.getWidth() + 20, this.getY());
			showHistoryFrame.setVisible(true);	
		}

	}

	public boolean getScaleOnResize() {
		return grid.getScaleOnResize();
	}

	public JMenuItem getDeleteJobMenuItem() {
		return deleteJobNodeMenuItem;
	}

	public AbstractButton getDeleteEdgeMenuItem() {
		return deleteEdgeMenuItem;
	}

	public void saveScreenshot(String fileName) {
		BufferedImage image =
				new BufferedImage(this.getWidth(), this.getHeight(), BufferedImage.TYPE_INT_RGB);
		Graphics2D g2 = (Graphics2D)image.createGraphics();
		this.drawAll(g2);
		File outputfile = new File(fileName);
		try {
			ImageIO.write(image, "png", outputfile);
		} catch (IOException e) {
			e.printStackTrace();
		}		
	}

	public Vector<Point> getJobNodePositions() {
		return graph.getJobNodePositions();
	}

	public Vector<Point> getEdgePositions() {
		return graph.getEdgePositions();
	}


	/**
	 * Sets which informations should be printed on the edges and nodes of the graph.
	 */

	public void setPrintInformations(boolean displayEdgeInformations, boolean displayNodeInformations, boolean alwaysDisplayOrderKey, boolean displayAllEdgeInformations, boolean displayAllNodeInformations) {
		Graph.DISPLAY_EDGE_INFORMATIONS = displayEdgeInformations;
		Graph.DISPLAY_NODE_INFORMATIONS = displayNodeInformations;
		Graph.ALWAYS_DISPLAY_ORDER_KEYS = alwaysDisplayOrderKey;
		Graph.DISPLAY_ALL_EDGE_INFORMATIONS = displayAllEdgeInformations;
		Graph.DISPLAY_ALL_NODE_INFORMATIONS = displayAllNodeInformations;
		repaint();

	}

	public ShowHistoryFrame getShowHistoryFrame() {
		return showHistoryFrame;
	}


} 

