/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package gui.helper;

import helper.SmallHelpers;

import java.io.File;
import java.util.Locale;
import java.util.Vector;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

import javax.swing.filechooser.FileFilter;

/**
 * This class defines the properties of the dialog managers which provides methods
 * to create dialogs and message boxes.
 */
public class DialogManager {

	/**
	 * This method creates a JFileChooser object and displays the "Save" dialog where 
	 * the user can determine the name and the target of the file which has to be saved.
	 * 
	 * @param rootDir root directory
	 * @param defaultName default name
	 * @param extension ending
	 * @param parent java.awt.Component
	 * @return the file to save
	 */
	public static File showSaveDialog(String rootDir, String defaultName, final String extension, java.awt.Component parent, boolean forceExtension) {
		JFileChooser fileChooser = new JFileChooser(rootDir);
		removeAllFileFilters(fileChooser);
		if (extension.equals(".trace")) {
			fileChooser.addChoosableFileFilter(new FileFilter(){
				public boolean accept(File f){
					return f.getName().endsWith(".trace") || f.isDirectory();
				}

				public String getDescription(){
					return "Trace-Files";
				}
			});
		} else if (extension.equals(".*")) {
			/*			fileChooser.addChoosableFileFilter(new FileFilter(){
				public boolean accept(File f){
					return true;
				}

				public String getDescription(){
					return "All Files";
				}
			});*/
		} else {
			fileChooser.addChoosableFileFilter(new FileFilter(){
				public boolean accept(File f){
					return f.getName().endsWith(extension) || f.isDirectory();
				}

				public String getDescription(){
					return extension.substring(1,extension.length()).toUpperCase() + "-Files";
				}
			});
		} 
		File result = null;
		while (result == null) {
			// Show until a valid file is selected
			if (fileChooser.showSaveDialog(parent) == JFileChooser.APPROVE_OPTION) {
				File selectedFile = null;
				if (forceExtension)
					selectedFile = addExtensionToFile(fileChooser.getSelectedFile(), extension);
				else
					selectedFile = fileChooser.getSelectedFile();	
				if (SmallHelpers.fileWriteable(selectedFile)) {
					result = selectedFile;
				} else {
					int dialogResult = JOptionPane.showConfirmDialog(parent, "You do not have the rights to access the selected file or the directory it is located in does not exist.\nWould you like to select another file?", 
							"Warning - " + selectedFile.getPath(), JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.WARNING_MESSAGE,null);
					if (dialogResult == JOptionPane.NO_OPTION)
						result = selectedFile;
					else if (dialogResult == JOptionPane.CANCEL_OPTION)
						break;
				}
			} else {
				// If user cancels dialog stop further processing
				break;
			}			
		}

		return result;
	}

	private static void removeAllFileFilters(JFileChooser fileChooser) {
		FileFilter filtes[] = fileChooser.getChoosableFileFilters();
		for (FileFilter curFilter : filtes) {
			fileChooser.removeChoosableFileFilter(curFilter);
		}
	}

	/**
	 * Adds the passed extension to the passed file if the extension is not
	 * already present.
	 */
	private static File addExtensionToFile(File selectedFile, String extension) {
		if ((extension.equals(".*")) || (selectedFile.getPath().endsWith(extension)))
			return selectedFile;				
		else
			return new File(selectedFile.getPath() + extension);	
	}

	/**
	 * This method creates a JFileChooser object and displays the "Open" dialog where the user can 
	 * select a file to open.
	 * 
	 * @param rootDir root directory
	 * @param defaultName default name
	 * @param extension ending
	 * @param parent java.awt.Component
	 * @return selected file to open
	 */
	public static File showOpenDialog(String rootDir, String defaultName, final String extension, java.awt.Component parent) {
		// Create dialog
		JFileChooser fileChooser = new JFileChooser(rootDir);
		fileChooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
		removeAllFileFilters(fileChooser);
		fileChooser.setLocale(Locale.ENGLISH);
		fileChooser.updateUI();
		if (extension.equals(".trace")) {
			fileChooser.addChoosableFileFilter(new FileFilter(){
				public boolean accept(File f){
					return f.getName().endsWith(".trace") || f.isDirectory();
				}

				public String getDescription(){
					return "Trace-Files";
				}
			});
		} else if (extension.equals(".*")) {
			/*
				fileChooser.addChoosableFileFilter(new FileFilter(){
				public boolean accept(File f){
					return true;
				}

				public String getDescription(){
					return "All Files";
				}
			});*/
		} else {
			fileChooser.addChoosableFileFilter(new FileFilter(){
				public boolean accept(File f){
					return f.getName().endsWith(extension) || f.isDirectory();
				}

				public String getDescription(){
					return extension.substring(1,extension.length()).toUpperCase() + "-Files";
				}
			});
		} 
		File result = null;
		while (result == null) {
			// Show until a valid file is selected
			if (fileChooser.showOpenDialog(parent) == JFileChooser.APPROVE_OPTION) {
				File selectedFile = fileChooser.getSelectedFile();	
				if (SmallHelpers.fileReadable(selectedFile)) {
					result = selectedFile;
				} else {
					int dialogResult = JOptionPane.showConfirmDialog(parent, "The selected file does not exist or you don't have the rights to access it.\nWould you like to select another file?", 
							"Warning - " + selectedFile.getPath(), JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.WARNING_MESSAGE,null);
					if (dialogResult == JOptionPane.NO_OPTION)
						result = selectedFile;
					else if (dialogResult == JOptionPane.CANCEL_OPTION)
						break;
				}
			} else {
				// If user cancels dialog stop further processing
				break;
			}			
		}

		return result;
	}

	/**
	 * This methods show a folder chose dialog in order to select folders.
	 * 
	 * @param rootDir root directory
	 * @param parent java.awt.Component
	 * @return selected folder
	 */
	public static File showFolderSeelct(String rootDir, java.awt.Component parent) {
		// Create dialog

		JFileChooser fileChooser = new JFileChooser(rootDir);
		fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);

		fileChooser.setLocale(Locale.ENGLISH);
		fileChooser.updateUI();

		File result = null;

		File selectedFile = fileChooser.getSelectedFile();	
		result = selectedFile;

		return result;
	}



	/**
	 * This method creates a JFileChooser object and displays the "Open" dialog where the user can select 
	 * a directory which will be set the default folder.
	 * 
	 * @param rootDir root directory
	 * @param parent java.awt.Component
	 * @return selected directory
	 */

	public static File showSelectFolderDialog(String rootDir, java.awt.Component parent) {
		JFileChooser fileChooser = new JFileChooser(rootDir);
		fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		int result = fileChooser.showOpenDialog(parent);

		if(result == JFileChooser.APPROVE_OPTION){
			if (fileChooser.getSelectedFile().exists())
				return fileChooser.getSelectedFile();						
		}

		return null;
	}

	/**
	 * This method creates a JFileChooser object and displays the "Open" dialog where the 
	 * user can select a directory which will be set the export folder. The directory has 
	 * to be selected to use the necessary information in the Result directory to display
	 *  the result of the plotter.
	 * 
	 * @param rootDir root directory
	 * @param parent java.awt.Component
	 * @return selected directory
	 */
	public static File showExportFolderDialog(String rootDir, java.awt.Component parent) {
		JFileChooser fileChooser = new JFileChooser(rootDir);
		fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		int result = fileChooser.showOpenDialog(parent);

		if(result == JFileChooser.APPROVE_OPTION){
			if (fileChooser.getSelectedFile().exists())
				return fileChooser.getSelectedFile();						
		}

		return null;
	}

	/**
	 * This method displays a selection dialog with the delivered options and returns the selected option as a string.
	 * This method is called by :	- newJob which waits for the output type of the job to create the job node
	 * 								- selectInputFileTypeAndGetConverterIndex which waits for the type of an input file to determine the converter id.
	 * 								- selectOutputFileTypeAndGetConverterIndex which waits for the type of an output file to determine the converter id. 
	 * 
	 * @param options option
	 * @param defaultOption default option
	 * @param title title of the window
	 * @param text text
	 * @param parent java.awt.Component
	 * @return selected option
	 * 
	 */
	public static String showSelectionDialog(Vector<String> options, String defaultOption, String title, String text, java.awt.Component parent) {
		if (options.size() == 0)
			return null;
		else {
			Object[] possibilities = options.toArray();
			return (String)JOptionPane.showInputDialog(parent, text, title, JOptionPane.QUESTION_MESSAGE, null, possibilities, defaultOption);
		}


	}
}
