/*
* 
* ProFiDo - The Processes Fitting Toolkit Dortmund
* Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License along
* with this program; if not, write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*
*/

package gui.helper;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;

/**
 * This class defines the properties of the grid of the workspace.
 *
 * A grid divides the workspace into cells which have a width and height.
 * The workspace has a determined number of cells for the x-axis and y-axis.
 *
 */

public class Grid {
	private int cellCountX, cellCountY;
	private int width, height;
	
	private int cellWidth, cellHeight;
	private int borderX;
	private int borderY;
	private int spaceWidth;
	private int spaceHeight;
	
	/**
	 * Determines whether the graph of the workflow will be resized when the 
	 * window is resized or not.
	 */
	private boolean scaleOnResize;
	
	/**
	 * Determines whether the grid is visible or not.
	 */
	private boolean visible;
	
	/**
	 * Indicates if no scaling should happen upon the next resize.
	 */
	private boolean dontScaleOnNextResize = false;
	
	
	public Grid(int cellCountX, int cellCountY, int cellWidth, int cellHeight, boolean scaleOnResize) {
		super();
		this.scaleOnResize = scaleOnResize;
		this.cellCountX = cellCountX;
		this.cellCountY = cellCountY;
		this.cellWidth = cellWidth;
		this.cellHeight = cellHeight;
	}

	public int getWidth() {
		return width;
	}

	public int getHeight() {
		return height;
	}
	
	/**
	 * This method sets the size of the grid and calculates the resulting size of the cells.
	 * 
	 * @param pWidth width of the grid
	 * @param pHeight height of the grid
	 */
	
	public void setSize(int pWidth, int pHeight) {
		this.width = pWidth;
		this.height = pHeight;
		if ((scaleOnResize) && (!dontScaleOnNextResize)) {
			this.cellWidth = width / cellCountX;
			this.cellHeight = height / cellCountY;	

		} else {
			this.cellCountX = width / cellWidth;
			this.cellCountY = height / cellHeight;
		}
		this.borderX = cellWidth / 8;
		this.borderY = cellHeight / 8;
		this.spaceWidth = cellWidth - (2 * borderX);
		this.spaceHeight = cellHeight - (2 * borderY);
		dontScaleOnNextResize = false;
	}
	
	
	/**
	 * This method draws the grid
	 */
	public void draw(Graphics g) {
		if (this.visible) {
			g.setColor(Color.LIGHT_GRAY);
			
			for (int curXPos = 0; curXPos < width; curXPos = curXPos + cellWidth) {
				g.drawLine(curXPos, 0, curXPos, height);
			}
			
			for (int curYPos = 0; curYPos < height; curYPos = curYPos + cellHeight) {
				g.drawLine(0, curYPos, width, curYPos);
			}				
		}
	}
	
	/**
	* This method returns the cell coordinates of the cell closest to 
	* the passed point.
	* 
	* @param pX x-coordinate of the point 
	* @param pY y-coordinate of the point
	* @return starting point of the cell
	*/

	public Point getNearestCell(int pX, int pY) {
		int cellX = (int) Math.ceil(pX / cellWidth);
		int cellY = (int) Math.ceil(pY / cellHeight);
		
		return new Point(cellX, cellY);
	}
	
	/**
	 *This method returns the coordinates of the center of the cell closest to 
	 * the passed point. 
	 * @param pX x-coordinate of the point
	 * @param pY y-coordinate of the point
	 * @return the center of a cell
	 */
	public Point getCellCenter(int pX, int pY) {
		int xPos = (pX * cellWidth) + (cellWidth / 2);
		int yPos = (pY * cellHeight) + (cellHeight / 2); 
		
		return new Point(xPos, yPos);
	}

	public int getCellWidth() {
		return this.cellWidth;
		
	}

	public int getCellHeight() {
		return this.cellHeight;
	}

	public Rectangle getDrawingSpace(int x, int y) {
		return new Rectangle(x - (spaceWidth / 2), y - (spaceHeight / 2), spaceWidth, spaceHeight);
	}

	public void scaleOnResize(boolean state) {
		scaleOnResize = state;
	}

	public void setVisible(boolean state) {
		visible = state;
		
	}
	public boolean getVisible() {
		return visible;
		
	}
	
	public boolean getScaleOnResize() {
		return scaleOnResize;
	}

	/**
	 * After calling this method the grid won't scale on
	 * the next resize no matter what the value of scaleOnResize is.
	 */
	public void dontScaleOnNextResize() {
		dontScaleOnNextResize = true;
	}
}
