/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

import java.awt.Component;
import java.io.File;
import java.net.URL;
import java.util.Locale;
import java.util.Vector;

import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.UIManager;

import core.TypedFile;
import core.storage.BasicXMLReader;
import core.storage.ConfigurationXMLReader;
import gui.AboutFrame;
import gui.MainWindowFrame;
import helper.LanguageManager;
import helper.SmallHelpers;
import helper.testmanager.TestManager;
import helper.testmanager.WorkflowPanelTestManager;

/**
 * This class defines the static launcher method used to process any given 
 * command line parameters and start the applications actual GUI. 
 */
public class main {	
	/**
	 * Applications start point. 
	 * 
	 * @param args Command line parameters passed to the application.
	 */
	public static void main(String[] args) {	
		// Find the path where the application is located
		URL pathToJar = main.class.getResource("profido.jar");
		String applicationRootDir = "";
		if (pathToJar != null) 
			applicationRootDir = pathToJar.getPath().substring(0, pathToJar.getPath().lastIndexOf("/") + 1);		

		Vector<String> parameters = SmallHelpers.arrayToVector(args);
		parseParametersAndStartApplication(applicationRootDir, parameters);		
	}

	/**
	 * This method parses any given command line parameters and passes the 
	 * values to the method which starts the applications GUI. 
	 */
	private static void parseParametersAndStartApplication(String applicationRootDir, Vector<String> parameters) {
		// Initialize values
		String inputFileName = "";
		String exportFilenName = "";
		String configFileName = applicationRootDir + "software.xml";
		int testInteractionCount = 10;
		String testMode = "";
		int testDelay = 5;
		long seed = System.currentTimeMillis();
		boolean checkFilesOnLoad = true;
		boolean helpNeeded = false;

		// Parse parameters
		int curPos = 0;
		while (curPos < parameters.size()) {
			String curParameter = parameters.get(curPos);

			// Search for config key
			if (curParameter.equals("-c")) {
				parameters.remove(curPos);
				if (curPos < parameters.size()) {
					configFileName = parameters.get(curPos);
					parameters.remove(curPos);
				}
			} else if (curParameter.equals("-t")) {
				parameters.remove(curPos);
				// Search for test key				
				if (curPos - 1 < parameters.size()) {
					testMode = parameters.get(curPos);
					parameters.remove(curPos);
					testInteractionCount = Integer.parseInt(parameters.get(curPos));
					parameters.remove(curPos);
				}
			} else if (curParameter.equals("-d")) {
				parameters.remove(curPos);
				// Search for test key				
				if (curPos < parameters.size()) {
					testDelay = Integer.parseInt(parameters.get(curPos));
					parameters.remove(curPos);
				}
			} else if (curParameter.equals("-s")) {
				parameters.remove(curPos);
				// Search for test key				
				if (curPos < parameters.size()) {
					seed = Long.parseLong(parameters.get(curPos));
					parameters.remove(curPos);
				}
			}
			// Search for no check key	
			else if (curParameter.equals("-n")) {
				parameters.remove(curPos);			
				checkFilesOnLoad = false;
			} else if (curParameter.equals("--help") || curParameter.equals("-h")) {
				parameters.remove(curPos);
				helpNeeded = true;
			} else
				curPos++;
		}

		// Parse input and output file and search for unknown parameters
		String unknownParameters = "";
		while (parameters.size() > 0) {
			String curParameter = parameters.remove(0);
			if (curParameter.startsWith("-")) {
				unknownParameters = unknownParameters + curParameter + ", ";
				helpNeeded = true;
			} else {
				if (inputFileName.equals("")) {
					inputFileName = curParameter;
				} else if (exportFilenName.equals("")) {
					exportFilenName = curParameter;
				}
			}
		}

		if (helpNeeded) {
			if (!unknownParameters.equals(""))
				System.out.println("The following parameters are unknown: " + unknownParameters.substring(0,unknownParameters.length() - 2));
			outputParameterInformations();	
		} else {
			startApplication(configFileName, inputFileName, exportFilenName, testMode, testInteractionCount, testDelay, checkFilesOnLoad, seed);					
		}
	}

	private static void outputParameterInformations() {
		String message = "ProFiDo " + AboutFrame.PROFIDO_VERSION + "\n";
		message = message + "Usage: profido.sh [OPTIONS] [INPUT-FILE] [EXPORT-FILE]\n";
		message = message + "\t-c <file>\t\t\tSelects a custom configuration file.\n";
		message = message + "\t\t\t\t\t(Default: software.xml)\n";
		message = message + "\t-n\t\t\t\tDisables all file checks when loading a workflow.\n";
		message = message + "\t-h, --help\t\t\tDisplays this message.\n";
		message = message + "\t-t <mode> <interactions>\tActivates the TestManager in the selected mode and performs the specified amount of user interactions.\n";
		message = message + "\t\t\t\t\t(Default: No mode selected, interactions = 10)\n";
		message = message + "\t-d <delay>\t\t\tSpecifies a custom time in milliseconds between two user interactions within the TestManager.\n";
		message = message + "\t\t\t\t\t(Default: 5)\n";	
		message = message + "\t-s <seed>\t\t\tSpecifies a custom seed for the TestManager.\n";
		message = message + "\t\t\t\t\t(Default: Time since midnight, January 1, 1970 UTC in milliseconds.)\n";		
		System.out.println(message); 

	}

	/**
	 * This method starts the applications GUI according to the passed 
	 * parameter values. 
	 */
	public static void startApplication(String configurationFileName, String inputFileName, String exportFilenName, String testMode, int testInteractionCount, int testDelay, boolean checkFilesOnLoad, long seed) {
		try {
			ConfigurationXMLReader xmlReader = new ConfigurationXMLReader(configurationFileName);

			// Force button and dialog language
			/*
			Locale l = new Locale("en", "US");
			Locale.setDefault(l);
			JComponent.setDefaultLocale(l);
			UIManager.setLookAndFeel(UIManager.getLookAndFeel());
			*/
			LanguageManager.setSystemButtonAndCaptionTexts();
			
			// Create main window
			MainWindowFrame window = new MainWindowFrame(xmlReader);
			window.setLocation(50, 50);
			window.setVisible(true);

			// Process parameters
			if (!inputFileName.equals(""))
				window.openGraph(inputFileName, checkFilesOnLoad);

			// Perform tests 
			if ((testInteractionCount > 0) && (!testMode.equals(""))) {
				Thread.sleep(1000); // Wait until the gui is loaded compeltely
				TestManager testManager = null;
				if (testMode.equals("workflowPanel"))
					testManager = new WorkflowPanelTestManager(window, window.getWorkflowPanel(), seed);

				if (testManager != null) {
					File tmpFile = new TypedFile(exportFilenName);
					String message = "Running TestManager...\n";
					message = message + "\tExport File: \t\t" + tmpFile.getName()+ "\n";
					message = message + "\tMode: \t\t\t" + testMode + "\n";
					message = message + "\tUser interactions: \t" + testInteractionCount + "\n";
					message = message + "\tDelay: \t\t\t" + testDelay + " ms\n";
					message = message + "\tSeed: \t\t\t" + seed + "\n";
					message = message + "To rerun this test call: profido.sh -n -c " + configurationFileName + " -t " + testMode + " " + testInteractionCount + " -d " + testDelay + " -s " + seed + " \"" + inputFileName + "\" \"" + exportFilenName + "\"\n";
					System.out.println(message);
					testManager.performTests(testInteractionCount, testDelay);
				}
			}

			if (!exportFilenName.equals("")) {
				window.exportWorkflowScript(exportFilenName);
				window.exitApplication(false);
			}


		} catch (Exception e) {
			e.printStackTrace();
		}		
	}
}
